/*
 * IBM Confidential
 * OCO Source Materials
 * Component IBM Rational ROSE_RT BUILD : ROSE_RT
 * (c) Copyright IBM Corp. [1998], [2004] .All Rights Reserved.
 * 
 * The source code for this program is not published or
 * or otherwise divested of its trade secrets, irrespective
 * of what has been deposited with the U.S. Copyright Office.
 * 
 * 
 * Copyright (c) 1991-2003 Rational Software Corporation. All rights
 * reserved.
 * 
 * Rational, the Rational logo, ObjecTime (and logo) are registered
 * trademarks of Rational Software Corporation.
 * 
 * The placing of the copyright notices on Rational source code is not
 * intended to mean that they are published works, but are included in the
 * event of inadvertent publication. The source code of Rational contains
 * trade secrets and other valuable confidential information of Rational
 * Software Corporation. Rational Software Corporation and its licensors
 * retain ownership to Rational Products in both source and object code
 * format, and related documentation and all rights therein including
 * copyrights, patents and trade secrets.
 * 
 * 
 * U.S. Government Rights
 * 
 * Use, duplication, or disclosure by the U.S. Government is subject to
 * restrictions set forth in the applicable Rational License Agreement
 * and in DFARS 227.7202-1(a) and 227.7202-3(a) (1995), DFARS
 * 252.227-7013(c)(1)(ii)(Oct 1988), FAR 12.212(a) 1995, FAR 52.227-19,
 * or FAR 52.227-14, as applicable.
 * 
 * Unless otherwise agreed to in writing, use of Rational products is
 * governed by the Rational Product License Agreement provided with
 * the product.
 * 
 */


#ifndef __RTPubl_Capsule_h__
#define __RTPubl_Capsule_h__ included

/*
** This abstract data type represents the common structure and
** functionality of a generated Capsule with respect to the RTS.
*/

/* inclusions */
#include <RTPubl/CompDesc.h>
#include <RTPubl/TimerId.h>

/* forward declarations */
struct _RTCapsuleRole;
struct _RTCapsule_class;
struct _RTController;
struct _RTEndPort;
struct _RTMessage;

/* structure declaration */
typedef struct _RTCapsule
{
    /* public fields */
    const struct _RTMessage       * msg;
    struct _RTController          * rts;

    /* private fields --- direct access to these fields
       is not recommended in user code and may not be
       supported in future releases */

    
    struct _RTCapsuleRole         * homeRole;
    int                             homeIndex;
    const struct _RTCapsule_class * class;
    RTStateIndex                  * history;

#if OBSERVABLE
    struct _RTProbe               * probe;
#endif
	RTCapsuleIndex                  Id;
    RTPortIndex                     PortOffset;

    RTStateIndex                    currentState;
} RTCapsule;

/* public operations */

/* return the name of the current state that the capsule is in */
#if RTS_NAMES
const char * RTCapsule_getCurrentStateString( const RTCapsule * );
#endif

/* equivalent to accessing the msg field of the structure */
RTS_INLINE const struct _RTMessage * RTCapsule_getMsg( const RTCapsule * );

/* return the RTController object that interfaces to the thread that
   this capsule is running upon */
RTS_INLINE struct _RTController    * RTCapsule_context( const RTCapsule * );

/*  return the replication index of this capsule index in its home role */
RTS_INLINE int          RTCapsule_getIndex( const RTCapsule * );

/*  return the name of the capsule role in which this capsule is running */
#if RTS_NAMES
RTS_INLINE const char * RTCapsule_getName( const RTCapsule * );
#endif

/*  return the name of the capsule class of which this capsule is */
#if RTS_NAMES
RTS_INLINE const char * RTCapsule_getTypeName( const RTCapsule * );
#endif

/* ---------------------------------------------------------------
   private operations and data structures

   access to these functions and structures may not be
   supported in future releases and are thus not recommended for
   use in user code
*/

/* allocate, deallocate, ctor, dtor */
RTCapsule * new_RTCapsule( struct _RTCapsuleRole *, int );
void        delete_RTCapsule( RTCapsule * );
void        RTCapsule_construct( RTCapsule *, struct _RTCapsuleRole *, int );

#if OBSERVABLE
/* A call to chainBegin appears at the beginning of every generated
** chain function. The first argument, origin, identifies the start
** point of the chain; numbers from 1 to getNumStates(), inclusive,
** refer to states, larger numbers refer to choice-points. The second
** argument, trans, is the transition string. Note that the
** first argument is implied by the second, but that would require
** generation of a mapping table which, on some architectures, could
** be larger than the code which passes the argument.
*/

void RTCapsule_chainBegin( RTCapsule * this, int origin, const char * trans );

/* A call to transitionBegin is generated immediately before and
** a call to transitionEnd is generated immediately after each
** call to a user transition function. If the transition is not
** generated (because it is empty) the calls to transitionBegin
** and transitionEnd must appear where they would be called if it
** were not empty.
*/

void RTCapsule_transitionBegin( RTCapsule * this );
void RTCapsule_transitionEnd  ( RTCapsule * this );

#else

#define RTCapsule_chainBegin(c,o,t)  ( (void)0 )
#define RTCapsule_transitionBegin(c) ( (void)0 )
#define RTCapsule_transitionEnd(c)   ( (void)0 )

#endif /* OBSERVABLE */

void RTCapsule_incarnateAll    ( RTCapsule * );
void RTCapsule_init            ( RTCapsule *, struct _RTController * );

void RTCapsule_exitToChainState( RTCapsule *, RTStateIndex );
void RTCapsule_enterState      ( RTCapsule *, RTStateIndex );
void RTCapsule_exitState       ( RTCapsule * );
void RTCapsule_processHistory  ( RTCapsule * );
void RTCapsule_initializeFSM   ( RTCapsule * );

void RTCapsule_unexpectedMessage( const RTCapsule * );
void RTCapsule_unexpectedState  ( const RTCapsule * );

#if RTS_NAMES
int RTCapsule_getCompIndex( const RTCapsule * this, const char * compName );
#endif

const struct _RTCapsule_class * RTCapsule_getClass( const RTCapsule * );
struct _RTCapsuleRole * RTCapsule_getRoleFromIndex( const RTCapsule *, int );

RTCapsule * RTCapsule_getCompFromIndex( const RTCapsule * this,
					int               compIndex,
					int               repIndex );

#if RTS_NAMES
RTCapsule * RTCapsule_getCompFromName( const RTCapsule * this,
				       const char      * compName,
				       int               repIndex );
const char * RTCapsule_getStateString( const RTCapsule *, int );
#endif

#if LOG_MESSAGE
void RTCapsule_logMsg( RTCapsule * );
#endif

#if OBJECT_ENCODE
int RTCapsule_displayVariable( const RTCapsule               *,
			       const struct _RTCapsule_class *,
			       int,
			       char                          *,
			       int );
#endif

#if TIMING_SERVICE
int RTCapsule_isTimerValid( RTCapsule *, RTTimerId );
#endif

/* "virtual" */

/* "subclass" methods */

/* static (class) methods */
void RTCapsule__initTop ( void );
int  RTCapsule__bindPort( struct _RTEndPort * );

typedef struct _RTCapsuleInstanceDescriptor
{
    const RTPortIndex   PortOffset;
    const RTThreadIndex ThreadIndex;
} RTCapsuleInstanceDescriptor;

#if RTS_INLINES
#   include <RTPubl/Capsule.inl>
#endif

#endif /* __RTPubl_Capsule_h__ */
