/*
 * IBM Confidential
 * OCO Source Materials
 * Component IBM Rational ROSE_RT BUILD : ROSE_RT
 * (c) Copyright IBM Corp. [1998], [2004] .All Rights Reserved.
 * 
 * The source code for this program is not published or
 * or otherwise divested of its trade secrets, irrespective
 * of what has been deposited with the U.S. Copyright Office.
 * 
 * 
 * Copyright (c) 1991-2003 Rational Software Corporation. All rights
 * reserved.
 * 
 * Rational, the Rational logo, ObjecTime (and logo) are registered
 * trademarks of Rational Software Corporation.
 * 
 * The placing of the copyright notices on Rational source code is not
 * intended to mean that they are published works, but are included in the
 * event of inadvertent publication. The source code of Rational contains
 * trade secrets and other valuable confidential information of Rational
 * Software Corporation. Rational Software Corporation and its licensors
 * retain ownership to Rational Products in both source and object code
 * format, and related documentation and all rights therein including
 * copyrights, patents and trade secrets.
 * 
 * 
 * U.S. Government Rights
 * 
 * Use, duplication, or disclosure by the U.S. Government is subject to
 * restrictions set forth in the applicable Rational License Agreement
 * and in DFARS 227.7202-1(a) and 227.7202-3(a) (1995), DFARS
 * 252.227-7013(c)(1)(ii)(Oct 1988), FAR 12.212(a) 1995, FAR 52.227-19,
 * or FAR 52.227-14, as applicable.
 * 
 * Unless otherwise agreed to in writing, use of Rational products is
 * governed by the Rational Product License Agreement provided with
 * the product.
 * 
 */


#ifndef __RTPubl_Control_h__
#define __RTPubl_Control_h__ included

#include <RTPubl/Capsule.h>
#include <RTPubl/Message.h>
#include <RTPubl/MsgQueue.h>
#include <RTPubl/ObjClass.h>
#include <RTPubl/Port.h>
#include <RTPubl/Priority.h>
#include <RTPubl/TimerId.h>

/* forward declarations */
struct _RTController;

#if OTRTSDEBUG != DEBUG_NONE
struct _RTEventInfo;
#endif

#if USE_THREADS
struct _RTMutex;
struct _RTSyncObj;
#endif

/*
** This header file represents the interface for the RTController class
** for the C Run-Time System.
*/

/* "nested" class definitions */
/* This enum represents the different phases that a job can be in. */

typedef enum _RTController_JobPhase
{
    RTController_created, /* created but not much else  */
    RTController_running, /* initialized, job executing */
    RTController_finished /* job finished, thread gone  */
} RTController_JobPhase;

/*  This enum represents the different primitives that the RSL provides.
    If something goes wrong, the user is expected to check the controller's
    error fields.  Thus, we need to provide a mapping between errors and
    primitives. */

typedef enum _RTController_Primitive
{
    RTController_CommDeliver,
    RTController_CommSend,
    RTController_CommSendAt,
#if MESSAGE_DEFERRAL
    RTController_CommDefer,
    RTController_CommRecall,
    RTController_CommRecallAll,
    RTController_CommRecallAt,
#endif
#if INTERNAL_LAYER_SERVICE
    RTController_LayerBind,
    RTController_LayerRegister,
    RTController_LayerDeregister,
#endif
#if TIMING_SERVICE
    RTController_TimerCancel,
    RTController_TimerInform,
#endif
    RTController_SystemService
} RTController_Primitive;

/* This enum represents all sorts of error codes that can
   be raised throughout the system.  */
typedef enum _RTController_PrimitiveError
{
    RTController_ok,                  /* all */
    RTController_internalError,       /* all */
    RTController_unexpectedStatus,    /* debugger ops */
    RTController_unexpectedPrimitive, /* debugger ops */
    RTController_cannotSetTimer,
    RTController_cannotRegTimer,
    RTController_unauthorizedMemoryAllocation,
    RTController_alreadyDeferred,     /* CommDefer */
    RTController_badClass,            /* CommDeliver, CommSend */
    RTController_badId,               /* TimerInform */
    RTController_badOperation,        /* LayerDeregister, LayerRegister */
    RTController_badMessage,          /* CommSend */
    RTController_badSignal,           /* CommSend */
    RTController_badState,            /* CommSend */
    RTController_badValue,            /* TimerInform */
    RTController_dereg,               /* LayerDeregister */
    RTController_noConnect,           /* TimerInform */
    RTController_noMem,               /* all */
    RTController_notBound,            /* CommSend */
    RTController_prio,                /* CommSend */
    RTController_reg,                 /* LayerRegister */
    RTController_tooManySAPs          /* LayerRegister */
} RTController_PrimitiveError;

/* pseudo-virtual function ptr type declarations  */
typedef void ( * RTController__mainLoop )( struct _RTController * );
typedef void ( * RTController__sleep    )( struct _RTController * );
typedef void ( * RTController__wakeup   )( struct _RTController * );

#if TIMING_SERVICE
typedef RTTimerId ( * RTController__informIn )( RTCapsule            * this,
						const RTPort         * port,
						RTPortIndex            index,
						const RTTimespec     * timeout,
						RTPriority             prio,
						const void           * data,
						const RTObject_class * type );

typedef int ( * RTController__cancel )( RTCapsule * this, RTTimerId timerId );
typedef int ( * RTController__valid  )( RTCapsule * this, RTTimerId timerId );
#endif /* TIMING_SERVICE */

/* the class */
/* note that none of these fields should be referenced directly.
   This is not part of the supported API for user code */

typedef struct _RTController
{
#if RTS_NAMES
    const char                  * _name;
#endif

#if TIMING_SERVICE
    RTCapsule                   * timer;
    RTController__informIn        informIn;
    RTController__cancel          cancel;
    RTController__valid           valid;
#endif /* TIMING_SERVICE */

    /* stuff we do not expect to have initialized at generate time */
    RTMessageQueue                internalQ[ NUMBER_OF_PRIORITIES ];

#if USE_THREADS
    RTMessageQueue                incomingQ[ NUMBER_OF_PRIORITIES ];
#endif

#if MESSAGE_DEFERRAL
    /* note: we do not support per actor defer queues */
    RTMessageQueue                deferredQ;
#endif

#if USE_THREADS
    struct _RTMutex             * _mutex;
    struct _RTSyncObject        * _sync;
#endif /* USE_THREADS */
    
	RTController__mainLoop        mainLoop;
	RTController__sleep           waitForEvents;
    RTController__wakeup          wakeup;
    
	RTMessage                   * freeList;

#if OTRTSDEBUG != DEBUG_NONE
    int                           debuggerIndex;
#endif

    RTController_JobPhase         _phase;
	RTController_PrimitiveError   errorNumber;

    RTMessageIndex                _numInternalMsgs;
	RTMessageIndex                freeListSize;
    
#if NUMBER_OF_PRIORITIES > 1
    short                         internalBits;
#if USE_THREADS
    short                         incomingBits;
#endif
#endif /* NUMBER_OF_PRIORITIES > 1 */

    short                         ioInterrupts;

} RTController;

/*
    public API

    error routines:
	getError - return the error code for the controller
	strError - translate the controller's error code into a
		   text description
	perror   - print out a text error, much like C perror()
*/

RTS_INLINE
RTController_PrimitiveError   RTController_getError( const RTController * );

const char                  * RTController_strError( const RTController * );
#if STDIO_ENABLED
void                          RTController_perror( const RTController *,
						   const char         * );
#endif

/* abort the entire running process */
void RTController_abort( RTController * );

/*
    Return the text name of the controller
    (equivalent to physical thread name in the model).
*/
#if RTS_NAMES
RTS_INLINE const char * RTController_name( RTController * );
#endif

#if TIMING_SERVICE

/*
    For *this* physical thread, register the capsule pointed to by *timer*
    as the timer capsule; *timer* should provide operations that implement
    *informInFn*, *cancelFn*, and *validFn*
*/

int RTController_registerTimer( RTController           * this,
				void                   * timer,
				RTController__informIn   informInFn,
				RTController__cancel     cancelFn,
				RTController__valid      validFn );

/*
    For *this* physical thread, override its go-to-sleep and wake-up
    primitives with user defined ones.  This is necessary to ensure that
    the controller object can work with an integrated timer as well as
    dispatch its own messages.
*/

int RTController_overrideSyncMethods( RTController         * this,
				      RTController__sleep    sleepFn,
				      RTController__wakeup   wakeupFn );

#endif /* TIMING_SERVICE */

/*
   protected/private interface:
   ---------------------------------------------------------------------

   The rest of these methods are part of the support API and
   are not meant to be used in user code.
*/

/* construction/destruction */
void           RTController_construct( RTController   *,
#if RTS_NAMES
				       const char     * name,
#endif
				       RTMessageIndex );

void           RTController_destruct( RTController * );

void RTController_setError( RTController                *,
			    RTController_PrimitiveError,
			    RTController_Primitive,
			    const void                  *,
			    int );

void RTController_init ( RTController * );
void RTController_enter( RTController *, RTController_JobPhase );
RTS_INLINE RTController_JobPhase RTController_phase( const RTController * );

#if OBSERVABLE
void RTController_breakPoint( RTController        *,
			      RTCapsule           *,
			      struct _RTEventInfo * );
#endif
#if OTRTSDEBUG != DEBUG_NONE
void RTController_setDebugger( RTController * );
void RTController_trace( RTController           *,
			 RTController_Primitive,
			 const void             *,
			 int                      index );
void RTController_printStats( const RTController * );
#endif /* OTRTSDEBUG != DEBUG_NONE */

/* send */
int RTController_send( RTController *,
		       RTCapsule    * receiver,
		       int            controlSignal,
		       const void   * data,
		       int            priority );

/* make a controller receive a message
   calls peerReceive if cross thread */
void RTController_receive( RTController *, RTMessage *, RTController * );

/* make a different controller receive a message using a mutex */
void RTController_peerReceive( RTController *, RTMessage * );

/* message pooling */
RTMessage * RTController_newMsg( RTController * );
void        RTController_freeMsg( RTController *, RTMessage * );

/* protected member functions */
typedef enum _RTController_DispatchResult
{
    RTController_OneDelivered,
#if OTRTSDEBUG != DEBUG_NONE
    RTController_PermissionDenied,
#endif
    RTController_NoneToDeliver
} RTController_DispatchResult;

RTController_DispatchResult RTController_dispatch( RTController *
#if NUMBER_OF_PRIORITIES > 1
						 , int            priorityLimit
#endif
								     );
void            RTController_recvSignal( RTController *,
					 RTSignal,
					 RTController * );
RTS_INLINE void RTController_kill      ( RTController * target,
					 RTController * source );

#if OTRTSDEBUG != DEBUG_NONE
RTS_INLINE void RTController_attention ( RTController * target,
					 RTController * source );
#endif

void RTController_awaitResponse( RTController * );

#ifdef USE_THREADS
void RTController_wakeup( RTController * );
#endif

#if RTS_INLINES
#include <RTPubl/Control.inl>
#endif

#endif /* __RTPubl_Control_h__ */
