/*
 * IBM Confidential
 * OCO Source Materials
 * Component IBM Rational ROSE_RT BUILD : ROSE_RT
 * (c) Copyright IBM Corp. [1998], [2004] .All Rights Reserved.
 * 
 * The source code for this program is not published or
 * or otherwise divested of its trade secrets, irrespective
 * of what has been deposited with the U.S. Copyright Office.
 * 
 * 
 * Copyright (c) 1991-2003 Rational Software Corporation. All rights
 * reserved.
 * 
 * Rational, the Rational logo, ObjecTime (and logo) are registered
 * trademarks of Rational Software Corporation.
 * 
 * The placing of the copyright notices on Rational source code is not
 * intended to mean that they are published works, but are included in the
 * event of inadvertent publication. The source code of Rational contains
 * trade secrets and other valuable confidential information of Rational
 * Software Corporation. Rational Software Corporation and its licensors
 * retain ownership to Rational Products in both source and object code
 * format, and related documentation and all rights therein including
 * copyrights, patents and trade secrets.
 * 
 * 
 * U.S. Government Rights
 * 
 * Use, duplication, or disclosure by the U.S. Government is subject to
 * restrictions set forth in the applicable Rational License Agreement
 * and in DFARS 227.7202-1(a) and 227.7202-3(a) (1995), DFARS
 * 252.227-7013(c)(1)(ii)(Oct 1988), FAR 12.212(a) 1995, FAR 52.227-19,
 * or FAR 52.227-14, as applicable.
 * 
 * Unless otherwise agreed to in writing, use of Rational products is
 * governed by the Rational Product License Agreement provided with
 * the product.
 * 
 */


#ifndef __RTPubl_Decoding_h__
#define __RTPubl_Decoding_h__ included

#include <RTPubl/Config.h>

#if OBJECT_DECODE

struct _RTIBuffer;
struct _RTObject_class;
struct _RTFieldDescriptor;

enum { RTDecoding_UngetMax = 3 };

typedef struct _RTDecoding
{    
	struct _RTIBuffer * input;
	int                 unget_size;
	char                unget_buffer[ RTDecoding_UngetMax ];
} RTDecoding;

/* allocate, deallocate, ctor, dtor */
void RTDecoding_construct( RTDecoding *, struct _RTIBuffer * );
/* destructor not needed - nothing allocated */

/* public member methods */
/* All the get/put functions return an int which is non-zero
** if the operation was successful. Otherwise, the following
** function may be called to discover the reason for the
** failure.
*/
int RTDecoding_getError    ( RTDecoding * );

int RTDecoding_get         ( RTDecoding *, void **,
			     const struct _RTObject_class ** );

/* basic data types */
int RTDecoding_get_address ( RTDecoding *, void  ** );
int RTDecoding_get_bool    ( RTDecoding *, int    * );
int RTDecoding_get_char    ( RTDecoding *, char   * );
#if RTUseFloatingPoint
int RTDecoding_get_double  ( RTDecoding *, double * );
#endif
int RTDecoding_get_enum    ( RTDecoding *, int *,
			     int, const struct _RTFieldDescriptor * );
#if RTUseFloatingPoint
int RTDecoding_get_float   ( RTDecoding *, float  * );
#endif
int RTDecoding_get_int     ( RTDecoding *, int    * );
int RTDecoding_get_long    ( RTDecoding *, long   * );
int RTDecoding_get_short   ( RTDecoding *, short  * );

/* a nul-terminated string */
int RTDecoding_get_string  ( RTDecoding *, char ** );

/* a homogeneous array of objects */
int RTDecoding_get_array   ( RTDecoding *, void *, int,
			     const struct _RTObject_class * );

/* an indirect object */
int RTDecoding_get_indirect( RTDecoding *, void *,
			     const struct _RTObject_class * );

/* a structure */
int RTDecoding_get_struct  ( RTDecoding *, void *,
			     const struct _RTObject_class * );

/* protected member methods */
int RTDecoding_getName     ( RTDecoding *, char * value, int length );

int  RTDecoding_getNonWhite( RTDecoding *, char * );
int  RTDecoding_getByte    ( RTDecoding *, char * );
RTS_INLINE void RTDecoding_ungetByte  ( RTDecoding *, char );

int  RTDecoding_getFieldListStart ( RTDecoding * );
RTS_INLINE int  RTDecoding_getFieldName      ( RTDecoding *, char * value, int length );
int  RTDecoding_getFieldSeparator ( RTDecoding * );
int  RTDecoding_getFieldListFinish( RTDecoding * );

#if RTS_INLINES
#include <RTPubl/Decoding.inl>
#endif

#endif /* OBJECT_DECODE */

#endif /* __RTPubl_Decoding_h__ */
