/*
 * IBM Confidential
 * OCO Source Materials
 * Component IBM Rational ROSE_RT BUILD : ROSE_RT
 * (c) Copyright IBM Corp. [1998], [2004] .All Rights Reserved.
 * 
 * The source code for this program is not published or
 * or otherwise divested of its trade secrets, irrespective
 * of what has been deposited with the U.S. Copyright Office.
 * 
 * 
 * Copyright (c) 1991-2003 Rational Software Corporation. All rights
 * reserved.
 * 
 * Rational, the Rational logo, ObjecTime (and logo) are registered
 * trademarks of Rational Software Corporation.
 * 
 * The placing of the copyright notices on Rational source code is not
 * intended to mean that they are published works, but are included in the
 * event of inadvertent publication. The source code of Rational contains
 * trade secrets and other valuable confidential information of Rational
 * Software Corporation. Rational Software Corporation and its licensors
 * retain ownership to Rational Products in both source and object code
 * format, and related documentation and all rights therein including
 * copyrights, patents and trade secrets.
 * 
 * 
 * U.S. Government Rights
 * 
 * Use, duplication, or disclosure by the U.S. Government is subject to
 * restrictions set forth in the applicable Rational License Agreement
 * and in DFARS 227.7202-1(a) and 227.7202-3(a) (1995), DFARS
 * 252.227-7013(c)(1)(ii)(Oct 1988), FAR 12.212(a) 1995, FAR 52.227-19,
 * or FAR 52.227-14, as applicable.
 * 
 * Unless otherwise agreed to in writing, use of Rational products is
 * governed by the Rational Product License Agreement provided with
 * the product.
 * 
 */


#ifndef __RTPriv_Debugger_h__
#define __RTPriv_Debugger_h__ included

#include <RTPriv/ContInfo.h>
#include <RTPriv/DebugInp.h>
#include <RTPriv/EventInf.h>
#include <RTPubl/CapsRole.h>
#include <RTPubl/Capsule.h>
#include <RTPubl/Control.h>
#if USE_THREADS
#include <RTPubl/PeerCont.h>
#endif
#include <RTPubl/Port.h>
#include <RTPubl/Priority.h>
#include <RTPubl/Timespec.h>

struct _RTMessage;
struct _RTObject_class;

/* "nested enumerations */

/* types of traces */
typedef enum _RTDebugger_TraceCategory
{
    RTDebugger_TraceComm,
#if INTERNAL_LAYER_SERVICE
    RTDebugger_TraceLayer,
#endif
#if TIMING_SERVICE
    RTDebugger_TraceTimer,
#endif
    RTDebugger_TraceSystem
} RTDebugger_TraceCategory;

enum
{
    RTDebugger_TraceAll = RTDebugger_TraceSystem + 1
};

/* types of modes */
typedef enum _RTDebugger_Mode
{
    RTDebugger_Halted,
    RTDebugger_Stepping,
    RTDebugger_Running
} RTDebugger_Mode;

/* types of commands */
typedef enum _RTDebugger_Command
{
    RTDebugger_CmdInvalid,
    RTDebugger_CmdNone,
    RTDebugger_CmdAnimate,
    RTDebugger_CmdAttach,
    RTDebugger_CmdCont,
    RTDebugger_CmdDetach,
    RTDebugger_CmdExit,
    RTDebugger_CmdGo,
    RTDebugger_CmdHelp,
    RTDebugger_CmdInfo,
    RTDebugger_CmdLog,
    RTDebugger_CmdQuit,
    RTDebugger_CmdSaps,
    RTDebugger_CmdStats,
    RTDebugger_CmdStep,
    RTDebugger_CmdStop,
    RTDebugger_CmdSystem,
    RTDebugger_CmdTasks
} RTDebugger_Command;

/* types of debuggers */
typedef struct _RTDebugger
{
	RTControllerInfo ** tasks;    /* array of tasks */
    unsigned            step_count;
    RTDebugger_Mode     mode;
    RTController      * controller;
#if OTRTSDEBUG != DEBUG_NONE
    RTTimespec          lastTimestamp;
    RTTimespec          zeroTimeBase;
#endif
	RTDebuggerInput     input;    /* info */
	unsigned char       traceLevel[ RTDebugger_TraceAll ];
} RTDebugger;

/* instance methods */
/* this is a singleton object -- no construction/destruction
   only startup & shutdown
*/
RTDebugger * RTDebugger_soleInstance( void );
#if OTRTSDEBUG != DEBUG_NONE
void         RTDebugger_startup( void );
void         RTDebugger_shutdown( void );
#endif

/* task oriented stuff */
RTControllerInfo  * RTDebugger_getInfo( int );
RTController      * RTDebugger_getTask( int );

/* job oriented stuff */
RTS_INLINE struct _RTMessage * RTDebugger_newMsg( void  );
RTS_INLINE void                RTDebugger_freeMsg( struct _RTMessage * );

/* controller stuff ... hopefully we will get rid of it and pass the buck off to the controller */
void         RTDebugger_initialize( void );
void         RTDebugger_debugIt( const char * cmdLine );
#if USE_THREADS
void         RTDebugger_mainLoop( RTPeerController * );
#endif

void         RTDebugger_doCmdAttach( void );
void         RTDebugger_doCmdDetach( void );
void         RTDebugger_doCmdLog   ( void );
void         RTDebugger_doCmdQuit  ( void );
void         RTDebugger_doCmdSystem( void );
void         RTDebugger_doCmdTasks ( void );
void         RTDebugger_doCmdStats( void );

void         RTDebugger_printCapsule( const RTCapsule * );
void         RTDebugger_printPort( const RTPort *, RTPortIndex );
void         RTDebugger_printRef( const RTCapsule *, const char * );

int          RTDebugger_getCommand( void );
int          RTDebugger_getCategory( void );
int          RTDebugger_getLevel( void );

void         RTDebugger_printHelp( void );
void         RTDebugger_printStructure( const RTCapsule *, const char *, int );
void         RTDebugger_printCapsuleInfo( RTCapsule * );

RTCapsule  * RTDebugger_getCapsule( void );
RTCapsule  * RTDebugger_findCapsule( const char *, RTCapsuleRole * );

void RTDebugger_setTimestamp( RTMessage *, const RTPort *, RTPortIndex );
void RTDebugger_getTimestamp( RTTimespec * );

/* general debugger stuff */
int  RTDebugger_doCmd( void );
void RTDebugger_doCmdExit( void );
void RTDebugger_trace( RTController_Primitive,
		       const char             * errorDescription,
		       const void             * info,
		       int                      index );

void RTDebugger_logMessage( const struct _RTMessage * );

#if TIMING_SERVICE
void RTDebugger_logTimer( const RTPort *, RTSignal, long );
#endif

#if INTERNAL_LAYER_SERVICE
RTS_INLINE void RTDebugger_logSAP( RTPort * role, const char * name );
RTS_INLINE void RTDebugger_logSPP( RTPort * role, const char * name );
void RTDebugger_logLayer( RTPort * role, const char * name, int type );
#endif

int  RTDebugger_registerRTS( RTController * );
int  RTDebugger_attach( int controllerId );
int  RTDebugger_detach( int controllerId );
void RTDebugger_deregisterRTS( int * controllerId );

void RTDebugger_printMessage( const RTMessage * );

int  RTDebugger_breakPoint( int controllerId, RTEventInfo * );
int  RTDebugger_step_begin( int controllerId );
void RTDebugger_step_end( int controllerId );

void RTDebugger_step( unsigned count );
void RTDebugger_proceed( void );

int  RTDebugger_doInject( RTPort *, RTPortIndex,
			  RTSignal, RTPriority,
			  void *, const struct _RTObject_class * );

#if ! USE_THREADS
int RTDebugger_isHalted( void );
#endif

#if RTS_INLINES
#include <RTPriv/Debugger.inl>
#endif

#endif /* __RTPriv_Debugger_h__ */
