# --- This is a perl file ----


BEGIN {
  $curdir=`dirname $0`;
  chomp($curdir);
  $pwd=`pwd`;
  chomp($pwd);
  if($curdir eq "."){ $fulldir="$pwd";}
  elsif ($curdir =~ /^\//) {$fulldir=$curdir;}
  else {$fulldir="$pwd/$curdir";}
  push(@INC,"$fulldir","$fulldir/tools");
}

# set autoflush for STDOUT
select(STDOUT);
$|=1;


package main;

$BOM_FILES_DIR        = "$ENV{'MWHOME'}/bin/mwpack_tools";

%components_full_name = (win32=>"Basic, Win32 run time components",
			 mfcatl=>"MFC and ATL run time",
			 xml=>"Microsoft XML run-time components",
			 scr=>"Scripting (JScript and VBScript) run-time components",
			 inet=>"Internet run-time components",
			 hh=>"HTML Help",
			 odbc=>"Data Direct ODBC run-time components",
			 jip=>"Java Integration Package run-time components");

%components_map       = (win32=>"win32",
			 mfcatl=>"mfc_atl",
			 xml=>"xml",
			 scr=>"scripting",
			 inet=>"internet",
			 hh=>"html_help",
			 odbc=>"ODBC",
			 jip=>"jip");

%platforms_map        = (solaris=>"SUNOS",
			 aix=>"AIX",
			 linux=>"LINUX",
			 hpux=>"HPUX11",
			 hpuxia=>"IAHPUX11");


%requires             = (mfc_atl=>"win32",
			 xml=>"win32",
			 scripting=>"win32",
			 internet=>"win32,scripting",
			 html_help=>"internet",
			 ODBC=>"win32",
			 jip=>"win32");

if ($ENV{'MWDEBUG'}) {
  $trace = $ENV{'MWDEBUG'};
}

sub trace{
  my ($level,$message)=@_;
  if ($level <= $::trace) {
    print STDERR "trace $level: $message\n";
  }
}

sub my_exit{
  my $message = shift;
  print STDERR "$message\n";
  exit 1;
}


sub usage{
  my $status = shift;
  print "mwpack <options> <components>  
      <options>
        -h
        Print this help text.
	 
       -p  { solaris | aix | linux | hpux | hpuxia}
       Platform selection - specify the platform. can be repeated. 
       Note - at least one platform must be specified.
 
       <components> 
        List of runtime components used by the application. At least one component must be specified. Valid component names are:
 
             win32 - Basic, Win32 run time components
             mfcatl - MFC and ATL run time components. <includes: win32>
             xml - Microsoft XML run-time components <includes: win32>
             scr - Scripting (JScript and VBScript) run-time components <includes: win32>
             inet - Internet run-time components <includes: win32, scr>
             hh - HTML Help <inet>
             odbc - Data Direct ODBC run-time components <includes: win32>
             jip - Java Integration Package run-time components <includes: win32>

      Note- dependencies are handled automatically. For example, if you specify mfc, there is no need to also specify win32 as it will be included automatically with mfc.
\n";
  exit $status;
}

sub validPlatform{
  my $platform = shift;
  
  if ($platforms_map{$platform} ne ""){
    return 1;
  }
  return 0;
}


sub validComponent{
  my $component = shift;
  if ($components_map{$component} ne ""){
    return 1;
  }
  return 0;
}
sub create_filelist_for_comp{
  my $component=shift;
  foreach $platform (@platforms){
    if (! -f "$BOM_FILES_DIR/${component}_${platform}.bom"){
      trace(1,"File $BOM_FILES_DIR/${component}_${platform}.bom does not exist");
      next;
    }
    trace(1,"Running cat $BOM_FILES_DIR/${component}_${platform}.bom\n");
    @files=`cat $BOM_FILES_DIR/${component}_${platform}.bom 2> /dev/null`;
    if ($? != 0){
      print STDERR "Warning: Failed read from $BOM_FILES_DIR/${component}_${platform}.bom\n";
    }
    push(@all_files,@files);
   
  }
  if ($requires{$component} ne ""){
    @comp_requires=split(/,/,$requires{$component});
    foreach $require (@comp_requires){
      create_filelist_for_comp($require);
    }
  }
}

sub main{
  foreach $component (@components){
    create_filelist_for_comp($component)
  }
  my @newlist;
  my @list;
  @sorted_list=sort @all_files;
  $prev_file="";
  foreach $file (@sorted_list){
    next if ($file eq "");
    next if ($file eq $prev_file);
    $prev_file=$file;
    push(@newlist,$file);
  }
  if ( $newlist[0] eq ""){shift @newlist;}
  print @newlist;
  return 0;

}

# ==== argument handline =======
@platforms;
@languages;
@components;
@all_files;

usage() if (@ARGV==0);

for ($i=0; $i<@ARGV; $i++) {
  if ($ARGV[$i] eq "-p" && $ARGV[$i+1]){
    if (!validPlatform($ARGV[$i+1])){
      print "Specified platform '$ARGV[$i+1]' is not valid.\n";
      usage(1);
    }
    push(@platforms,$platforms_map{$ARGV[$i+1]}); $i++; next;
  }
  
  if ($ARGV[$i] eq "-h")
    {usage(0);}
  
  if (validComponent($ARGV[$i])){
    push(@components,$components_map{$ARGV[$i]});next;
  }
  else{
    print "Specified component '$ARGV[$i]' is not valid.\n";
    usage(1);
  }
}
if ($components[0] eq ""){shift @components;}
if ($platforms[0] eq ""){shift @platforms;}
if ($languages[0] eq ""){shift @languages;}
if ($#components == -1){ print "No components specified\n";exit 1;}
if ($#platforms == -1){ print "No platforms specified\n";exit 1;}
push(@platforms,"GENERIC");
main();
exit 0;
