#########################################################################
#                                                                       #
#  This program is the CONFIDENTIAL and PROPRIETARY property            #
#  of Mainsoft(R) Corporation. Any unauthorized use, reproduction       #
#  or transfer of this computer program is strictly prohibited.         #
#                                                                       #
#  Copyright (c) 1991-1998 Mainsoft Corporation.                        #
#  This is an unpublished work, and is subject to limited distribution  #
#  and restricted disclosure only. ALL RIGHTS RESERVED.                 #
#                                                                       #
#########################################################################


# NOTE: 
#   This is just a temporary version of the package. 
#   The permanent one will be a PPService derivative, which would enable
#   it to communicate with the MWXDE utility, standardize progress 
#   report, etc.

# ...
# Template classes are stored in a hash table, in which the key designates
# the class name and the value - an array of strings containing default 
# template parameters. The first member of the array indicates index of 
# the first default parameter in template's parameter list.
# ...


package atlpp;
use PPService;

sub new {
    my $class = shift;
    my $self = {};

    if( @_ < 2 ) {
	bless $self;
	$self->Abort( "new", "Insufficient number of arguments" );
    }
    $self->{refPPS} = shift;
    $self->{strTClassDataFile} = shift;
    $self->{bIfdefChanges} = shift;
    # decl %{$self->{hTClassUsage}};
    # decl %{$self->{hUuidofUsage}};
    # decl %{$self->{hTClassDefs}};
    bless $self;
}


sub SetIfdefChanges {
    my $self = shift;
    $self->{bIfdefChanges} = shift;
}


sub Abort {
    my $self = shift;
    my $strErr = sprintf( "\aFatal error in %s%s%s", ref( $self ), ( @_ > 1 ) ? "::" . shift : ":    ", shift );
    ${$self->{refPPS}}->PrintError( $strErr );
    exit( -1 );
}


sub PrintWarning {
    my $self = shift;
    ${$self->{refPPS}}->PrintError( shift );
}


sub RetrieveTClassData {
    my $self = shift;
    my $strSrcFile = $self->{strTClassDataFile};
    my $strClassName;
    my @arFileBuffer;
    my @arTemp;

    ReadFile( $strSrcFile, \@arFileBuffer ) || $self->Abort( "Unable to retreive default template parameters table." );

    foreach( @arFileBuffer ) {
	chomp;
	next if /^\/\//;                     # skip comments
	@arTemp = split( /\s*\#\s*/, $_ );
	next if scalar( @arTemp ) <= 2;      # default parm definition must contain at least 3 components
	$strClassName = shift( @arTemp );
	@{${$self->{hTClassDefs}}{$strClassName}} = @arTemp;
    }	
    close( FILE );
}


sub GetTClassNames {
    my $self = shift;
    my @arTClassNames;

    foreach( keys %{$self->{hTClassDefs}} ) {
	push( @arTClassNames, $_ );
    }
    return sort( @arTClassNames );
}


sub InsertMissingParams {
    my $self = shift;
    my $refStrString = shift;
    my $strClassName = shift;
    my @arDefParms = @{${$self->{hTClassDefs}}{$strClassName}};
    my $nParmIndex = shift( @arDefParms );
    my $nFirstMissingParmNdx = 0;
    my $strNewParms;

    pos() = 0;

    SWITCH: {
	# PERL BUG! 
	# Perl goes nuts when using global matching on dereferenced references.
	# For example, stuff like ( $$refStrString =~ m/\b$strClassName\s*(?=\<)/gs )
	# simply fails the pattern matching mechanism in cases where the above 
	# query evaluates as TRUE with a plain variable on the left side of the
	# expression.
	# That's why I used $_ (although solely for the matching purposes).
	$_ = $$refStrString;

	# If template initially contains any params, things get real messy...
	if( m/\b$strClassName\s*(?=\<)/gs ) {
	    my $pos = pos();
	    my $strPresentParms = GetBalancedSubstr( $$refStrString, '<', '>', $pos );

	    return -1 if !defined( $strPresentParms );

	    # Split present parameters into an array and compute index of the first
	    # missing parameter in the default  parms. array of current class.
	    my @arPresentParms = split( /,/, $strPresentParms );
	    $nFirstMissingParmNdx = scalar( @arPresentParms ) - $nParmIndex;
	    if( $nFirstMissingParmNdx < 0 ) {
		$self->PrintWarning( "Insufficient number of required parameters found in\n$$refStrString" );
		return -1;
	    }

	    # Insert missing parameters, if any
	    if( $nFirstMissingParmNdx - scalar( @arDefParms ) < 0 ) {
		my $strPresentParmsQM = quotemeta( $strPresentParms );
		while( $nFirstMissingParmNdx-- ) {
		    shift( @arDefParms );
		}
		
		# Replace references to non-default parameters
		map( s/@@(\d+)/$arPresentParms[$1]/e, @arDefParms );

		if( $self->{bIfdefChanges} ) {
		    $strNewParms = "#if defined( MAINWIN ) && defined( MWNO_DEF_IN_TEMPLATES )\n, " . join( ', ', @arDefParms ) . "\n#endif";
		    $$refStrString =~ s/\<$strPresentParmsQM\>/<$strPresentParms\n$strNewParms\n>/; #can't use /e here bacause of the <>'s
		}
		else {
		    $strNewParms = join( ', ', @arDefParms );
		    $$refStrString =~ s/\<$strPresentParmsQM\>/<$strPresentParms,$strNewParms>/; #can't use /e here bacause of the <>'s
		}
		return 1;
	    }
	    last SWITCH;
	}
	# Template definition has no parameters
	if( /\b$strClassName\b/ ) {
	    $strNewParms = join( ', ', @arDefParms );
	    $$refStrString =~ s/\b$strClassName\b/$strClassName<$strNewParms>/;
	    return 1;
	}
	return -1;  # unrecognized pattern
    }
    return 0;
}


sub ProcessTemplates {
    my $self = shift;
    my $strSrcFile = shift;
    my @arTClassNames = $self->GetTClassNames();
    my @arSrcFileBuffer;
    my $refStrCurLine;
    my $nCurLine;
    my $bChanged = 0;
    my $strClassDecl;
    my $nInsertMP_RetVal;

    ReadFile( $strSrcFile, \@arSrcFileBuffer ) || $self->Abort( "Unable to continue." );
    
    foreach( @{${$self->{hTClassUsage}}{$strSrcFile}} ) {
        $nCurLine = $_ - 1;
	$refStrCurLine = \$arSrcFileBuffer[ $nCurLine ];

        # skip comments, preprocessor directives and empty lines
	next if( $$refStrCurLine =~ /(^\/[\/\*]|^\#|^$)/ );

	foreach $strClassName( @arTClassNames ) {
	    if( $$refStrCurLine =~ /\b$strClassName\b/ ) {
		# unwrap the string till we get balanced <>'s
		while( IsBalancedStr( $$refStrCurLine, '<', '>' ) > 0 ) {
		    $$refStrCurLine .= $arSrcFileBuffer[ ++$nCurLine ];
		    $arSrcFileBuffer[ $nCurLine ] = "";
		}

		$nInsertMP_RetVal = $self->InsertMissingParams( $refStrCurLine, $strClassName );

	        SWITCH: {
		    if( $nInsertMP_RetVal > 0 ) {			
			$bChanged = 1;
			last SWITCH;
		    }
		    if( $nInsertMP_RetVal < 0 ) {
			chomp( $$refStrCurLine );
			$self->PrintWarning( "Unrecognized pattern [$$refStrCurLine]\n  encountered in $strSrcFile: ~$nCurLine\n" );
			last SWITCH;
		    }
		}

	    }
	}
    }
	
    if( $bChanged ) {
        WriteFile( $strSrcFile, \@arSrcFileBuffer ) || $self->Abort( "Unable to continue." );
	return 1;
    }

    return 0;
}


sub Run {
    my $self = shift;
    my $strTargetFiles = shift;
    my @arTClassNames;
    my @arTargetFiles;

    # Process templates
    $self->RetrieveTClassData();
    @arTClassNames = $self->GetTClassNames();
    ${$self->{refPPS}}->PrintTitle( scalar( @arTClassNames ), "Looking for source files that require ATL preprocessing" );

    foreach $strTClassName( @arTClassNames ) {
	# Since that's a relatively time-consuming operation and
	# the user might get really bored, we better show the scanning progress.
	${$self->{refPPS}}->PrintItem( "Looking for usage of class $strTClassName" );
	foreach( `fgrep -n $strTClassName $strTargetFiles` ) {
	    /^(\S+):(\d+)/;
	    /^(\d+):/ if !defined( $2 );
	    # If $strTargetFiles points to strictly one file in current directory,
	    # fgrep's output format will contain no filename component. Therefore,
	    # if $1 is undefined, it's safe to assume that $strTargetFiles is our
	    # one and only target.
	    push( @{${$self->{hTClassUsage}}{ defined( $2 ) ? $1 : $strTargetFiles }}, defined( $2 ) ? $2 : $1 );
        }
    }

    if( scalar( keys %{$self->{hTClassUsage}} ) ) {
	${$self->{refPPS}}->PrintTitle( scalar( keys %{$self->{hTClassUsage}} ), "Complementing missing default parameters in ATL templates" );
	foreach( keys %{$self->{hTClassUsage}} ) {
	    ${$self->{refPPS}}->PrintItem( "Processing file $_" );
	    $self->ProcessTemplates( $_ );
	}
    }
}


#
# Static methods
#

sub ReadFile {
    my $strSrcFile = shift;
    my $refArBuffer = shift;

    if( !open( FILE, "<$strSrcFile" ) ) {
	$self->PrintWarning( "Unable to open file $strSrcFile for reading\n" );
	return 0;
    }

    @$refArBuffer = <FILE>;
    close( FILE );
    return 1;
}


sub WriteFile {
    my $strSrcFile = shift;
    my $strTmpFile = "$strSrcFile.tmp";
    my $refArBuffer = shift;

    if( !open( FILE, ">$strTmpFile" ) ) {
	$self->PrintWarning( "Unable to open file $strTmpFile for writing\n" );
	return 0;
    }

    print FILE @$refArBuffer;
    close( FILE );

    if( ! -z $strTmpFile ) {
	if( 0 == system("diff $strSrcFile $strTmpFile > /dev/null")) {
	    unlink( "$strTmpFile" ) ;
	}
	else {
	    rename( "$strTmpFile", "$strSrcFile" );
	}
    }
    else {
	$self->PrintWarning( "Unable to create file of size ~" . ( -s $strSrcFile ) . ".\n" );
	return 0;
    }

    return 1;
}


sub IsBalancedStr {
    my( $strString, $strLToken, $strRToken ) = ( shift, shift, shift );
    my @arTmp;
    return scalar( @arTemp = split( /$strLToken/, $strString ) ) - 
	scalar( @arTemp = split( /$strRToken/, $strString ) );
}


#
# Returns substring located between 2 balanced (amountically) tokens. 
# Limitation: No check of [L...R] token order is performed.
#
sub GetBalancedSubstr {
    my( $str, $strLToken, $strRToken, $nLPos ) = ( shift, shift, shift, shift );
    my $nBalance = 0;

    $_ = $str;
    pos = $nLPos;

    while( m/($strLToken|$strRToken)/g ) {
	$nBalance += ( $1 =~ $strLToken ) ? -1 : 1;
	last if !$nBalance;
    }
    return !$nBalance ? substr( $_, $nLPos + 1, pos() - $nLPos - 2 ) : undef;
}


1;
