##########################################################################
#    This program is the CONFIDENTIAL and PROPRIETARY property
#    of Mainsoft(R) Corporation. Any unauthorized use, reproduction 
#    or transfer of this computer program is strictly prohibited.
#
#    Copyright (c) 1991-1998 Mainsoft Corporation.
#    This is an unpublished work, and is subject to limited distribution
#    and restricted disclosure only. ALL RIGHTS RESERVED.
#
##########################################################################
use PPService;

my $bAutoLoad = 0;
my $lowercase_include = 1;
my( $file, $incfile );

if(-z $pps) {
    $pps = new PPService(0,defined( $ENV{ MWXDE_TRACE } ) ? 1 : 0); 
}

if( $ARGV[0] eq "-autoload" ) {
    shift @ARGV;
    $bAutoLoad = 1;
    ChangeInclude( @ARGV );
}


sub Abort {
    $pps->PrintError( shift ) if( @_ );
    exit( -1 );
}


sub AUTOLOAD {
    print "      Uncknown function $AUTOLOAD called at line ".__LINE__.".\n";
    print "      Defaulting to ChangeInclude().\n";
    ChangeInclude( @_ );
}


sub ChangeInclude {
    while ($file= shift(@_)) {
	if ($file eq "-nolowercase") {
	    $lowercase_include = 0;
	    next;
	} elsif( $file =~ /^-/ ) {
	    Abort("invalid option: $file\n");
	}
	next if ((-l $file) || (! -f $file));
	
	Abort("Couldn't open file $file for reading") unless open(FROM_FILE,"<$file");
	$already_preprocessed = 0;
	while (<FROM_FILE>) {
	    if (/processed by mwprepro/) {
		$already_preprocessed = 1;
		last;
	    }
	}
	close (FROM_FILE);
	if ($already_preprocessed == 0) {
	    %unixfilename = (
			     'strstrea.h', 
			     'strstream.h'
			     );
	    
	    print "Updating \#include directives for $file\n" if $bAutoLoad;
	    Abort("Couldn't open file $file for reading") unless open(FROM_FILE,"<$file");
	    Abort("Couldn't open file $file.out for writing") unless open(TO_FILE,">$file.out");
	    while(<FROM_FILE>) {
		## first make sure that OpenGL includes use <GL/...>
		if (/^(\s*\#\s*include\s*[<\"])([gG][lL][\/\\])([^\"\s]+[>\"])/) {
		    print TO_FILE "${1}GL/${3}\n";
		} 
		elsif (/^\s*\#\s*include/) {
		    if (/\\/) {
			$orig = $_;
			$_ = "\L$_" if ($lowercase_include);
			tr/\\/\//;
			if (/^(\s*\#\s*include\s[\"<])[A-Za-z]:(.+)$/) {
			    #get rid of drive name
			    $_=$1.$2."\n";
			}
			&add_conditional_include($_, $orig);
		    } 
		    elsif (/^\s*\#\s*include\s*[<\"]([^\"\s]+)[>\"]/) {
			# Any filename replacements to be done here
			$translated = 0;
			$orig = $_;
			$incfile = "\L$1" if ($lowercase_include);
			$_ = "\L$_" if ($lowercase_include);
			foreach $i (keys(%unixfilename)) {
			    if ($i eq $incfile) {
				if ($translated == 0) {
				    s/$i/$unixfilename{$i}/;
				    $translated = 1;
				}
			    }
			}
			if ($translated == 1) {
			    &add_conditional_include($_, $orig);
			} else {
			    print TO_FILE $_;
			}
		    } else {
			print TO_FILE $_;
		    }
		} else {
		    print TO_FILE $_;
		}
	    }

	    close (FROM_FILE);
	    close (TO_FILE);
	    
	    if( -z "$file.out" ) {
		print STDERR "Conversion failed: couldn't create file of size " . ( -s $file ) . ". Please check your disk space.\n";
		Abort("Couldn't delete file $file.out") unless unlink( "$file.out" );
	    } # Add mwprepro label only if the file has actually been changed
	    elsif( system( "diff -bw $file $file.out > /dev/null" ) ) {
		my @arBuffer;
		Abort("Couldn't open file $file.out for reading") unless open( FILE, "<$file.out" );
		@arBuffer = <FILE>;
		close( FILE );
		Abort("Couldn't delete file $file.out") unless unlink( "$file.out" );

		Abort("Couldn't open file $file for writing") unless open( FILE,">$file" );
		unshift( @arBuffer, "// processed by mwprepro\n" );
		print FILE @arBuffer;
		close( FILE );
	    } # Nothing significant has been changed - remove the temp. file
	    else {
		Abort("Couldn't delete file $file.out") unless unlink( "$file.out" );
	    }
	}
    }   
}


sub add_conditional_include {
    local ($old_line, $new_line) = @_;
    print TO_FILE "#ifdef unix\n";
    print TO_FILE $old_line;
    print TO_FILE "#else\n";
    print TO_FILE $new_line;
    print TO_FILE "#endif\n";

}


1;
