# Target : VMW reguire specific C++ compiler version
# Author : Olga Podvalner
# Searches 'PATH' for the full path for the command.
sub which
{
    my ($exe) = @_;
    my $path = `/bin/sh -c "type $exe" 2>&1`;
    return $' if ($path =~ /\Q$exe\E is /);
    return "";
}

# Common base class for outputing shell commands.
package basic_shell;
{
    sub new {
	my ($type) = @_;
	my $self = bless {}, $type;
	$self->{lib_path_name} = $ENV{$::LIB_PATH_ENV};
	$self->{new_path} = "";
	$self->{new_lib_path} = "";
	return $self;
    }
    sub setenv; # pure. Used to set envirnment vars.
    sub unsetenv; # pure. Used to unset envirnment vars.
    sub subst_current_path;

    # Used to add components to $PATH.
    sub add_to_path
    {
        my ($self) = (shift);
	my $val = "";
	while ($_ = pop (@_))
	     { $val = "$_:$val" if (-d $_); }
	$self->{new_path} = "$val$self->{new_path}";
    }

    # Used to add components to library path
    sub add_to_libpath
    {
        my ($self) = (shift);
	my $val = "";
	while ($_ = pop (@_))
	     { $val = "$_:$val" if (-d $_); }
	$self->{new_lib_path} = "$val$self->{new_lib_path}";

    }

    sub set_path_libpath
    {
        my ($self) = (shift);
	if ($self->{new_path}) {
	    $self->subst_current_path("PATH","MWCURRENT_COMP_PATH",
				      substr($self->{new_path},0,-1));
	}
	if ($self->{new_lib_path}) {
	    $self->subst_current_path("$self->{lib_path_name}","MWCURRENT_COMP_LIBPATH",
				      substr($self->{new_lib_path},0,-1));
	}
    }
}

# Outputs Csh/tcsh commands.
package Cshell; @ISA = qw (basic_shell);
{
    sub setenv
    {
	my ($self, $var, $val) = @_;
	print "setenv $var \"$val\"\n";
    }

    sub unsetenv
    {
	my ($self) = shift @_;
	print "unsetenv ", join(" ", @_), "\n";
    }

    sub subst_current_path
    {
	my ($self,$var,$curr_var,$val) = @_;
	print "if (! \$?${curr_var}) then\n";
	print "    setenv ${curr_var} \"+None+\"\n";
	print "endif\n";
	print "setenv ${var} \`echo \"::\${${var}}\" | sed -e \"s,:\${${curr_var}}:,,\" -e 's,^::,:,'\`\n";
	print "setenv ${curr_var} \"${val}\"\n";
	print "setenv ${var} \"\${${curr_var}}\${${var}}\"\n";
    }
}

# Outputs sh/ksh commands.
package Shell; @ISA = qw (basic_shell);
{
    sub setenv
    {
	my ($self, $var, $val) = @_;
	print "$var=\"$val\"\n";
	print "export $var\n";
    }

    sub unsetenv
    {
	my ($self) = shift @_;
	print "unset ", join(" ", @_), "\n";
    }

    sub subst_current_path
    {
	my ($self,$var,$curr_var,$val) = @_;
	print "if [ \"\${${curr_var}:-notset}\" = \"notset\" ]; then\n";
	print "    ${curr_var}=\"+None+\"\n";
	print "fi\n";
	print "${var}=\`echo \"::\${${var}}\" | sed -e \"s,:\${${curr_var}}:,,\" -e 's,^::,:,'\`\n";
	print "${curr_var}=\"${val}\"\n";
	print "${var}=\"\${${curr_var}}\${${var}}\"\n";
	print "export ${var} ${curr_var}\n";
    }
}

# Handle calculation of the following environment variables:
# MWCC, MWCCPP, MWCC_HOME, MWCCPP_HOME, MWCC_INCLUDE_DIR, MWCCPP_INCLUDE_DIR
package compiler_setup;
{
    sub new
    {
	my $type = shift;
	my $self = bless {}, $type;
	# By default, add the C and C++ compiler directories to the PATH.
	$self->{set_ccpp_path} = $::TRUE;
	$self->{set_cc_path} = $::TRUE;
	# When version of compiler is part of path, holds the version string
	$self->{version_path} = "";
	# name of C++ / C compiler
	$self->{ccpp_compiler_name} = "";
	$self->{cc_compiler_name} = "";
	# default location of C++ / C compiler (full path)
	$self->{ccpp_default_home} = "";
	$self->{cc_default_home} = "";
	# the necessary part of the compilers path, if it's not there the
	# location is not valid
	$self->{ccpp_necessary_path} = "";
	$self->{cc_necessary_path} = "";
	# location of C++ / C compiler (relative to mwcc/pp_home)
	$self->{ccpp_dir} = "bin";
	$self->{cc_dir} = "bin";
	# location of include directories (relative to mwcc/pp_home)
	$self->{ccpp_include} = [];
	$self->{cc_include} = [];
	# list of environment variable compiler_setup has to set
	%{$self->{env_vars}} =
	    (
	     $::MWCC_HOME_ENV => "",
	     $::MWCCPP_HOME_ENV => "",
	     $::MWCC_ENV => "",
	     $::MWCCPP_ENV => "",
	     $::MWCC_INCLUDE_DIR_ENV => "",
	     $::MWCCPP_INCLUDE_DIR_ENV => ""
	     );
	# Are MWCC_HOME / MWCCPP_HOME valid values
	$self->{accepted_cc} = $::FALSE;
	$self->{accepted_ccpp} = $::FALSE;
	return $self;
    }

# Read values of MWCC_HOME and MWCCPP_HOME environment variables,
# set by user.
    sub user_defined_home
    {
	#print "in user_defined_home\n";
	my $self = shift;
	my $cc_home = $ENV{$::MWCC_HOME_ENV};
	my $ccpp_home = $ENV{$::MWCCPP_HOME_ENV};

	$self->{env_vars}{$::MWCC_HOME_ENV} = $cc_home;
	$self->{env_vars}{$::MWCCPP_HOME_ENV} = $ccpp_home;

	my $status;
	# if user defined one of the variables, try to deduce
	# the second one from it
	if (($cc_home && !$ccpp_home) ||
	    (!$cc_home && $ccpp_home)) {
	    # deduce_one_from_another calls accept_home
	    $status = $self->deduce_one_from_another();
	}
	else {
	    # see if MWCC_HOME & MWCCPP_HOME are valid values
	    $status = $self->accept_home();
	}

	if ($self->{env_vars}{$::MWCC_HOME_ENV} ||
	    $self->{env_vars}{$::MWCCPP_HOME_ENV}) {
	    # if user defined MWCC_HOME / MWCCPP_HOME we won't change
	    # the values, even if they are not correct, we return
	    # DONE and issue a warning.
	    if (($status != $::DONE) && ! $::silent) {
		print STDERR "Warning: Compiler home is not valid.\n";
	    }
	}

	# We take what the user wrote to be true, even if we think it's
	# not.
	$self->{accepted_cc} = $::TRUE
	    if ($self->{env_vars}{$::MWCC_HOME_ENV});

	$self->{accepted_ccpp} = $::TRUE
	    if ($self->{env_vars}{$::MWCCPP_HOME_ENV});

	return $::DONE if ($self->{accepted_cc} &&
			   $self->{accepted_ccpp});

	return $::FAILED if (! $self->{accepted_cc} &&
			     ! $self->{accepted_ccpp});

	return $::PARTIALLY;
    }

# Try to deduce MWCC_HOME & MWCCPP_HOME from the location of the C / C++
# compilers.
    sub deduce_from_path
    {
	my $self = shift;

	if (! $self->{accepted_ccpp}) {
	    # look for C++ compiler in PATH
	    $ccpp_home = ::which($self->{ccpp_compiler_name});

	    # check if version of compiler we found is ok
	    if ($self->check_compiler_version($ccpp_home, "ccpp")) {
		# calculate CCPP_HOME from location of compiler
		$ccpp_home =~ /(.*)\/\Q$self->{ccpp_dir}\E\/+\Q$self->{ccpp_compiler_name}\E/;
		$ccpp_home = $1;
		# check if CCPP_HOME includes necessary path
		if ($ccpp_home =~
		    /.*\b\Q$self->{ccpp_necessary_path}\E\b/) {
		    # concatenate version path to CCPP_HOME, if needed
		    if ($ccpp_home !~ /\Q$self->{version_path}\E$/) {
			$ccpp_home .= "$self->{version_path}";
		    }
		    $self->{env_vars}{$::MWCCPP_HOME_ENV} = "$ccpp_home";
		    $set_ccpp_by_which = $::TRUE;
		}
	    }
	}

	if (! $self->{accepted_cc}) {
	    # look for C compiler in PATH
	    $cc_home = ::which($self->{cc_compiler_name});

	    # check if version of compiler we found is ok
	    if ($self->check_compiler_version($cc_home, "cc")) {

		# calculate CC_HOME from location of compiler
		$cc_home =~ /(.*)\/\Q$self->{cc_dir}\E\/+\Q$self->{cc_compiler_name}\E/;
		$cc_home = $1;
		# check if CC_HOME includes necessary path
		if ($cc_home =~ /.*\b\Q$self->{cc_necessary_path}\E\b/) {
		    # concatenate version path to CC_HOME, if needed
		    if ($cc_home !~ /\Q$self->{version_path}\E$/) {
			$cc_home .= "$self->{version_path}";
		    }
		    $self->{env_vars}{$::MWCC_HOME_ENV} = "$cc_home";
		    $set_cc_by_which = $::TRUE;
		}
	    }
	}

	# Check if values of MWCC_HOME and MWCCPP_HOME are valid
	my $status = $self->accept_home();
	if ($status == $::PARTIALLY) {
	    # If only one of MWCC_HOME, MWCCPP_HOME is valid, try
	    # to deduce one from the other. This function calls
	    # accept_home
	    $status = $self->deduce_one_from_another();
	}

	# If we found compiler in PATH, we don't have to add it's
	# location to PATH.
	$self->{set_cc_path} = $::FALSE
	    if ($set_cc_by_which && $self->{accepted_cc});
	$self->{set_ccpp_path} = $::FALSE
	    if ($set_ccpp_by_which && $self->{accepted_ccpp});

	return $status;
    }

# Take default values for MWCC_HOME / MWCCPP_HOME
    sub use_defaults
    {
	#print "in use_defaults\n";
	my $self = shift;
	if (! $self->{accepted_cc}) {
	    $self->{env_vars}{$::MWCC_HOME_ENV} =
		$self->{cc_default_home};
	}

	if (! $self->{accepted_ccpp}) {
	    $self->{env_vars}{$::MWCCPP_HOME_ENV} =
		$self->{ccpp_default_home};
	}

	# check if default values are valid.
	my $status = $self->accept_home();
	if ($status == $::PARTIALLY) {
	    # if only one default value set, try to deduce one
	    # from the other
	    $status = $self->deduce_one_from_another();
	}
	return $status;
    }

# Check if values of MWCC_HOME && MWCCPP_HOME are valid,
# and set the values of all other environment variables
# according to HOME, if the values are valid.
# Returns: DONE if both values are valid.
#          PARTIALLY if only one value is valid.
#          FAILED if none of the values is valid.
    sub accept_home
    {
	my $self = shift;
	if ($self->{accepted_cc} && $self->{accepted_ccpp}) {
	    return $::DONE;
	}
	if (! $self->{accepted_cc}) {
	    my $cc_home = $self->{env_vars}{$::MWCC_HOME_ENV};
	    if ($cc_home) {
		my $cc_include;
		my @include_list;
		# go over all include paths, and add each value to include
		# list if it is an existing directory
		foreach $include (@{$self->{cc_include}}) {
		    # full path include dir
		    if ($include =~ /^\// && -d $include) {
			push (@include_list, $include);
		    }
		    # include dir relative to MWCC_HOME
		    if ($include !~ /^\// &&
			-d "$self->{env_vars}{$::MWCC_HOME_ENV}/${include}") {
			push (@include_list,
			      "$self->{env_vars}{$::MWCC_HOME_ENV}/${include}");
		    }
		}
		$cc_include = join(":", @include_list);

		# calculate compiler location relative to home
		my $cc_comp =
		    "$cc_home/$self->{cc_dir}/$self->{cc_compiler_name}";

		# if MWCC_HOME is a direcory, and compiler is found -
		# accept C variables, and save them in hash
		if (-d $cc_home && -e $cc_comp) {
		    #print "accepted cc_home\n";
		    $self->{accepted_cc} = $::TRUE;
		    $self->{env_vars}{$::MWCC_INCLUDE_DIR_ENV} = $cc_include;
		}
		# always set MWCC_ENV and MWCC_INCLUDE_DIR_ENV according to cc_home even if it does not exist
		$self->{env_vars}{$::MWCC_ENV} = $cc_comp;

	    }
	} # end ! accepted_cc

	if (! $self->{accepted_ccpp}) {
	    my $ccpp_home = $self->{env_vars}{$::MWCCPP_HOME_ENV};
	    if ($ccpp_home) {
		my $ccpp_include;
		my @include_list;
		# go over all include paths, and add each value to include
		# list if it is an existing directory
		foreach $include (@{$self->{ccpp_include}}) {
		    # full path include dir
		    if ($include =~ /^\// && -d $include) {
			push (@include_list, $include);
		    }
		    # include dir relative to MWCCPP_HOME
		    if ($include !~ /^\// &&
			-d "$self->{env_vars}{$::MWCCPP_HOME_ENV}/${include}") {
			push (@include_list,
			      "$self->{env_vars}{$::MWCCPP_HOME_ENV}/${include}");
		    }
		}
		$ccpp_include = join(":", @include_list);
		# calculate compiler location relative to home
		my $ccpp_comp =
		    "$ccpp_home/$self->{ccpp_dir}/$self->{ccpp_compiler_name}";

		# if MWCCPP_HOME is a direcory, and compiler is found -
		# accept C++ variables, and save them in hash
		if (-d $ccpp_home && -e $ccpp_comp) {
		    $self->{accepted_ccpp} = $::TRUE;
		    #print "accepted ccpp_home\n";
	            $self->{env_vars}{$::MWCCPP_INCLUDE_DIR_ENV} = $ccpp_include;
		}
		# always set MWCCPP_ENV and MWCCPP_INCLUDE_DIR_ENV according to ccpp_home even if it does not exist
	       $self->{env_vars}{$::MWCCPP_ENV} = $ccpp_comp;
	    }
	} # end ! accepted_ccpp

	return $::DONE if ($self->{accepted_cc} && $self->{accepted_ccpp});
	return $::FAILED if (!$self->{accepted_cc} && !$self->{accepted_ccpp});
	return $::PARTIALLY;
    }

# Try to deduce MWCC_HOME from MWCCPP_HOME or vice versa
    sub deduce_one_from_another
    {
	my $self = shift;
	my $ccpp_home = $self->{env_vars}{$::MWCCPP_HOME_ENV};
	my $cc_home = $self->{env_vars}{$::MWCC_HOME_ENV};
	if ($cc_home && ! $ccpp_home) {
	    $ccpp_home = $self->deduce_ccpp_from_cc_home($cc_home);
	    $self->{env_vars}{$::MWCCPP_HOME_ENV} = $ccpp_home;
	}
	if ($ccpp_home && ! $cc_home) {
	    $cc_home = $self->deduce_cc_from_ccpp_home($ccpp_home);
	    $self->{env_vars}{$::MWCC_HOME_ENV} = $cc_home;
	}
	return $self->accept_home();
    }

# Set all calcualted values in environment
    sub set_env_vars
    {
	my $self = shift;
	foreach $env_name (keys %{$self->{env_vars}}) {
	    $::Shell->setenv ($env_name, $self->{env_vars}{$env_name})
		if ($self->{env_vars}{$env_name});
	}
    }

# Add location of compilers to PATH
    sub set_path
    {
	my $self = shift;
	# in most platform we only need to add C++ compiler to path
	if ($self->{set_ccpp_path}) {
	    my $ccpp_comp = $self->{env_vars}{$::MWCCPP_ENV};
	    my $ccpp_dir = $1 if ($ccpp_comp =~ /(.*)\/.*/);
	    $::Shell->add_to_path ("$ccpp_dir");
	}
    }

    sub check_compiler_version
    {
	return $::TRUE;
    }
    sub set_gcc_exec_prefix
    {
      # demo. this function is used only for linux and defined in linux_compiler_shell package
    }
}

# sunos5 specific compiler_setup
package sunos5_compiler_setup; @ISA = qw(compiler_setup);
{
    sub new
    {
        my $type = shift;
        my $self = compiler_setup::new ($type);
	my $cc_ver;

	$self->{ccpp_compiler_name} = "CC";
	$self->{cc_compiler_name} = "cc";

	my $instdir = $self->get_install_base();

	my ($WS8,$WS6U2) = ("${instdir}/SUNWspro/prod", "${instdir}/SUNWspro/WS6U2");

	# If WS8 or WS7 is not found or WS6U2 is found
	# - default compiler will be WS6U2 compiler
	$cc_ver = "";
        if (-d $WS8){
	    $self->{ccpp_default_home} = $WS8;
	    $self->{cc_default_home} = $WS8;
	} else {
	    $self->{ccpp_default_home} = $WS6U2;
	    $self->{cc_default_home} = $WS6U2;
	}

	$self->{ccpp_necessary_path} = "SUNWspro";
	$self->{cc_necessary_path} = "SUNWspro";

	@{$self->{ccpp_include}} = ("include/CC", "include/CC/Cstd");
	my $sun_ver = `uname -r 2>&1`;
	push (@{$self->{ccpp_include}}, "include/CC/std")
	    if ($sun_ver lt "5.8");

	@{$self->{cc_include}} = ("include/cc");

	$cc_ver = `($self->{ccpp_default_home}/bin/$self->{ccpp_compiler_name} -V) 2>&1`;
	$cc_ver =~ /C\+\+ (\S*)/;
	$cc_ver = $1;
	if ($cc_ver eq "5.0") {
	    $self->{version_path} = "/SC5.0";
	} elsif ($cc_ver eq "5.1") {
	    $self->{version_path} = "/WS6";
	} elsif ($cc_ver eq "5.2") {
	    $self->{version_path} = "/WS6U1";
	} elsif ($cc_ver eq "5.3") {
	    $self->{version_path} = "/WS6U2";
	} elsif ($cc_ver eq "5.4" || $cc_ver eq "5.5") {
	    $self->{version_path} = "/prod";
	}

	return $self;
    }

    sub get_install_base
    {
	# Find highest version installed of C compiler package
	my $cmd = "pkginfo -x 'SPROcc*' |";
	open(PKGI,$cmd) or die "Failed to execute pkginfo";
	my @pkginfo = <PKGI>;
	my $allinfo = join("@",@pkginfo);
	$allinfo =~ s/S([^\s]+)\s+.+\n@\s+\(\w+\)\s+([0-9.]+).*\n/\2 S\1/g;
	@pkginfo = sort { -($a <=> $b) } split /@/,$allinfo;
	$pkginfo[0] =~ s/.*\s//;

	my $installdir = "";
	if ($pkginfo[0]) {
	    $installdir = `pkginfo -r $pkginfo[0]`;
	    chomp $installdir;
	}
	$installdir = "/opt" unless $installdir && -x "$installdir/SUNWspro/bin/cc";
	return $installdir;
    }

    sub deduce_ccpp_from_cc_home
    {
	#print "in deduce_ccpp_from_cc_home\n";
	my ($self, $cc_home) = (shift, shift);
	my $ccpp_home  = $cc_home;

	return $ccpp_home;
    }

    sub deduce_cc_from_ccpp_home
    {
	#print "in deduce_cc_from_ccpp_home\n";
	my ($self, $ccpp_home) = (shift, shift);
	my $cc_home = $ccpp_home;

	return $cc_home;
    }

    sub set_path
    {
	my $self = shift;
	my $cc_comp = $self->{env_vars}{$::MWCC_ENV};
	my $ccpp_comp = $self->{env_vars}{$::MWCCPP_ENV};
	my $cc_home = $self->{env_vars}{$::MWCC_HOME_ENV};
	my $ccpp_home = $self->{env_vars}{$::MWCCPP_HOME_ENV};

	my $where_cc = $1 if ($cc_comp =~ /(.*)\/.*/);
	my $where_ccpp = $1 if ($ccpp_comp =~ /(.*)\/.*/);
	my $cc_parent_dir = $1 if ($cc_home =~ /(.*)\/.*/);
	my $ccpp_parent_dir = $1 if ($ccpp_home =~ /(.*)\/.*/);

	# no need to set path if we couldn't find compiler name
	if ($cc_comp && $self->{set_cc_path}) {
	    if ($cc_parent_dir ne $ccpp_parent_dir) {
		$::Shell->add_to_path ("${cc_parent_dir}/$self->{cc_dir}");
	    }
	    if ($where_cc ne $where_ccpp) {
		$::Shell->add_to_path ("$where_cc");
	    }
	}
	if ($ccpp_comp && $self->{set_ccpp_path}) {
	    $::Shell->add_to_path ("$where_ccpp", "${ccpp_parent_dir}/$self->{ccpp_dir}");
	}
    }
}

package sunos5_64_compiler_setup; @ISA = qw(sunos5_compiler_setup);
{
    sub new
    {
        my $type = shift;
        my $self = sunos5_compiler_setup::new ($type);
	return $self;
    }
}

# sunos5 specific compiler_setup
package i86_sunos5_compiler_setup; @ISA = qw(compiler_setup);
{
    sub new
    {
        my $type = shift;
        my $self = compiler_setup::new ($type);
	my $cc_ver;

	$self->{ccpp_compiler_name} = "CC";
	$self->{cc_compiler_name} = "cc";

	my $instdir = $self->get_install_base();
	my ($WS8,$WS6U2) = ("${instdir}/SUNWspro/", "${instdir}/SUNWspro/WS6U2");

	# If WS8 or WS7 is not found  or WS6U2 is found - default compiler
	# will be WS6U2 compiler
	$cc_ver = "";
        if (-d $WS8){
	  $cc_ver = `${instdir}/SUNWspro/bin/CC -V 2>&1`;
	  $cc_ver =~ /C\+\+ (\S*)/;
	  $cc_ver = $1;
	}
	if ($cc_ver eq "5.5"){
	    $self->{ccpp_default_home} = $WS8;
	    $self->{cc_default_home} = $WS8;
	}

	else {
	    $self->{ccpp_default_home} = $WS6U2;
	    $self->{cc_default_home} = $WS6U2;
	}

	$self->{ccpp_necessary_path} = "SUNWspro";
	$self->{cc_necessary_path} = "SUNWspro";

	@{$self->{ccpp_include}} = ("include/CC", "include/CC/Cstd");
	my $sun_ver = `uname -r 2>&1`;
	push (@{$self->{ccpp_include}}, "include/CC/std")
	    if ($sun_ver lt "5.8");

	@{$self->{cc_include}} = ("include/cc");

	$cc_ver = `($self->{ccpp_compiler_name} -V) 2>&1`;
	$cc_ver =~ /C\+\+ (\S*)/;
	$cc_ver = $1;
	if ($cc_ver eq "5.0") {
	    $self->{version_path} = "/SC5.0";
	} elsif ($cc_ver eq "5.1") {
	    $self->{version_path} = "/WS6";
	} elsif ($cc_ver eq "5.2") {
	    $self->{version_path} = "/WS6U1";
	} elsif ($cc_ver eq "5.3") {
	    $self->{version_path} = "/WS6U2";
	} elsif ($cc_ver eq "5.4" || $cc_ver eq "5.5") {
	    $self->{version_path} = "/prod";
	}

	return $self;
    }

    sub get_install_base
    {
	# Find highest version installed of C compiler package
	my $cmd = "pkginfo -x 'SPROcc*' |";
	open(PKGI,$cmd) or die "Failed to execute pkginfo";
	my @pkginfo = <PKGI>;
	my $allinfo = join("@",@pkginfo);
	$allinfo =~ s/S([^\s]+)\s+.+\n@\s+\(\w+\)\s+([0-9.]+).*\n/\2 S\1/g;
	@pkginfo = sort { -($a <=> $b) } split /@/,$allinfo;
	$pkginfo[0] =~ s/.*\s//;

	my $installdir = "";
	if ($pkginfo[0]) {
	    $installdir = `pkginfo -r $pkginfo[0]`;
	    $installdir = "${installdir}/SUNWspro";
	}
	$installdir = "/opt" unless $installdir && -d $installdir;
	return $installdir;
    }

    sub deduce_ccpp_from_cc_home
    {
	#print "in deduce_ccpp_from_cc_home\n";
	my ($self, $cc_home) = (shift, shift);
	my $ccpp_home  = $cc_home;

	return $ccpp_home;
    }

    sub deduce_cc_from_ccpp_home
    {
	#print "in deduce_cc_from_ccpp_home\n";
	my ($self, $ccpp_home) = (shift, shift);
	my $cc_home = $ccpp_home;

	return $cc_home;
    }

    sub set_path
    {
	my $self = shift;
	my $cc_comp = $self->{env_vars}{$::MWCC_ENV};
	my $ccpp_comp = $self->{env_vars}{$::MWCCPP_ENV};
	my $cc_home = $self->{env_vars}{$::MWCC_HOME_ENV};
	my $ccpp_home = $self->{env_vars}{$::MWCCPP_HOME_ENV};

	my $where_cc = $1 if ($cc_comp =~ /(.*)\/.*/);
	my $where_ccpp = $1 if ($ccpp_comp =~ /(.*)\/.*/);
	my $cc_parent_dir = $1 if ($cc_home =~ /(.*)\/.*/);
	my $ccpp_parent_dir = $1 if ($ccpp_home =~ /(.*)\/.*/);

	# no need to set path if we couldn't find compiler name
	if ($cc_comp && $self->{set_cc_path}) {
	    if ($cc_parent_dir ne $ccpp_parent_dir) {
		$::Shell->add_to_path ("${cc_parent_dir}/$self->{cc_dir}");
	    }
	    if ($where_cc ne $where_ccpp) {
		$::Shell->add_to_path ("$where_cc");
	    }
	}
	if ($ccpp_comp && $self->{set_ccpp_path}) {
	    $::Shell->add_to_path ("$where_ccpp", "${ccpp_parent_dir}/$self->{ccpp_dir}");
	}
    }
}

package aix4_compiler_setup; @ISA = qw(compiler_setup);
{
    sub new
    {
        my $type = shift;
        my $self = compiler_setup::new ($type);
	$self->{ccpp_compiler_name} = "xlC_r";
	$self->{cc_compiler_name} = "xlc_r";
	$self->{ccpp_default_home} = "/usr/vacpp";
	$self->{cc_default_home} = "/usr/vac";
	$self->{ccpp_necessary_path} = "vacpp";
	$self->{cc_necessary_path} = "vac";
	@{$self->{ccpp_include}} = ("include");
	@{$self->{cc_include}} = ("include");
	return $self;
    }

    sub deduce_ccpp_from_cc_home
    {
	#print "in deduce_ccpp_from_cc_home\n";
	my ($self, $cc_home) = (shift, shift);
	my $ccpp_home;

	if ($cc_home =~ /(.*)\b\Q$self->{cc_necessary_path}\E\b/) {
	    $ccpp_home = "$1$self->{ccpp_necessary_path}";
	}

	return $ccpp_home;
    }

    sub deduce_cc_from_ccpp_home
    {
	#print "in deduce_cc_from_ccpp_home\n";
	my ($self, $ccpp_home) = (shift, shift);
	my $cc_home;

	if ($ccpp_home =~ /(.*)\b\Q$self->{ccpp_necessary_path}\E\b/) {
	    $cc_home = "$1$self->{cc_necessary_path}";
	}
	return $cc_home;
    }

    sub set_env_vars
    {
       my $self = shift;
       my $compiler_version = `lslpp -Lqc vacpp.cmp.core | cut -f 3 -d :`;
       if ($compiler_version =~ /6.[\d]+.[\d]+.[\d]+/) {
       		$::Shell->setenv ("MWAIX_VA6","1");
	}
       compiler_setup::set_env_vars($self);
    }

}

package linux_compiler_setup; @ISA = qw(compiler_setup);
{
    sub new
    {
        my $type = shift;
	my $gnu_arch = shift;
        my $self = compiler_setup::new ($type);
	$self->{ccpp_compiler_name} = "g++";
	$self->{cc_compiler_name} = "gcc";

	$self->{gnu_arch} = (defined $gnu_arch ? $gnu_arch : "i686-pc-linux-gnu");
	$self->{cc_default_home} = "$ENV{$::MWHOME_ENV}/../misc/$::MWCONFIG_NAME/gcc/fixed3";
	if (-x "$self->{cc_default_home}/bin/gcc") {
	    $self->{cc_necessary_path} = "fixed3";
	    $self->{ccpp_necessary_path} = "fixed3";
	} elsif (-x "/usr/bin/gcc") {
	    $self->{cc_default_home} = "/usr";
	} elsif (-x "/usr/local/bin/gcc") {
	    $self->{cc_default_home} = "/usr/local";
	} else {
	    $self->{cc_default_home} = "/";
	}
	$self->{ccpp_default_home} = $self->{cc_default_home};

	my $default_ver = "3.2";
	my $current_ver = `($self->{cc_default_home}/bin/$self->{cc_compiler_name} -dumpversion) 2>&1`;
	if ($current_ver =~ /^([\d\.]+)$/) { $current_ver = $1 }
	else { $current_ver = $default_ver; }
	$self->{cc_version} = $self->{ccpp_version} = $current_ver;
	@{$self->{ccpp_include}} = ("include/c++/$self->{cc_version}","include/c++/$self->{cc_version}/$self->{gnu_arch}","include/c++/$self->{cc_version}/backward");
	@{$self->{cc_include}} =
	    ("lib/gcc-lib/$self->{gnu_arch}/$self->{cc_version}/include");
	return $self;
    }

    sub deduce_ccpp_from_cc_home
    {
	#print "in deduce_ccpp_from_cc_home\n";
	my ($self, $cc_home) = (shift, shift);
	my $ccpp_home  = $cc_home;

	return $ccpp_home;
    }

    sub deduce_cc_from_ccpp_home
    {
	#print "in deduce_cc_from_ccpp_home\n";
	my ($self, $ccpp_home) = (shift, shift);
	my $cc_home = $ccpp_home;

	return $cc_home;
    }

    sub check_compiler_version
    {
	my ($self, $comp_name, $comp_type) = @_;
	chomp($comp_name);
	my $ver = `(${comp_name} -dumpversion) 2>&1`;
	$ver =~ /([\d\.]+)/;
	$ver = $1;
	if ($ver lt $self->{"${comp_type}_version"}) {
	    return $::FALSE;
	}
	return $::TRUE;
    }
    sub set_env_vars
    {
       my $self = shift;
       my $libgcc = "libgcc_s.so.1";
       $self->{env_vars}{$::GCC_EXEC_PREFIX_ENV} =
               "$self->{env_vars}{$::MWCC_HOME_ENV}/lib/gcc-lib/";
       if ("$self->{env_vars}{$::MWCC_HOME_ENV}" ne "/usr") {
	   $::Shell->add_to_libpath("$self->{env_vars}{$::MWCC_HOME_ENV}/lib");
	   $::Shell->add_to_libpath("$self->{env_vars}{$::MWCC_HOME_ENV}/lib_rh7.3") if ((! -r "/lib/$libgcc") && (! -r "/usr/lib/$libgcc"));
       }
       compiler_setup::set_env_vars($self);
    }
}

package ia64_linux_compiler_setup; @ISA = qw(linux_compiler_setup);
{
    sub new
    {
        my $type = shift;
        my $self = linux_compiler_setup::new ($type, "ia64-unknown-linux");
	return $self;
    }
}

package amd64_linux_compiler_setup; @ISA = qw(linux_compiler_setup);
{
    sub new
    {
        my $type = shift;
        my $self = linux_compiler_setup::new ($type, "x86_64-redhat-linux");
	return $self;
    }
}

package s390_linux_compiler_setup; @ISA = qw(linux_compiler_setup);
{
    sub new
    {
        my $type = shift;
        my $self = linux_compiler_setup::new ($type, "s390-ibm-linux");
	return $self;
    }
}

package ux11_compiler_setup; @ISA = qw(compiler_setup);
{
    sub new
    {
        my $type = shift;
        my $self = compiler_setup::new ($type);
	$self->{ccpp_compiler_name} = "aCC";
	$self->{cc_compiler_name} = "c89";
	$self->{ccpp_default_home} = "/opt/aCC";
	$self->{cc_default_home} = "/opt/ansic";
	$self->{ccpp_necessary_path} = "aCC";
	$self->{cc_necessary_path} = "ansic";
	if (defined($::ENV{"MWHPUX_ANSI_CCPP"})) {
	    @{$self->{ccpp_include}} = ("include_std",
					"include_std/iostream_compat", "/usr");
	}
	else {
	    @{$self->{ccpp_include}} = ("include", "include/iostream", "/usr");
	}
	return $self;
    }

    sub deduce_ccpp_from_cc_home
    {
	#print "in deduce_ccpp_from_cc_home\n";
	my ($self, $cc_home) = (shift, shift);
	my $ccpp_home;

	if ($cc_home =~ /(.*)\b\Q$self->{cc_necessary_path}\E\b/) {
	    $ccpp_home = "$1$self->{ccpp_necessary_path}";
	}

	return $ccpp_home;
    }

    sub deduce_cc_from_ccpp_home
    {
	#print "in deduce_cc_from_ccpp_home\n";
	my ($self, $ccpp_home) = (shift, shift);
	my $cc_home;

	if ($ccpp_home =~ /(.*)\b\Q$self->{ccpp_necessary_path}\E\b/) {
	    $cc_home = "$1$self->{cc_necessary_path}";
	}
	return $cc_home;
    }

    sub set_path
    {
	my $self = shift;
	my ($cc_dir, $ccpp_dir);
	if ($self->{set_cc_path}) {
	    my $cc_comp = $self->{env_vars}{$::MWCC_ENV};
	    $cc_dir = $1 if ($cc_comp =~ /(.*)\/.*/);
	    $::Shell->add_to_path ("$cc_dir");
	}
	if ($self->{set_ccpp_path}) {
	    my $ccpp_comp = $self->{env_vars}{$::MWCCPP_ENV};
	    $ccpp_dir = $1 if ($ccpp_comp =~ /(.*)\/.*/);
	    $::Shell->add_to_path ("$ccpp_dir") if ($ccpp_dir ne $cc_dir);
	}
    }
}

package ia64_ux11_compiler_setup; @ISA = qw(ux11_compiler_setup);
{
    sub new
    {
        my $type = shift;
        my $self = compiler_setup::new ($type);
	$self->{ccpp_compiler_name} = "aCC";
	$self->{cc_compiler_name} = "aCC";
	$self->{ccpp_default_home} = "/opt/aCC";
	$self->{cc_default_home} = "/opt/aCC";
	$self->{ccpp_necessary_path} = "aCC";
	$self->{cc_necessary_path} = "ansic";
	#if (defined($::ENV{"MWHPUX_ANSI_CCPP"})) {
        # this is the default on itanium. need to define an environment variable for the other option
	    @{$self->{ccpp_include}} = ("include_std",
					"include_std/iostream_compat", "/usr");
	#}
	#else {
	#    @{$self->{ccpp_include}} = ("include", "include/iostream", "/usr");
	#}
	return $self;
    }
}

package main;

$MWHOME_ENV = "MWHOME";
$MWCC_HOME_ENV = "MWCC_HOME";
$MWCCPP_HOME_ENV = "MWCCPP_HOME";
$MWCC_ENV = "MWCC";
$MWCCPP_ENV = "MWCCPP";
$MWCC_INCLUDE_DIR_ENV = "MWCC_INCLUDE_DIR";
$MWCCPP_INCLUDE_DIR_ENV = "MWCCPP_INCLUDE_DIR";
$GCC_EXEC_PREFIX_ENV = "GCC_EXEC_PREFIX";
$LIB_PATH_ENV = "lib_path_name";
$TRUE = 1;
$FALSE = 0;

$DONE = 2;
$PARTIALLY = 1;
$FAILED = 0;

$MWCONFIG_NAME = $ENV{MWCONFIG_NAME};
$silent = 0;

my $os_compiler = "${MWCONFIG_NAME}_compiler_setup";
$compiler_setup = new $os_compiler();

$Shell = ($ARGV [0] eq "-csh" ? new Cshell : new Shell);

# Unset environment variables that we will define in this script
$::Shell->unsetenv($MWCC_INCLUDE_DIR_ENV, $MWCCPP_INCLUDE_DIR_ENV,
    $MWCC_ENV, $MWCCPP_ENV);

my $status;

# See if user defined cc_home and ccpp_home
if (($status = $compiler_setup->user_defined_home()) != $DONE) {

    if (($status = $compiler_setup->deduce_from_path()) != $DONE) {

	$status = $compiler_setup->use_defaults();
    }
}

if ($status == $DONE) {
    $compiler_setup->set_env_vars();
    $compiler_setup->set_path();
    $Shell->set_path_libpath();
}
elsif (! $silent) {
    $compiler_setup->set_env_vars();
    print STDERR "Warning: Failed to retrieve compiler home.\n";
}

exit(0);
