#!/bin/env mwperl
do "$ENV{MWHOME}/lib/la/Object_Module.pl";

use strict;
# Represents a shared object file. 
package SObject_Module; @SObject_Module::ISA = qw(Object_Module B_Status_Aware); {
sub new {
    my ($type, $data, $bstatus, $lib_obj) = (shift, shift, shift, shift);
    my $self = Link_Line_Atom::new($type, $data);

    $self->set_b_status($bstatus);
    # is this module needed in the final link?
    $self->{_needed} = 0;

    $self->{_depends_on} = {};
    $self->{_lib_obj} = $lib_obj;
    $self->_process_so_lib_file();

    return $self;
}

# Only add if needed and only add to the running symbol table
# not to the final module symbol table.
sub add_self {
    my ($self, $parser) = @_;
    
    if ( $self->is_system() || $self->is_empty_stripped() ) {
	$self->{_needed} = 1;
	return;
    }

    # Log apperence of library on the link line.
    $parser->saw_library( $self->stripped_name(), $self );

    # all system libs get added and we don't need to analyze them
    if( $self->is_system() || $self->is_empty_stripped() ) { return; }

    if (not $LA_Flags::Compliant) {
        # Add the libraries required by this one even if this one 
        # isn't needed, this emulates the old link analyzer and if 
        # the added libraries really aren't needed they won't be used.
        $parser->require_libs_from_pragma_comment_lib( $self->{_sym_tbl} );
        # require all the libraries we depend on.
        $parser->require_libraries( $self->dependencies() );
    }
    
    if( not $self->is_added() ) { # Need to test if we're needed.
	my @unres = $parser->get_unresolved();
	my @resolves =  $self->{_sym_tbl}->any_defined(\@unres);
	if( @resolves ) {
	    if ($linker_analyzer::Full_Trace) {
		my $stripped_name = $self->stripped_name();
		my $symbol = $resolves[0];
		print STDERR "Adding shared library $stripped_name because it resolves $symbol\n";
	    }
	    # lib is needed
	    $self->{_needed} = 1;
	}
    }

    if( $self->is_added() ) {
        if ($LA_Flags::Compliant) {
            # require all the libraries we depend on.
            $parser->require_libraries( $self->dependencies() );
        }
    
	# register as used mainwin so file.
	$parser->saw_mainwin_so($self->{_lib_obj}) if $self->{_lib_obj}->{is_mainwin};

	$self->B_Status_Aware::add_self($parser);
	$parser->merge_dynamic($self->{_sym_tbl});

	# update priority (for AIX)
	if( $linker_analyzer::Check_Priority ) {
	    my $p = $self->{_lib_obj}->{priority};
	    if( $p ne "n/a" ) {
		push(@Link_Parser::All_Priorities, $p);
	    }
	}
    }
    else {
	$parser->unneeded_lib($self);
    }
	return;
}

sub get_text {
    my $self = shift;

    if( $self->is_added() ) {
	my $ret = $self->B_Status_Aware::get_text();
	$ret .= $self->Object_Module::get_text();
	return $ret;
    }
    return "";
}

sub is_added {
    return 1 if $LA_Flags::Do_Not_Remove;
    my $self = shift;
    $self->{_needed} = 1 if $self->is_system() || $self->is_empty_stripped();
    return $self->{_needed};
}

sub flush {
    my $self = shift;
    $self->{_needed} = 0;
    return;
}

sub get_unused_defined {
    my $self = shift;
    if( $self->is_added() ) { return (); }
    return $self->{_sym_tbl}->get_defined();
}

# Explicity set that the .so file is needed
sub set_added {
    my $self = shift;
    $self->{_needed} = 1;
}

sub is_empty_stripped {
    my $self = shift;
    return ($self->{_lib_obj}->{is_stripped} && $self->{_lib_obj}->{is_empty});
}

sub is_system {
    my $self = shift;
    return $self->{_lib_obj}->{is_system};
}

sub stripped_name {
    my $self = shift;
    my $ret = $self->{_lib_obj}->{stripped_name};
    if( not $ret ) {
	if ( $self->{_data} =~ /\/lib(.*)\.(a|s[ol])$/ ) {
	    $ret = $1;
	} 
	else {
	    $ret = $self->{_data};
	    $ret =~ s/^-l//;
	}
    }
    return $ret;
}
   

# Special case for shared objects treat dependancy nodes
sub _build_sym_table {
    my $self = shift;
    my $lib_obj = $self->{_lib_obj};
    my $libname = $lib_obj->{stripped_name};

    my @defs = ();
    my @unres = ();
    foreach my $line (@_ ) {
	my ($ty, $info) = split(' ', $line);
	
	if( $ty eq 'D' ) {
	    push(@defs, $info);
	}
	elsif( $ty eq 'U' ) {
	    push(@unres, $info);
	}
	if( $info =~ /^Mw___(.*)_DependancyNode$/ ) {
	    my $orig_name = linker_analyzer::uncalc_safe_name($1);

	    if ( $ty eq "U" ) {
		# is an unresolved symbol
		# is that a .so file which this file depends on?
		my $fulllibs = linker_analyzer::locate_lib($orig_name);
		foreach my $fulllib (@$fulllibs) { 
		    # sometimes fullname can be an archive
		    if( $fulllib->{type} eq $linker_analyzer::Sharedlib_t ) {
			my $fullname = $fulllib->{stripped_name};
			$self->{_depends_on}->{$fullname} =  1;
		    }
		}
		if ( $linker_analyzer::Full_Trace ) { print STDERR "$libname depends on ==>>  $orig_name\n"; } 
		next; 
	    }
	    
	    if ( $orig_name eq $lib_obj->{"stripped_name"}) { 
		if ($linker_analyzer::Full_Trace) { print STDERR "$libname is a mainwin lib\n"; }
	    }
	    $lib_obj->{"is_mainwin"} = 1;
	}
      
    }

    # create returned object
    my $sym_tbl = Symbol_Table::new Symbol_Table;
    $sym_tbl->add_defined(\@defs);
    $sym_tbl->add_unresolved(\@unres);
    
    return $sym_tbl;
}
 
#
# Description:
# process a .lib file of an .so file
sub _process_so_lib_file
{
    my $self = shift;
    my $lib_obj = $self->{_lib_obj};

    if( $self->is_system() || $self->is_empty_stripped() ) { 
	return; 
    }

    my $libname = $lib_obj->{stripped_name};

    my($full_path) = $lib_obj->{"path"};

    my $nm_file = $lib_obj->get_nm_data_file_handle();
    # get all the information.
    my @nm_data = <$nm_file> if $nm_file;
    close($nm_file) if $nm_file;

    $self->{_sym_tbl} = $self->_build_sym_table(@nm_data);

    if ( not defined $full_path ) { 
	return;
    }

   if ( $linker_analyzer::Full_Trace ) {
	print STDERR "processing so : $libname [$full_path]\n";
	if ( $lib_obj->{"is_default"} ) { 
	    print STDERR "$libname is a default lib\n";
	}
    }

} # sub _process_so_lib_file

sub dependencies {
    my $self = shift;
    return keys %{$self->{_depends_on}};
}

} # SObject_Module

# A shared object that was added using the -l flag
package SObject_Module_L; @SObject_Module_L::ISA = qw(SObject_Module); {

sub new {
    my ($type, $flag, $file) = (shift, shift, shift);
    my $self = SObject_Module::new($type, $file, @_);
    $self->{_flag} = $flag;
    return $self;
}

sub get_text {
    my $self = shift;
    if( $self->is_added() ) {
	my $ret = $self->B_Status_Aware::get_text();

	# if the library was found via the .lib file add the directory
	# to the lib path.
	if( $self->{_lib_obj} and $self->{_lib_obj}->{add_dash_L} ) {
	    my $dir = $self->{_lib_obj}->{add_dash_L};
	    if( not $Link_Parser::added_lib_dirs{$dir} ) {
		$ret .= " -L".$self->{_lib_obj}->{add_dash_L}." ";
		$Link_Parser::added_lib_dirs{$dir} = 1;
	    }
	}

	$ret .= "-l".$self->{_flag};
	return $ret;
    }
    return "";
}

}

# This class is used to represent objects that we know are libraries but
# we don't know anything about them, therefore they behave exaclty like 
# a Link_Line_Atom but are in fact a SObject_Module
# using Object_Module in the ISA for stuff like is_added and name.
# NOTE: I put 'return' in methods that should return empty lists because
# empty functions in list context return their arguments (bug in perl).
package Corrupt_Module; @Corrupt_Module::ISA = qw(Link_Line_Atom Object_Module SObject_Module); {

sub set_added { } 

# Don't know about defined symbols
sub resolves_symbol { return; }
sub get_unused_defined { return; }

# Nope
sub is_empty_stripped {  return; }
sub is_system { return; }
sub dependencies { return; }

# get real name
sub stripped_name {
    my $self = shift;
    my $stripped = $self->name();
    $stripped =~ s/^-l//;
    return $stripped;
}


}
1;
