do "$ENV{'MWHOME'}/lib/common-utilities.pl";

package dip;
{

sub process_args
{
  my ($self, $list) = @_;
  my $i =0;
  my $target_name;

  WHILE_DIP_PARAM: 
  for($_ = $$list[$i]; $i <= $#$list; $_ = $$list[++$i]) {
  $_ = $$list[$i];
    SWITCH: {
      if(/^-modulename$/) {
        $self->{"module"} = $$list[++$i]; 
		$self->{"safemodule"} = &::calc_safe_name($self->{"module"});
		last SWITCH; 
      }

      if(/^-funcname$/)   {
# commented this out because this only happens when -ismain comes
# before -funcname and otherwise it passes and I want it to pass 
# without regard to order.
#        if ( $self->is_main($self->{"type"}) ){ $self->usage("-funcname flag can not be main"); }
        $self->{"type"} |= $self->{"TY_DLL"};
        $self->{"entry"} = $$list[++$i]; last SWITCH; 
      }

      if(/^-mwdeploy$/) {
         $application_name = $$list[++$i];
         $self->{"application_name"} = $application_name; last SWITCH; 
      }

      if(/^-o$/) {
         $target_name = $$list[++$i];
         if ($target_name =~ /.+\/.+/ ) {
             $target_name =~ /.*\/([^\/]+)/;
             $target_name = $1;
         } else {
             $target_name =~ /-o[\b]*(.+)/;
             $target_name = $1;
         }
         $self->{"exe_name"} = $target_name; last SWITCH; 
      }

      if(/^-noentry$/)    {
        $self->{"noentry"} = 1; last SWITCH; 
      }

      if(/^-embed_cs$/)   { 
        $self->{"embed_cs"} = 1; last SWITCH; 
      }

      if(/^-outdir$/)     {
        $self->{"objdir"} = $$list[++$i]; last SWITCH; 
      }

      if(/^-suffix$/)     {
        $self->{"suff"} = $$list[++$i]; last SWITCH; 
      }

      if(/^-ismain$/)     {
        if ( $self->has_wep($self->{"type"}) ) { $self->usage("-ismain flag can not have wep"); }
        $self->{"type"} |= $self->{"TY_MAIN"}; 
        $self->{"entry"} = "main"; last SWITCH;
      }
 
      if(/^-haswep$/)     {
        if ( $self->is_main($self->{"type"}) ) { $self->usage("-haswep flag can not be main"); }
        $self->{"type"} |= $self->{"TY_WEP"}; last SWITCH;
      }

      if(/^-plugin$/)     {
        $self->{"type"} |= $self->{"TY_PLUGIN"}; last SWITCH;
      }

      if(/^-dll_process_detach$/)  {
	      $self->{"dll_process_detach"} = 1; last SWITCH;
      }
    }
  }
  if ( ! $self->{"suff"} ) {
    $self->usage("no -suffix flag specified"); 
  }
  if ( ! $self->{"module"} ) { 
    $self->usage("no -modulename flag specified"); 
  }
  if ( $self->is_main($self->{"type"}) && $self->{"entry"} ne "main" ) {
    $self->usage("main must have entry=main");
  }
  if ( !$self->is_main($self->{"type"}) && $self->{"noentry"} ) {
    $self->{"entry"} = "";
  }    
}

sub gen
{
  my ($self,  $lib_names, $params, $define_raw_ptr, $raw_func) = @_;

  if ($Full_Trace) { print STDERR "Creating dip source. Libraries are @$lib_names\n"; }

  $self->process_args($params);
  $self->{"lib_names"} = $lib_names;
  $self->{"define_raw_ptr"} = $define_raw_ptr;
  $self->{"raw_func"} = $raw_func;

  $self->getDip();
}

sub getDip
{
  my ($self) = shift;
  my $fname = "$self->{objdir}/$self->{module}_init.c";
  my $line;

  open(FP, "> $fname ") || ::print_error(1, "Can't open $fname");
  print FP "/* mwdip automatically generates this file every time the App is linked. */\n\n";

# Make  sure MWROOT is defined (it isn't when using a cd)
  if( defined $ENV{MWROOT} ) {
      # if mwupdate.h exists, take it to produce full mw_version string
      my $buf = sprintf ("< %s/userx/userx/public/mainwin/inc/mwupdate.h", $ENV{"MWROOT"});
      if ( open(MWUPDATE,$buf) ) {
	  while ( $line = <MWUPDATE> ) {
	      next if  $line =~ /#include/;
	      print FP $line;
	  }
	  close MWUPDATE;
	  print FP "#define _MWUPDATE_H_\n\n\n\n";
      }

      $buf = sprintf ("< %s/userx/userx/public/mainwin/inc/mwversion.h", $ENV{"MWROOT"});
      if( open(MWVERSION, $buf) ) {
	  while ( $line = <MWVERSION> ) {
	      next if  $line =~ /#undef/;
	      print FP $line;
	  }
	  close MWVERSION;
	  print FP "\n\n\n";
      }
  }
 
  $self->mkextern(*FP);

  if ( $self->{"embed_cs"} && $self->MwIsEnvTrue("MW_EMBED_CS")) {
    $self->do_embed_cs(*FP);
  } else {
	printf FP "char **Mw___%s_keep_these[] = {0};\n",$self->{"safemodule"};
  }

  if ( $self->{"entry"} && !$self->is_main($self->{"type"}) ) {
    printf FP "EXTERN void %s(%s);\n\n", $self->{"entry"}, 
              $self->{"DLLARGS32_PROTO"};
  }
  

  # Create a node in the dependancy tree
  printf FP 		"typedef struct __DependancyNode {\n\tchar *name;\n\tstruct __DependancyNode *node;\n} DependancyNode;\n\n";
  foreach $lib (@{$self->{"lib_names"}}) {
    my $safe_lib = &::calc_safe_name($lib);
    printf FP "extern DependancyNode Mw___%s_DependancyNode[];\n", $safe_lib;
  }
  printf  FP "\n";
	
# If ISMAIN, set the root of the tree to a constant name so that kernel can find it 
  printf FP "DependancyNode Mw___%s_DependancyNode[] = {\n", 
    ($self->is_main($self->{"type"}) ? "MainWin" : $self->{"safemodule"});
  foreach $lib (@{$self->{"lib_names"}}) {
    my $safe_lib = &::calc_safe_name($lib);
    printf FP "\t{\"%s\", &Mw___%s_DependancyNode[0]},\n", $lib, $safe_lib;
  }
  printf FP 		"\t{0, 0}\n};\n\n";

  if ($self->is_plugin($self->{'type'})) {
        printf FP "DependancyNode Mw___MainWin_DependancyNode[] = {\n";
	printf FP "\t{\"%s\", &Mw___%s_DependancyNode[0]},\n", $self->{"module"}, $self->{"safemodule"};
	printf FP 		"\t{0, 0}\n};\n\n";
    }

  # End of creating node in dependancy graph
 
#
# Insert library unlocking capabilities (called from MainWin.C)
#

  if ($self->is_plugin($self->{'type'})) {
      my $safe_exe = &::calc_safe_name($self->{"exe_name"});
      $szUnlock=`mwgetdeployementlicense -n \"$self->{"exe_name"}\" \"$self->{"application_name"}\" `;
  } else {
      my $safe_exe = &::calc_safe_name($self->{"exe_name"});
      $szUnlock=`mwgetdeployementlicense \"$self->{"exe_name"}\" \"$self->{"application_name"}\" `;
  }
#
# if deployement license is unavailable, link should abort
#
  if ( $? != 0 ) {
      if ($Full_Trace) { print STDERR "No deployement license available";}
      return 0;
  }
  chop $szUnlock;
  if ( $szUnlock ) {
      if ( $self->is_main($self->{"type"}) ) {
	  printf FP "\nchar Mw__main_UnlockString[1024]=\"M\\rW\\rU\\rN\\rL\\rO\\rC\\rK\\r_\\rS\\rT\\rR\\rI\\rN\\rG\\r/\\r$szUnlock\";\n";
      } else {
	  printf FP "\nchar Mw__%s_UnlockString[1024]=\"M\\rW\\rU\\rN\\rL\\rO\\rC\\rK\\r_\\rS\\rT\\rR\\rI\\rN\\rG\\r/\\r$szUnlock\";\n",$self->{"safemodule"};
      }
  } else {
      if ( $self->is_main($self->{"type"}) ) {
	  printf FP "\nchar Mw__main_UnlockString[1]={0};\n";
      } else {
	  printf FP "\nchar Mw__%s_UnlockString[1]={0};\n",$self->{"safemodule"};
      }
  }

  # create wrapper for DllMain function
  if ( $self->is_main($self->{"type"}) ) {
    printf FP "\nEXTERN void\n%s%s%s(void* handle)",
      $self->{"DLL_PREF"}, 
      $self->{"entry"},
      $self->{"DLL_W_SUFF"};
  }
  else {
    printf FP "\nEXTERN void\n%s%s%s(%s)\n",
      $self->{"DLL_PREF"}, 
      $self->{"safemodule"},
      $self->{"DLL_W_SUFF"},
      $self->{"DLLARGS32_PROTO"};
  }

  printf FP  "{\n";

  if (! $self->is_main( $self->{"type"} ) && $self->{"entry"} ) {
    printf FP "\t%s(%s);\n", $self->{"entry"}, $self->{"DLLARGS32"};
  }

  printf FP  "}\n\n";

  if ( $self->{"define_raw_ptr"} ) {
    if ( $self->{"raw_func"} ) {
      printf FP "extern void " , $self->{"raw_func"}, "();\n";
      printf FP "void (*_pRawDllMain)() = &", $self->{"raw_func"}, ";\n";
    } 
    else {
      printf FP "void (*_pRawDllMain)() = 0;\n";
    }
  }
  close(FP);   
  return 1;
}

sub mkextern
{
  my ($self) = shift;
  local *FP = shift;

  printf FP "#ifdef __cplusplus\n%s\n#else\n%s\n#endif\n\n",
            "#define EXTERN extern \"C\"", "#define EXTERN extern";
}

sub is_main
{
  my($self, $ty) = @_;
  return $ty & $self->{"TY_MAIN"};
}

sub is_plugin
{
  my($self, $ty) = @_;
  return $ty & $self->{"TY_PLUGIN"};
}


sub has_wep
{
  my($self, $ty) = @_;
  return $ty & $self->{"TY_WEP"};
}

sub MwIsEnvTrue
{
  my ($self, $v) = @_;
  my $res = lc $ENV{$v};

  if ( ! $res ) { return 0; }
  if ( $res eq "1"  ||
       $res eq "true" ||
       $res eq "on" ||
      $res eq "yes" ) { return 1; }
  return 0;
}

sub do_embed_cs
{
  my $self = shift;
  my $decl_prefix = "static";

  if ( $::MWOS eq "irix5" || $::MWOS eq "irix53" || 
       $::MWOS eq "irix6" ) {
    $decl_prefix = "volatile";
  }

  my $buf;
  my $view_name;
  my $ccase_root = $ENV{"CLEARCASE_ROOT"};	## /view/viewname
  if ($ccase_root) {
    $buf = sprintf("cleartool catcs > /tmp/catcs%d", $$);
  } else {
    $ccase_root = $ENV{"MWROOT"} and
    $buf = sprintf("%s/buildtools/bin/getremotecs > /tmp/catcs%d",
                    $ccase_root, $$);
  }

  if ($ccase_root) {

    ## obtain config_spec
    &::ExecuteCommand($buf);
    $buf = sprintf("/tmp/catcs%d", $$);
    open(FPCS, "< $buf" ) || ::print_error (1, "Can't open $buf");

    ## obtain view name
    $_ = $ccase_root;
    s/^\/view\///;
    s/\/.*//;
    $view_name = $_;

  } else {
    $view_name = "*UNKNOWN*";
  }

  ## obtain current time
  my $time_buf = scalar gmtime(time());

  ## obtain user name
  my $user_id = `id`;
  $user_id =~ /=(\d+)/;
  $user_id = $1;
  my @user_info = getpwuid($user_id);
  my $user_name = $user_info[0];
  if ( ! $user_name ) { 
    $user_name = "*UNKNOWN*";
  }

  ## obtain hostname
  my $host_name = `hostname`; chomp($host_name);
  if ( ! $host_name ) {
    $host_name = "*UNKNOWN*";
  }

  ## embedding data
  printf FP "#define MWVER(_B) \"MW_RELEASE_VERSION: \"  _B\n";
  printf FP "%s char *mw_release_version_%s = MWVER(B);\n",
            $decl_prefix, $self->{"safemodule"};
  printf FP "%s char *mw_link_time_%s = \"MW_LINK_TIME: %s GMT\";\n", 
            $decl_prefix, $self->{"safemodule"}, $time_buf;
  printf FP "%s char *mw_user_name_%s = \"MW_USER_NAME: %s\";\n", 
	    $decl_prefix, $self->{"safemodule"}, $user_name;
  printf FP "%s char *mw_host_name_%s = \"MW_HOST_NAME: %s\";\n", 
	    $decl_prefix, $self->{"safemodule"}, $host_name;
  printf FP "%s char *mw_view_name_%s = \"MW_VIEW_NAME: %s\";\n", 
	    $decl_prefix, $self->{"safemodule"}, $view_name;


  ## embedding config_spec
  if ($ccase_root) {
    printf FP "%s char *mw_link_confspec_%s =\"MW_LINK_CONFIG_SPEC: ", 
	      $decl_prefix, $self->{"safemodule"};
    #  print the config spec, on one line 
    my $line;
    while ( $line = <FPCS> ) {
      chomp($line);
      $line =~ s/\"/\\\"/g;
      print FP "$line!";
    }
    printf FP "\";\n";
    close(FPCS);
    unlink($buf);
  	printf FP "char **Mw___%s_keep_these [] = { &mw_release_version_%s, &mw_link_time_%s, &mw_user_name_%s, &mw_host_name_%s, &mw_view_name_%s, &mw_link_confspec_%s };\n",
		$self->{"safemodule"}, $self->{"safemodule"}, $self->{"safemodule"}, $self->{"safemodule"}, $self->{"safemodule"}, $self->{"safemodule"}, $self->{"safemodule"};
  } else {
  	printf FP "char **Mw___%s_keep_these [] = { &mw_release_version_%s, &mw_link_time_%s, &mw_user_name_%s, &mw_host_name_%s, &mw_view_name_%s };\n",
		$self->{"safemodule"}, $self->{"safemodule"}, $self->{"safemodule"}, $self->{"safemodule"}, $self->{"safemodule"}, $self->{"safemodule"};
	}
}

sub usage
{
  my ($self, $msg) = (shift, shift);
  ::print_error (1, "dip: $msg");
}

sub new
{
  my ($class) = @_;
  my $self = bless {}, $class;

  $Full_Trace = (lc($ENV{MWTOOL_TRACE}) eq "full");
  $Partial_Trace = (defined $ENV{MWTOOL_TRACE} 
		    && ($ENV{MWTOOL_TRACE} ne "FALSE"));

  $self->{"TY_MAIN"} = 1;
  $self->{"TY_DLL"} = 2;
  $self->{"TY_WEP"} = 4;
  $self->{"TY_PLUGIN"} = 8;
  $self->{"DLLARGS32"} = "handle, reason, reserved";
  $self->{"DLLARGS32_PROTO"} = "void* handle, unsigned int reason, void* reserved";
  $self->{"DLL_PROCESS_DETACH"} = 0;
  $self->{"DLL_PROCESS_ATTACH"} = 1;
  $self->{"DLL_THREAD_ATTACH"} = 2;
  $self->{"DLL_THREAD_DETACH"} = 3;
  $self->{"DLL_PREF"} = "Mw___";
  $self->{"DLL_W_SUFF"} = "_wrapDllMain";

  $self->{"entry"} = "";   # funcname arg
  $self->{"suff"} = "";    # suffix arg
  $self->{"objdir"} = "."; #outdir arg
  $self->{"type"} = 0;    # funcname,ismain,haswep
  $self->{"noentry"} = 0;  # noentry arg
  $self->{"module"} = "";  # modulename arg
  $self->{"embed_cs"} = 0; # embed_cs arg
  $self->{"ismain"} = 0;   # ismain arg
  $self->{"haswep"} = 0;   # haswep arg
  $self->{"dll_process_detach"} = 0; # dll_process_detach arg
  $self->{"exe_name"} = main; # target file name
  $self->{"application_name"} = ""; #used for deployement

  return $self;
}
} 
