
package lib;
{
#    sub new; constructor
#    sub process; public
#    sub is_valid; private
#    sub create; private
#    sub process_header; private


#
# Description:
# checks if lib file is the correct one .This is done by comparing the 
# mtime of the original lib to the mtime of specified in the lib header 
#
# Parameters:

# Side effects:
#  none
# Return:
#   true if lib is ok, false otherwise 
sub is_valid
{
    my ($self)  = @_;

    if ( $self->{"do_not_validate"} ) { return $linker_analyzer::True; }
    if ( $self->{"is_stripped"} ) { return $linker_analyzer::True; }

    if ($self->{"timestamp"} ne "") {
	my($dev, $ino, $mode, $nlink, $uid, $gid, $rdev, 
	   $size, $atime, $mtime, $ctime, $blksize, $blocks) 
	    = lstat $self->{"source"};
	$self->{"source_timestamp"} = $mtime;
	if ( $mtime eq "" ) {
	    &linker_analyzer::la_warning("failed stat $self->{source}");
	    return $linker_analyzer::False;
	}
	return $linker_analyzer::True if ($self->{"timestamp"} >= $mtime);
	if ( $linker_analyzer::Full_Trace ) {
	    print STDERR "Timestamp in lib file is $self->{timestamp}\n";
	    print STDERR "Calculated checksum is $mtime\n";
	    print STDERR "Timestamp does not match: we will check sum\n";
	}
    } else {
	if ( $linker_analyzer::Full_Trace ) {
	    print STDERR "Lib file does not have timestamp.\n";
	}
	
    }

    if ($self->{"checksum"} eq "") {
	if ( $linker_analyzer::Full_Trace ) {
	    print STDERR "Lib file does not have sum. It is not valid\n";
	}
	return linker_analyzer::False;
    }

    my $cmd = "sum $self->{source} | ";
    open ( SUM , $cmd) || mwmklib_error("command failed: $cmd");
    my ($line, @words,@word2);
    $line = <SUM>;
    chomp ($line);
    @words = split(/\s+/,$line);
    @words2 = split(/,/,$self->{"checksum"});

    my $valid = ( $words[0] == $words2[0] && $words[1] == $words2[1]);
    if ( $linker_analyzer::Full_Trace ) {
	print STDERR "checksum in lib file is $self->{checksum}\n";
	print STDERR "Calculated checksum is $words[0] $words[1]\n";
	print STDERR "valid = $valid \n";
    }

    return $valid;

}

#
# Description:
# this is only a required behavior, and this function should be changed to
# reflect it!!!
#
# $Mw_Libs should be by platforms !!! because of (C)
#
#
# Description:
# locate the .so (.a) by @linker_analyzer::Lib_Path .
#   if found .so ? 
#      locate .lib file in the following order: 
#         near the .so
#         by @linker_analyzer::Lib_Path
#      if found .lib ?
#         (A)
#         we have .so + .lib - go and process the header (don't need to 
#         search the .so in process_header())
#      else
#         if is system .so ?
#           mark as system, and pass to ld
#         else
#           try to generate lib on the fly
#           if .lib was generated successfully ?
#             if lib is stripped ?
#                search .lib in $Mw_Libs directory
#                if found ?
#                   take that .lib, and goto (A)
#                else
#                   display a message that lib is stripped and 
#                   -) pass to ld
#             else
#                same as (A)
#           else
#             display message and pass to ld
#   else 
#     search .lib by @linker_analyzer::Lib_Path
#     if found .lib?
#        (B)
#        procss_header() (try to find .so by the header of the .lib)
#        if .so found ?   
#          add the .so dir as -L to @linker_analyzer::Lib_Path
#          -) trace a warning that LD_LIBRARY_PATH should be updated
#          -) with this directory as well
#        else
#          we have a .lib but don't have .so: use the .lib to find
#          -) dependencies as if the .so was found. If this .so is 
#          -) needed ld will shout.
#     else
#       if is system .so ?
#          mark as system, and pass to ld
#       else
#          if .lib is in $Mw_Libs directory ?
#            (C)
#            use that .lib. goto (B)
#          else
#            pass to ld
#
#
# Parameters:
#   
# Side effects:
#  see process_header()
#
# Return:
#   full path name or "" if none exists
#

sub process
{
    my ($self) = @_;
    my ($sofile, $afile, $file, $dir, $direc, $alt_lib );
    my $path, $base, $ext,$susp_lib;

    # Set the path we will acutally look in. If we got a syntax ...lib##.a then
    # don't search in all paths.
    my @lib_path;
    if ($self->{path_received_on_cmd}) {
	@lib_path = ("$self->{directory_name}");
    } else {
	foreach $dir (@linker_analyzer::Lib_Path) {
	    $dir = "$dir/" if ($dir !~ /\/$/);
	    push (@lib_path, "$dir");
	}
    }

    # let's check if .lib is locatd in $Mw_Libs
    $alt_lib = &::flocate
	("$linker_analyzer::Mw_Libs/$self->{stripped_name}.$linker_analyzer::LIB_Extension",
	 "quiet",
	 $linker_analyzer::Vmw && ! $self->{"is_mapped"});
    
    # .lib file is not found in Lib_Path, let's locate the .so (.a)
    foreach $dir ( @lib_path ) {

	# The priority order is as follows:
	# 1. Is there a local lib file ? use it.
	# 2. Is there a local binary ? In dynamic mode a local binary can be a so 
	#    (preferred) or an archive. In static mode this can be only an archive.
        #    If not, go to next directory.
	# 3. Is there a lib file in Mw_Libs ? If so, prefer this lib file assuming it
	#    covers the current binary.
	# 4. Use the local binary.
    	
	$susp_lib = "";

	# case 1. look for local lib file
	$self->{"path"} = &::flocate
	    ("${dir}$self->{stripped_name}.$linker_analyzer::LIB_Extension",
	     "quiet",
	     $linker_analyzer::Vmw && ! $self->{"is_mapped"});
	if ($self->{"path"} ne "") { 

            # we don't want to process system libs, even the .lib exists
	    &::split_path($self->{"path"}, \$path, \$base, \$ext);
	    if ( &linker_analyzer::is_system_dir($path) ) {
		$self->{"path"} = "";
		last;
	    }

	    if ( $self->process_header($linker_analyzer::True, @lib_path) ) {
		if ($self->{"type"} eq $linker_analyzer::Sharedlib_t &&
		    $self->{"mode"} eq "static") 
		    {
		    # This lib file does not match, look for a shared object
		    $self->{"path"} = "";
		} elsif ($self->{"type"} eq $linker_analyzer::Archive_t &&
		    $self->{"mode"} eq "dynamic")
		    {
    	    	    #first try to find shared lib: if no - we will revert to lib found"
		    $susp_lib = $self->{"path"}; 
		} else {
		    return $self->{"path"};
		}
	    }
	}

	# case 2. look for local binary. Continue of none found
	if ((defined($self->{"path_received_on_cmd"}) and 
	     $self->{"path_received_on_cmd"} eq "yes") || 
	    $self->{"mode"} eq "static") {
	    # either recieved full path of .a file, or received request not to accept
	    # shared objects. Don't look for so.
	    $sofile = "";
	} else {
	    $sofile   = &::flocate
		("${dir}$self->{stripped_pfx}$self->{stripped_name}.$::lib_suffix", "quiet",
		 $linker_analyzer::Vmw && ! $self->{"is_mapped"});
	}
	$file     = "";
	
	if    ( $sofile ) { 
	    $file = $sofile; $self->{"type"} = $linker_analyzer::Sharedlib_t; 
	} else {
	    #check if some lib was found on previose stage
	    if ( $susp_lib ne "" )
	    {
	    	return $self->{"path"};
	    }
	    $afile = &::flocate
		("${dir}$self->{stripped_pfx}$self->{stripped_name}.a", "quiet",
		 $linker_analyzer::Vmw && ! $self->{"is_mapped"} );
	    if ( $afile  ) {
		$file = $afile; $self->{"type"} = $linker_analyzer::Archive_t;
	    } else  { 
		next;
	    }
	}

	$self->{"source"} = $file;

        # set the correct $self->{"stripped_name"},
	# according to the found library
        &::split_path($file, \$path, \$base, \$ext);
	$base =~ /^$self->{"stripped_pfx"}/;
	$self->{"stripped_name"} = $';

	# case 3. Prefer lib from Mw_Libs
	$self->{"path"} = $alt_lib;
	if ( $self->{"path"} ne "" ) {
	    # we don't want to validate timestamps if the .lib is taken
	    # from $Mw_Libs
	    $self->{"do_not_validate"} = $linker_analyzer::True;
	}


	# handle system libs
	if ( &linker_analyzer::is_system_dir($dir) ) {
	    if ( $linker_analyzer::Full_Trace ) {
		print STDERR $self->{"stripped_name"}, " is a system lib\n";
	    }

	    if ( $self->{"path"} ne "" ) {
		$self->process_header($linker_analyzer::False, @lib_path);
		return $self->{"path"};
	    }
	    else {
		$self->{"is_system"} = $linker_analyzer::True;
		return $file;
	    }
	}

	# if no alternate .lib -> create one on the fly
	if ( $self->{"path"} eq "" ) {
	    $self->{"path"} = $self->{"source"};
	    if ( $linker_analyzer::Full_Trace ) {
		print STDERR 
		    "$self->{source}: creating .lib on the fly.\n";
	    }
            # Don't regenerate if created from def file
            if ( $self->{"created_from_def_file"} ) { return ""; }
	    if ( ! $self->create() ) { return ""; }
	}
	elsif ( $linker_analyzer::Full_Trace ) {
	    print STDERR
		"$self->{source}: using .lib from $linker_analyzer::Mw_Libs instead of creating one on the fly\n";
	}
	
	# in this point the .lib file is taken from $Mw_Libs or
	# created on the fly
	$self->process_header($linker_analyzer::True, @lib_path);
	return $self->{"path"};
    }

    # .so & .lib where not found. 
    # as a last resort try to locate the .lib file in $Mw_Libs
    $self->{"do_not_validate"} = $linker_analyzer::True;
    if ( $alt_lib ne "" ) {
	# use the .lib from $Mw_Libs 
	$self->{"path"} = $alt_lib;
	if ( $linker_analyzer::Full_Trace ) {
	    print STDERR "taking .lib of ", $self->{"stripped_name"},
	    " from $alt_lib. .so was not found\n";
	}
	$self->process_header($linker_analyzer::False, @lib_path);
	return $self->{"path"};
    }

    # no .so, .lib or alternate lib
    $self->{"path"} = "";
    return "";
}

sub create
{
    my ($self) = @_;
    my $path, $base, $ext;
    my $lext = lc($linker_analyzer::LIB_Extension);

    if ( $self->{"source"} eq "" ) { return $linker_analyzer::False; }
    if ( $self->{"path"} eq "" ) { $self->{"path"} = $self->{"source"}; }
    if ( lc($self->{"path"}) !~ /\.$lext$/ ) {
	&::split_path($self->{"source"}, \$path, \$base, \$ext);
	$base =~ /^lib/;
	$self->{"path"} = "$path/$'.$linker_analyzer::LIB_Extension";
    }
    # We already created this lib file in /tmp
    if( exists $linker_analyzer::Tmp_libs{$self->{stripped_name}} ) {
	my $path = $linker_analyzer::Tmp_libs{$self->{stripped_name}};
	if( $path ) { # safe side, make sure it's not empty
	  linker_analyzer::la_warning("Using previously created tmp lib file $path for ".$self->{stripped_name}.".\n") if $linker_analyzer::Full_Trace;
	    $self->{path} = $path;
	    return $linker_analyzer::True;
	}
    }

    my @cmd = ("mwmklib", "-i$self->{source}",
	       "-o$self->{path}", "-noerrors");

    if ( $linker_analyzer::Partial_Trace ) { push (@cmd, "-noquiet"); }
    
    if ($ENV{MWROOT} && !$linker_analyzer::NoRegenLibs)
    {
      linker_analyzer::la_warning("Auto generating lib file at ".$self->{path}.".\n");
    }
    if ( $linker_analyzer::NoRegenLibs ||
         &::SilentExecuteCommandNoExit(@cmd) != 0 ) {
	if ( $linker_analyzer::Partial_Trace ) {
	    &linker_analyzer::la_warning("can't create lib file: $self->{path} - using tmp file");
	}
	$temp_file_counter++;
	$self->{"path"} =  "$linker_analyzer::LA_Tmp_Dir/la_tmp_${$}_${temp_file_counter}.$linker_analyzer::LIB_Extension";
	@cmd = ("mwmklib", "-i$self->{source}",
		"-o$self->{path}", "-noerrors");
	if ( $linker_analyzer::Partial_Trace ) { push (@cmd, "-noquiet"); }
	    if ( &::SilentExecuteCommandNoExit(@cmd) != 0 ) {
		&linker_analyzer::la_warning("can't create tmp lib file: $self->{path}");
		return $linker_analyzer::False;
	    }
	$linker_analyzer::Tmp_libs{$self->{"stripped_name"}} = $self->{"path"};
    }
    return $linker_analyzer::True;
}


#
# Description:
#  process information in the lib header, check version and regenrate if necessary
#
# Parameters:
#   
# Side effects:
#  update $True_lib_name{libname} = the full path name of the original lib file
#  update $LIB_Priority{libname} = priority of the original lib file
#
# Return:
#   True if lib file is ok, false otherwise
#
sub process_header
{
    my ($self, $search_orig_lib, @lib_path) = @_;
    my $path, $base, $ext, $dir, $file, $fpath;
    open (HEADER, $self->{"path"}) or &linker_analyzer::la_error("can't open lib file: $self->{path}");
    
    if ( $linker_analyzer::Full_Trace ) {
	print STDERR "Processing header of $self->{path}\n";
    }
    my $line = <HEADER>; # title
    $line = <HEADER>; $line =~ /:\s*(.*)/; $self->{"version"} = $1;
    if ( substr($line, 0, 15) ne "Header Version:" ) {
	if ( $linker_analyzer::Partial_Trace ) {
	    &linker_analyzer::la_warning("lib: $self->{path}. Corrupted header - regenerating");
	}
        # process() will try to locate the original lib by LD_LIBRARY_PATH, and to 
        # regenerate the lib file.
	close HEADER;
	return $linker_analyzer::False;

    }

    if ( $self->{"version"} > $linker_analyzer::LIB_Header_Version ) {
	if ( $linker_analyzer::Partial_Trace ) {
	    &linker_analyzer::la_warning("lib: $self->{path}. illegal version of lib file: $self->{version} - regenerating");
	}
	$line = <HEADER>; chomp($line); # source lib
	my @fields = split (/\s+/, $line);
	close HEADER;
	if ($fields[0] ne "Source:") { 
	    if ( $linker_analyzer::Partial_Trace ) {
		&linker_analyzer::la_warning("lib: $self->{path}. Corrupted header - regenerating");
	    }
	    # process() will try to locate the original lib by LD_LIBRARY_PATH, and to 
	    # regenerate the lib file.
	    return $linker_analyzer::False;
	}

	my $f = &::flocate($fields[1], "quiet", 
			   $linker_analyzer::Vmw && ! $self->{"is_mapped"});
	if ( $f eq "" ) {
	    # process() will try to locate the original lib by LD_LIBRARY_PATH, and to 
	    # regenerate the lib file.
	    return $linker_analyzer::False;
	}

	$self->{"source"} = $f;
	$self->create();
	return $self->process_header($search_orig_lib);
    }


    $line = <HEADER>; chomp($line); # source lib
    my @fields = split (/\s+/, $line);
    if ($fields[0] ne "Source:") { 
	if ( $linker_analyzer::Partial_Trace ) {
	    &linker_analyzer::la_warning("lib: $self->{path}. Corrupted header - regenerating");
	}
        # process() will try to locate the original lib by LD_LIBRARY_PATH, and to 
        # regenerate the lib file.
	close HEADER;
	return $linker_analyzer::False;

    }

    if ( $search_orig_lib) {
	# find binary library in lib_path. if not found check the header path
	$self->{"source"} = "";
	$fpath = $fields[1];
	&::split_path($fpath, \$path, \$base, \$ext);
	foreach $dir ( @lib_path ) {
	    $self->{"source"} = &::flocate
		("${dir}$self->{stripped_pfx}$self->{stripped_name}.$ext", "quiet",
		 $linker_analyzer::Vmw && ! $self->{"is_mapped"});
	    if ( $self->{"source"} ne "" ) { last; }
	}
	if ( $self->{"source"} eq "" ) {
	    $self->{"source"} = 
		&::flocate($fpath, "quiet", 
			   $linker_analyzer::Vmw && ! $self->{"is_mapped"});
	}
	if ( $self->{"source"} eq "" ) {
	    if ( $linker_analyzer::Partial_Trace ) {
		&linker_analyzer::la_warning("could not locate $self->{stripped_pfx}$self->{stripped_name}.$ext or $fpath");
	    }
	    close HEADER;
	    return $linker_analyzer::False;
	}
    

	# take true stripped name from the located original library
	&::split_path($self->{"source"}, \$path, \$base, \$ext);
	if ( $base =~ /^lib/ ) { $base = $'; }
	if ( $linker_analyzer::Full_Trace && $self->{"stripped_name"} ne $base ) {
	    print STDERR "changing  $self->{stripped_name} to $base\n";
	}
	$self->{"stripped_name"} = $base;
    }
    
    if ((not defined $self->{path_received_on_cmd}) or
	$self->{path_received_on_cmd} ne "yes") {
	$self->{add_dash_L} = $path if $path !~ /^\s*$/;
    }
    &::list_make_unique(\@linker_analyzer::Lib_Path);

    $line = <HEADER>;	chomp($line); # type
    @fields = split (/\s+/, $line);
    if    ( $fields[1] eq "archive" ) { 
	$self->{"type"} = $linker_analyzer::Archive_t; 
    }
    elsif ( $fields[1] eq "shared"  ) { 
	$self->{"type"} = $linker_analyzer::Sharedlib_t;
    }
    else                              { 
	$self->{"type"} = ""; 
    }
    $line = <HEADER>; chomp($line); # checksum
    if ($line =~ /^Time stamp of original lib was/) {
	$line =~ /\[(.*)\]/; $self->{"timestamp"} = $1;
	$line = <HEADER>; chomp($line);
    }
    if ($line =~ /^Sum of original lib was/) {
	$line =~ /\[(.*)\]/; $self->{"checksum"} = $1;
	$line = <HEADER>; chomp($line); # checksum
    }

    $line = <HEADER>; # created from def file
    if( $line =~ /CreatedFromDefFile:\s1/ ) {
        $self->{"created_from_def_file"} = $linker_analyzer::True;
    }
    $line = <HEADER>; # priority
    $line =~ /:\s*(.*)/; 
    $self->{"priority"} = $1;

    $line = <HEADER>; # is mainwin lib?
    @fields = split (/\s+/, $line);
    if ( defined $fields[2] && $fields[2] eq "yes" ) { 
	$self->{"is_mainwin"} = $linker_analyzer::True; 
    }
    if ( defined $fields[3] and $fields[3] eq "(stripped)" ) { 
	$self->{"is_stripped"} = $linker_analyzer::True; 
	$self->{"do_not_validate"} = $linker_analyzer::True;

	# try to read past header, to see if .lib has symbols,
	# cause, maybe the symbols were manually added
	$self->{"is_empty"} = ( <HEADER> =~ /^\s*$/ );
	if ( $linker_analyzer::Full_Trace ) {
	    print STDERR $self->{"source"}, " is a stripped lib.\n";
	}
	# try to locate the .lib in $Mw_Libs directory
	my $alt_lib = &::flocate
	    ("$linker_analyzer::Mw_Libs/$self->{stripped_name}.$linker_analyzer::LIB_Extension",
	     "quiet",
	     $linker_analyzer::Vmw && ! $self->{"is_mapped"});
	
	if ( $alt_lib ne "" && $alt_lib ne $self->{"path"} ) {
	    $self->{"path"} = $alt_lib;
	    $self->{"is_empty"} = $linker_analyzer::False;
	    $self->{"is_stripped"} =  $linker_analyzer::False;
	    if ( $linker_analyzer::Full_Trace ) {
		print STDERR "$self->{source}: taking .lib from $alt_lib\n";
	    }
	    return $self->process_header($search_orig_lib);
	}
	    
	if ( $self->{"is_empty"} ) { 
	    print STDERR "$self->{path} is empty => passed to ld as is\n"; }
	else { print STDERR "\n"; }
    }

    close HEADER;

    if (  ! $self->is_valid() ) {
	if ( $linker_analyzer::Partial_Trace ) {
	    &linker_analyzer::la_warning("lib: $self->{path}, dll: $self->{source}. Timestamp mismatch [h=$self->{timestamp} f=$self->{source_timestamp}]");
	}
	# Dont call $self->create() if .lib was created from def file
	if ( $self->{"created_from_def_file"} ) {
		&linker_analyzer::la_warning("$self->{source} is out of date. Please rebuild this DLL. Until you do so, the binary you are currently building may not link correctly !!");
		return $linker_analyzer::True; # We will use it anyway.
	}
	$self->create();
	return $self->process_header($search_orig_lib);
    }
    return $linker_analyzer::True;
}


sub new
{
    my ($class) = shift;
    my $self = bless {}, $class;
    $self->{"received_name"} = shift;
    if ($self->{"received_name"} =~ /(^|.*\/)(lib|)([^\/]+)\.(a|s[ol])/) {
	$self->{"path_received_on_cmd"} = "yes";
	$self->{"directory_name"} = $1;
        $self->{"stripped_pfx"} = $2;
	$self->{"stripped_name"} = $3;
	if ($self->{"directory_name"} eq "") {
	    $self->{"directory_name"} = `pwd`;
	    chomp($self->{"directory_name"});
	}
	$self->{"directory_name"} = "$self->{directory_name}/" 
	    if ($self->{"directory_name"} !~ /\/$/);
    } else {
	$self->{"stripped_pfx"} = "lib";
	$self->{"stripped_name"} = $self->{"received_name"};
	$self->{"directory_name"} = "";
    }
    $self->{"is_mapped"} = shift;
    $self->{"mode"} = shift;
    $self->{"mode"} = "dynamic" if ($self->{"mode"} eq ""); # provide default

    $self->{"is_mainwin"} = $linker_analyzer::False;
    $self->{"path"} = ""; # path of the .lib file
    $self->{"source"} = ""; # path of the original lib
    $self->{"timestamp"} = "";
    $self->{"checksum"} = "";
    $self->{"source_timestamp"} = "";
    $self->{"type"} = "";
    $self->{"is_system"} = $linker_analyzer::False;
    $self->{"version"} = 0;
    $self->{"priority"} = "n/a";
    $self->{"is_stripped"} = $linker_analyzer::False;
    $self->{"is_empty"} = $linker_analyzer::False;
    $self->{"do_not_validate"} = $linker_analyzer::False;
    $self->{"created_from_def_file"} = $linker_analyzer::False;
    my %h = %linker_analyzer::Def_Lib;
    if ( defined $h{$self->{"stripped_name"}} ) {
         $self->{"is_default"} =  $linker_analyzer::True;
    }
    else {
        $self->{"is_default"} = $linker_analyzer::False;
    }
    return $self;
}

# Return a file handle to the data section of the lib file
sub get_nm_data_file_handle {
    my $self = shift;

    if( not $self->{path} or $self->{path} eq $self->{source} ) { return undef; } # No .lib file, fend for yourself.

    open (LIBFILE, $self->{path}) or
	&linker_analyzer::la_error("Can't open lib file $self->{path}: $!");
    
    my $l; # discard header
    $l = <LIBFILE>; # title
    $l = <LIBFILE>; # header version
    $l = <LIBFILE>; # source lib
    $l = <LIBFILE>; # lib type
    $l = <LIBFILE>; chomp($l); # lib timestamp
    if ($l =~ /^Time stamp of original lib was/) {
	$l = <LIBFILE>; chomp($l); #either sum, or nm flag
    }
    if ($l =~ /^Sum of original lib was/) {
	$l = <LIBFILE>; chomp($l); #nm flag
    }
    $l = <LIBFILE>; # created from def or not
    $l = <LIBFILE>; # priority 
    $l = <LIBFILE>; # mainwin lib flag
    return *LIBFILE;
}


# static members
$temp_file_counter = 0;

} # end of package lib;

