
$LA_Dir = "$ENV{'MWHOME'}/lib/la/";
do "$LA_Dir/$ENV{MWOS}/module.pl";
do "$LA_Dir/pragma.pl";
do "$LA_Dir/utils.pl";
do "$LA_Dir/lib.pl";
do "$LA_Dir/mwmklib.pl";
do "$ENV{'MWHOME'}/XDE/lib/vs-handlers.pl";
do "$ENV{'MWHOME'}/XDE/lib/errors.pl";
do "$::MWHOME/XDE/lib/libmap.pl";    
do "$LA_Dir/$ENV{MWOS}/flocate.pl";
do "$LA_Dir/mapfile.pl";
# Link_Parser includes all needed files
do "$LA_Dir/Link_Parser.pl";

package linker_analyzer;
{
 


#######################
### package structures
######################

# SO_Needed is used for topological sorting. Maybe remove after implementing.
# %SO_Needed: hash of needed .so files (including system libs). key: stripped name.
#              val:dont care
# A_Files is used by locate_lib and process_lib
# %A_Files: key: .a file stripped name (C) , val: full path name /usr/lib/libC.a (or .lib)
#           The val is set according to the process of finding a lib file by LD_LIBRARY_PATH
#           + -L flags
# SO_Files is used in locate_lib and process_lib
# %SO_Files: key = .so(.sl) file stripped name. val: see %A_Files
# SO_Depends is used for topological sorting.
# %SO_Depends: key: a .so name val: a list of .so files that 
#              the "key" depends on
#              Used for the topoligical sort. Updated while processing a .so file
# SO_Sorted is used in topoligical order
# @SO_Sorted: a topological sort of SO_Needed (using %SO_Depends)
#  @Lib_Path - the effective lib path. take into account the 
#     -L & $LD_LIBRARY_PATH (or SHLIB_PATH or LPATH)
#  $Building_So - in aix .so files we should handle the priorities.
#  $Target = the path after the -o option
#  $LIB_Extension
#  $Archive_t = "a";
#  $Sharedlib_t = "sl";
#  %System_Dirs : key = system dir, val=dont care. any library which is located in a system
#                 dir is taken as is into the command line
#  $Check_Priority = true if platform is aix and building an so file
#  $Priority = priority for mwdip (put in -p and also returned from main, for the 
#              convinience of mwdip)
# Unknown_Libs is used in locate_lib
#  %Unknown_Libs - libs which couldn't be found. passed to result as is.
# Tmp_libs is used by lib.pl
#  %Tmp_libs key=stripped lib name, val=tmp lib file
# Corrupted_Libs is used by locate lib
#  %Corrupted_Libs key = a stripped lib name which it's header was found corrupted, val = ""                     used for not reprocessing corrupted ones
# $Use_Libmap - if True, instructs la to map libraries through libmap.pl. This is by default
#   False, cause MainWin developers which build kernel32 & usergdi32  wouldn't want
#  the mapping. (things like kernel32->usergdi32 are not wanted unless working with nmake)
# $Implib - equivalent to -implib: flag of microsoft linker. Specifies the name of the .lib file to be created.
# %Common_Syms - key: symbol name, val: n/a. contains the COMMON symbols (sun only)
# $Nodef_Libs = true if -nodeflibs is specified
# %Nodef_Lib key: lib specified in -nodeflib: switch. value: n/a
# %Def_Lib key: lib specified in -deflib: switch. value: n/a
# $Local_Mapfile  - mapfile that contains LOCAL symbols to hide
#   This map file is assumed to have the first word in each line as the hidden
#   symbol . this is a temporary solution to aix symbol - hiding problem. A 
#   complete solution will be when map files will be identified by la.
# $Use_Priority = true if to use -p, false otherwise

###############################
######## sub routines #########
###############################

sub create_topological_order
{
    # this is very important for linux.i add to command line
    # all the libs (direct & indirect), which current lib
    # depends on, even if they are not needed by needed .o or .a.
    # If for example, this module does not need Registry APIs,
    # it doesn't use advapi directly. But if advapi does not appear
    # in the command line, linux will initialize it BEFORE all
    # libs in the command line, ie: before kernel32 - WHOOPS!!

    # The most efficient way to do this is to sort topologically, and then
    # start from the root of the tree and propagete the needed flag down the
    # dag. This is less efficient because it is O(n^2).
    my $changed = 1;
    while ($changed) {
	$changed = 0;
        foreach $so_file (@SO_Files_Iter) {
	    next if (!defined $SO_Needed{$so_file});
	    my( $dep_on_hash ) = $SO_Depends{$so_file};
	    foreach $dep ( keys %$dep_on_hash ) {
		if (! defined $SO_Needed{$dep}) {
		    my $allibs = locate_lib($dep);
		    my $lib;
		    foreach $lib (@$allibs) { 
			if (! defined $SO_Needed{$lib} ) {
			    $changed = 1;
			    $SO_Needed{$lib} = 0;
			}
		    }
		}
	    }
        }
    }
}


# for the sake of non mainwin .so's there is a need to analyze dependencies
# between those .so's and other mw or non-mw .so's.
sub resolve_dlls
{
    my ($dependent, $dependee, @depend_list);
    my($so_needed, $so_depends, $unresolved_so,$resolved_so, $so_files ) = @_;

    if ( $Full_Trace ) { print STDERR ">> resolving .so \n"; }

    foreach $unresolved_symbol (keys(%$unresolved_so)) {
	@depend_list = @{$$unresolved_so{$unresolved_symbol}};
	$dependee  = $$resolved_so{$unresolved_symbol};
	next if ( ! defined $dependee );
	foreach $dependent (@depend_list) {
        next if ( $$so_files{$dependent}->{"is_mainwin"} && $$so_files{$dependee}->{"is_mainwin"} );
 
	if ( defined $$so_needed{$dependent} ) {
	    my $dep_on = $$so_depends{$dependent};
	    if ( $Full_Trace && ! defined $$dep_on{$dependee} ) { 
		local $, = " ";
		print STDERR "$dependent depends on $dependee, [$unresolved_symbol]\n";
		print STDERR "adding $dependee to ", %$dep_on, "\n";
	    }

	    $$dep_on{$dependee} =  "";
        my( @tmp_depend_list );
        foreach (@depend_list) {
            if( $_ ne $dependent ) {
                push( @tmp_depend_list, $_ );
            }
        }
        @{$$unresolved_so{$unresolved_symbol}} = @tmp_depend_list;
	    $$so_needed{$dependee} = 0;
	}
}
    }
    my( @UnprocessedSO ) = keys %$so_needed;
    while( shift  @UnprocessedSO ) {
        foreach $dependent (keys %${$$so_depends{$_}}) {
            if( ! defined $$so_needed{$dependent} ) {
                $$so_needed{$dependent} = 0;
                push( @UnprocessedSO, $dependent );
            }
        }
    }
}

#
# Description:
# 
# Parameters:
#   none
#
# Side effects: 
#   @SO_Sorted
#
# Return:
#   none
#
sub sort_so_files_in_topological_order
{
    # sort topologically all x in SO_Needed. (use %SO_Depends{x} )

    # create a root node for the tree, which depends on every thing
    my %depends;
    @depends{keys %SO_Needed} = values %SO_Needed;
    $SO_Depends{'@@@ROOT'} = \%depends;

    # make a list of all system libs
    my @system_libs;
    foreach $lib (keys %SO_Needed) {
	if ( $SO_Files{$lib}->{"is_system"} ) {
	    push @system_libs, $lib;
	}
    }

    # make a list of  stripped+empty libs
    my @stripped_empty;
    foreach $lib (keys %SO_Needed) {
	if ( $SO_Files{$lib}->{"is_stripped"} && 
	     $SO_Files{$lib}->{"is_empty"} )  {
	    push @stripped_empty, $lib;
	}
    }

    # make every non-system lib dependent on system libs
    foreach $lib (keys %SO_Needed) {
	my $h = $SO_Depends{$lib};
	if ( !$SO_Files{$lib}->{"is_system"} ) {
	    @$h{@system_libs} = @system_libs;
	}
    }

    # make every non-system & non stripped+empty lib depends on stripped+empty
    foreach $lib (keys %SO_Needed) {
	my $h = $SO_Depends{$lib};
	if ( !($SO_Files{$lib}->{"is_system"} && 
	       $SO_Files{$lib}->{"is_stripped"} && 
	       $SO_Files{$lib}->{"is_empty"}) )  {
	    @$h{@stripped_empty} = @stripped_empty;
	}
    }

    @SO_Sorted = ();
    local %visited = (); # used for preventing infinite recursions
    make_dfs('@@@ROOT');    
    pop @SO_Sorted; # remove @@@ROOT

    # reverse @SO_Sorted
    if (  $Platform !~ /aix.*/ ) {
	my @tt = ();
	foreach $l ( @SO_Sorted ) {
	    unshift @tt, $l;
	}
	@SO_Sorted = @tt;
    }
}

#
# Description:
# dfs the dependency tree
#
# Parameters:
#   [in] $node - first is root of the tree , depends on all elements
#
# Side effects:
#   @SO_Sorted
#
# Return:
#  none
#   
#
sub make_dfs
{
    my ($node) = @_;

    return if defined $visited{$node};
    my $children = $SO_Depends{$node};
    $visited{$node} = 0;

    # call childs
    foreach $child ( keys %$children ) {
	make_dfs($child);
    }

    # print myself
    push (@SO_Sorted, $node);
}


sub la_msg
{
    my $the_msg = shift;
    &::print_msg("mwla", $the_msg);
}


sub la_error
{
    my $the_error = shift;
    &::print_err("mwla", $the_error);
}

sub la_warning
{
    my $the_msg = shift;
    &::print_warn("mwla", $the_msg);
}


sub process_link_analyzer_flags {
    # self isn't used but I may get around to cleaning up this code later on
    # and then it will be needed so we can get rid of all these globals.
    my $self = shift;
    my @internal_flags = @_;

    my @initial_unresolved = ();
    while ( $_ = shift @internal_flags ) {
	   if (/^-noquiet/)    { $Partial_Trace = $True; }
	elsif (/^-vmw/)        { $Vmw = $True; }
	elsif (/^-use_libmap/) { $Use_Libmap = $True; }
	elsif (/^-implib:(.*)/){ $Implib = $1; }
	elsif (/^-ismain/)     { $Building_So = $False;  }
	elsif (/^-local_map/)  { $Local_Mapfile = shift @internal_flags; }
	elsif (/^-export_map/) { $Export_Mapfile = shift @internal_flags; }
	elsif (/^-nt_def_file/){ $Nt_Def_File = shift @internal_flags; }
	elsif (/^-map_support:(.*)/) { 
	    $Map_Support_Type = $1;
	    if( !($Map_Support_Type =~ m/(strong|weak|cplusplus)/) ) {
		print "Bad parameter for -map_support flag\n";
		$Map_Support_Type = "weak";
	    }
	    else {
		$Map_Support_Type = $1;
	    }
	} 
	elsif (/^-outdir/)     { $OutDir = shift @internal_flags; }
	elsif (/^-unresolve/)  { push(@initial_unresolved, shift @internal_flags);}
        elsif (/^-noregenlib/) { $NoRegenLibs = 1; }
	elsif (/^-plugin/)     { $IsPlugin = 1 ;}
    }
    @linker_analyzer::Initial_Unresolved = @initial_unresolved;
}


sub process_lib_path {
    my $args = shift;
    foreach ( @$args ) {
	next unless defined $_;
	
	if ( /^-L(.*)/ ) {
	    my $dir = $1;
	    if ( defined $dir ) {
		$dir =~ s/^\s*//; # get rid of initial white spaces
		push(@Lib_Path, $dir) if $dir;
	    }
	}
	elsif (/^-nodefaultlib:/) { 
	    my $rest = $';
	    if ( $rest =~ /^\"(.*)\"/ ) { $rest = $1 }
	    foreach (split(' ', $rest)) {
		&::split_path($_, \$path, \$base, \$ext);
		if ( $base =~ /^lib/) { $base = $'; }
		$Nodef_Lib{$base} = 0;
		push (@Lib_Path, $path) if $path; 
	    }
	}
	elsif (/^-defaultlib:/) { 
	    my $rest = $';
	    if ( $rest =~ /^\"(.*)\"/ ) { $rest = $1 }
	    foreach (split(' ', $rest)) {
		&::split_path($_, \$path, \$base, \$ext);
		if ( $base =~ /^lib/) { $base = $'; }
		$Def_Lib{$base} = 0;
		push (@Lib_Path, $path) if $path;
	    }
	}
	elsif (/^-nodefaultlib/) { $Nodef_Libs = $True }
	elsif (/^-force/) {push @Other_Flags, $_;}
    }
    # handle deflibs
    $Def_Lib{"msvcrt"} = 0 if $Vmw;
    %Def_Lib = () if $Nodef_Libs;
       if ( %Nodef_Lib && %Def_Lib ) {
	#compute the difference %Def_Lib - %Nodef_Lib
	delete @Def_Lib{keys %Nodef_Lib};
    }     
    if ( $Partial_Trace ) { 
	print STDERR "\tmwla  ", join(" ", @LA_Args), "\n"; 
    }
    # optimizing @Lib_Path by eliminating similer paths
    &::list_make_unique(\@Lib_Path);
}

sub init_target {
    while( @_ ) {
	$_ = shift;
	if( s/^-o//o ) {
	    if( /^\s*$/o ) {
		$_ = shift;
	    }
	    $Target = $_;
	}
    }
}

#
# Description:
# sets initial global data, which is platform specific
#
# Parameters:
#   none
#
# Side effects:
#   creates: 
#  %System_Dirs : systems directories . helps to 
#    identify whether a lib is a system lib, and if yes, don't process it, just pass it 
#    through.
#
# Return:
#   void 
#   
#
sub init
{
    my(%lib_path);
    my(%ld_lib);

    my($default_builtin_lib_search_path) = "/usr/ccs/lib:/usr/lib";
    my $builtin_lib_search_path;

    # platform system directories
    $lib_path{"sunos5_lib_path"} = "$default_builtin_lib_search_path:/usr/openwin/lib:/lib:.*/SUNWspro/lib";
    $lib_path{"aix4_lib_path"} = "/usr/lib:/lib"; 
    $lib_path{"linux_lib_path"} = "/lib:/usr/lib:/usr/X11R6/lib";
    $lib_path{"irix6_lib_path"} = "/usr/lib:/lib:/usr/local/lib:/usr/lib64:/lib64:/usr/local/lib64:/usr/lib32:/lib32:/usr/local/lib32";
    $lib_path{"osf1v4_lib_path"} = "/usr/shlib:/usr/ccs/lib:/usr/lib/cmplrs/cc:/usr/lib:/usr/local/lib:/var/shlib";
    
    $deflib = $ENV{LPATH};
    if ( defined($deflib) and $deflib ne "" ) {
	$lib_path{"ux10_lib_path"} = $deflib;
	$lib_path{"ux11_lib_path"} = $deflib;
    }
    else {
	$lib_path{"ux10_lib_path"} = "$default_builtin_lib_search_path:/opt";
	$lib_path{"ux11_lib_path"} = "$default_builtin_lib_search_path:/opt";
    }
    
    $builtin_lib_search_path = $lib_path{"${Platform}_lib_path"};
    
    # set def lib dir: used also for aix , to identify system .a files as 
    # shared libs
    my @sdirs = split (/:+/, $builtin_lib_search_path);
    @System_Dirs{@sdirs} = @sdirs;

    if ( ! -e "$LA_Tmp_Dir" ) { la_error("$LA_Tmp_Dir does not exist"); } 
}

sub push_mode
{
    my ($libs, $cur_mode, $mode_ptr) = @_;
    return if ($cur_mode eq $$mode_ptr);
    $$mode_ptr = $cur_mode;
    push (@$libs, "-B$$mode_ptr");
}

sub  skip_header
{
    my $file = shift;
    my $l;
    $l = <$file>; # title
    $l = <$file>; # header version
    $l = <$file>; # source lib
    $l = <$file>; # lib type
    $l = <$file>; # lib timestamp
    $l = <$file>; # nm flags
    $l = <$file>; # created from def or not
    $l = <$file>; # priority 
    $l = <$file>; # mainwin lib flag
}

sub cleanup
{
    foreach $lib (keys %Tmp_libs) {
	unlink $Tmp_libs{$lib};
    }
    %Tmp_libs = ();
}


sub process_local_mapfile
{
    my ($mapfile, $sym_tbl) = @_;
    my $line;
    my @cols;

    if ( $Full_Trace ) { 
	print STDERR "ignoring hidden symbols from file: $mapfile\n"; 
    } 

    if ( ! open(MAPFILE, "< $mapfile") ) {
	la_error("can't open local mapfile:  $mapfile");
	return $False;
    }
       
    while ( defined MAPFILE and $line = <MAPFILE> ) {
    	chomp($line);
	@cols = split(' ', $line);
	delete $$sym_tbl{$cols[0]};
    }
    close(MAPFILE);
}

###############################
########    main      #########
###############################

sub main {
    my ($command_line,
	$link_params,
	$mkmapflags) = @_;

    @MkMapDefines = @$mkmapflags;

    @Command_Line = @$command_line;

    my(%Deffile_EXPORTS);
    my(%Deffile_LOCALS);

    init();

    process_lib_path([@$link_params, @$command_line]);

    # add each def lib as -l 
	# (def libs were initialized in process_lib_path)
    foreach my $lib (keys %Def_Lib ) { 
		push @Command_Line, "-l$lib" 
	}

    init_target(@Command_Line);

    my ($ext, $base, $path);
    &::split_path($Target, \$path, \$base, \$ext); 
    $Target_Stripped .= $base . ".$ext";
    if ($Target_Stripped =~ /^lib(.*)\.s.$/) {
	$Target_Stripped = $1;
    }

    my $deffile_obj = get_symbols_from_def_file($Target, \%Deffile_EXPORTS, \%Deffile_LOCALS);

    $linker_analyzer::link_parser = new Link_Parser(\%Deffile_EXPORTS);
    $link_parser->force_unresolved(@Initial_Unresolved);
    if ($Map_Support_Type eq "cplusplus") {
	$link_parser->extract_cpp_exports();
    }
    
    $link_parser->parse_command_line(@Command_Line);
    $link_parser->build_line();
    my $ret = $link_parser->get_link_line();

    generate_unix_def_file(\%Deffile_EXPORTS, \%Deffile_LOCALS, $link_params);
    cleanup();
    if ( $Full_Trace ) { print STDERR "\tmwla finished\n"; }
    return $ret;
}

# Gets EXPORTS and LOCALS lists from .def file
sub get_symbols_from_def_file
{
    my($DllName, $ExportsFromDef, $LocalsFromDef) = @_;
    my(@MkMapParams); 
    my($ext, $base, $path, $Target_Stripped);

    &::split_path($DllName, \$path, \$base, \$ext); 
    $Target_Stripped .= $base . ".$ext";
    if ($Target_Stripped =~ /^lib(.*)\.s.$/) {
	$Target_Stripped = $1;
    }

    push( @MkMapParams, "-l ", "${base}.${ext}" );
    if ($Export_Mapfile eq "") { $Export_Mapfile = "${OutDir}/${base}.def"; }
    push( @MkMapParams, "-o ", "$Export_Mapfile" );
    push( @MkMapParams, "-map ", "$Nt_Def_File" ) if ($Nt_Def_File ne "" && $Building_So);
    push( @MkMapParams, "-plugin_dll" ) if $IsPlugin ;
    push( @MkMapParams, "-cplusplus" ) if ($Map_Support_Type eq "cplusplus");
    push( @MkMapParams, @MkMapDefines );
    $deffile_obj = new deffile(@MkMapParams);
    # Now get symbols in EXPORTS and LOCALS sections in def file
    $deffile_obj->get_EXPORTS_and_LOCALS_lists($ExportsFromDef, $LocalsFromDef);
    if ($Map_Support_Type eq "weak") {
        # From now on we behave as if we have no def file. We have done 
        # all we need to do with the def file
        $Nt_Def_File = "";
    }
    return $deffile_obj;
}

# generates unix def file
sub generate_unix_def_file
{
    my( $ExportsFromDef, $LocalsFromDef, $linker_params ) = @_;
    my( %tmp_hash );
    my( @SymbolsToExport );
    my( @SymbolsToHide );

    my @O_Symbols = $link_parser->get_globals_in_module();

    # On AIX we have problems if we dont give linker a map file
    # So we'll generate it here
    if( $ENV{"MWOS"} eq "aix4" && $Nt_Def_File eq "" && $Building_So ) {
        $deffile_obj->write_2_disk(\@O_Symbols);
        $ExportMapGenerated = $True;
        # Remember symbols that we exported
        @SymbolsExportedFromDll{@O_Symbols} = ();
        if( %$ExportsFromDef ) {
            @SymbolsExportedFromDll{keys %$ExportsFromDef} = values %$ExportsFromDef;
        }
        push(@$linker_params, "-bexport:$Export_Mapfile");
    } elsif( $Nt_Def_File ne "" && $Building_So ) {
        # Now depending on -map_support flag, calculate @SymbolsToExport, @SymbolsToHide
        # export only symbols exported from def file, hide others
        if( %$ExportsFromDef ) {
	    if ($Map_Support_Type eq "cplusplus") {
		@SymbolsToExport = keys %$ExportsFromDef;
	    } else {
		@SymbolsToExport = ();
	    }
            @tmp_hash{@O_Symbols} = (); 
	        if ($ENV{"MWOS"} ne "sunos5") {
		        SubstractHash(\%tmp_hash, $ExportsFromDef, \@SymbolsToHide); 
	        } else {
		        @SymbolsToHide = ();
	        }
        } else {
            # There are no exports in .def file
            # Export all, but %DEFFILE_LOCALS
            @SymbolsToHide = ();
            @tmp_hash{@O_Symbols} =(); 
            SubstractHash(\%tmp_hash, $LocalsFromDef, \@SymbolsToExport); 
        }
        # Generate UNIX map file from NT's def file
        $deffile_obj->generate(\@SymbolsToExport, \@SymbolsToHide);
        $ExportMapGenerated = $True;
        # Remember symbols that we exported
        @SymbolsExportedFromDll{@SymbolsToExport} = ();
        if( %$ExportsFromDef ) {
            @SymbolsExportedFromDll{keys %$ExportsFromDef} = values %$ExportsFromDef;
        }
    } else {
        $Export_Mapfile = ""; # No map file
    }

}

# Substracts one array from another
sub SubstractHash
{
    my( $FirstHash, $SecondHash, $ResultArray ) = @_;
    foreach $key ( keys %$FirstHash )
    {
        if( !exists($$SecondHash{$key}) ) { push @$ResultArray, $key; }
    }
}

# checks if dir is contained in one of System_Dirs{}
sub is_system_dir
{
    my $dir = shift;
    # Stuff in mainwin isn't system.
    return $False if( $dir =~ m@^$ENV{MWHOME}@ );

    foreach $d (keys %System_Dirs) {
	if ( $dir =~ m@^$d@  ) { 
	    return $True; 
	}
    }
    return $False;
}

sub locate_lib
{
    my $stripped_lib_name = shift;
    my $mode = shift;
    if (!defined $mode) { $mode = "dynamic" }
	
    my ($lib_type, @libs, $mapped);
    $mapped = $False;

    my $mapped_lib ;
    if ( $Use_Libmap ) {
	$mapped_lib = $::libmap{$stripped_lib_name};
	if ( !defined $mapped_lib || $mapped_lib eq "" ) { 
	    # if !defined it could be a system lib or user lib or a third  party lib
	    # if eq "" it is  a known NT lib which does not have a mw matching lib
	    # currently, to add a third party lib the user must update the LD_LIBRARY_PATH
	    # or add -L for la/ld to find it.
	    # If la couldn't locate  the lib it passes it as is to ld (!defined case) or
	    # ignores it (eq "" case).
	    @libs = ( $stripped_lib_name );
	}
	elsif ( $mapped_lib eq "@" ) {
	    la_msg("lib: $stripped_lib_name: ignored by request"); 
	    return ; # ignore lib
	}
	else {
	    $mapped = $True;
	    @libs = split(' ', $mapped_lib);
	}
    }
    else {
	$mapped_lib = $stripped_lib_name;
	@libs = ( $mapped_lib );
    }
 
    my $lib;
    my @outlibs = ();
    foreach $lib (@libs) {
	if (defined $SO_Files{$lib} || defined $A_Files{$lib}) {
	    my $lib_obj = $SO_Files{$lib} || $A_Files{$lib};
	    push (@outlibs, $lib_obj);
	    next;
	}
	my $lib_obj = process_lib($lib, $mode, $mapped, $mapped_lib);
	push(@outlibs, $lib_obj);
    }
    return \@outlibs;
}

sub process_lib {
    my ($lib, $mode, $mapped, $mapped_lib) = @_;
    $mode ||= "";
    $mapped ||= 0;
    my $lib_obj = new lib($lib, $mapped, $mode); 
    if ( $lib_obj->process() ) {
	$lib = $lib_obj->{"source"};
	push (@outlibs, $lib_obj);
	if ( $lib_obj->{"type"} eq $Archive_t ) {
	    if ( not defined $A_Files{$lib} ) {
		push(@A_Files_Iter, $lib);
		$A_Files{$lib} = $lib_obj;
	    }
	}
	elsif ( $lib_obj->{"type"} eq $Sharedlib_t ) {
	    if ( not defined $SO_Files{$lib} ) {
		push(@SO_Files_Iter, $lib);
		$SO_Files{$lib} = $lib_obj;
	    }
	}
	else { 
	    $Unknown_Libs{$lib} = $lib_obj;
	}
    }
    else {
	if (  ! $Use_Libmap || ! defined $mapped_lib || $mapped) {
	    if ( $Partial_Trace ) {
		la_warning("Could not locate lib: $lib - pass on to ld");
	    }
	    $Corrupted_Libs{$lib} = $lib_obj;
	}
	else {
	    # is mapped to  "" 
	    if ( $Partial_Trace ) {
		la_warning("Could not locate lib: $lib - ignoring");
	    }
	}
    }
    return $lib_obj;
}

sub new
{
    my $self = bless {}, shift;
    
    $Full_Trace = (lc($ENV{MWTOOL_TRACE}) eq "full");
    $Partial_Trace = (defined $ENV{MWTOOL_TRACE} 
		      && ($ENV{MWTOOL_TRACE} ne "FALSE"));

    $^W =1 if $Full_Trace; # turn warnings on if trace is full
    $True = 1;
    $False = 0;
    $LIB_Extension = "lib";
    $Archive_t = "a";
    $Sharedlib_t = $ENV{DLL_SUFFIX};
    $Building_So = $True;
    $Platform = $ENV{MWOS};
    $LA_Tmp_Dir = "/tmp";
    $MW_Dir = "mw/";
    $Target = "";
    $Target_Stripped = "";
    $Check_Priority = $False;
    $LIB_Header_Version = 2.0;
    $IsMainwinLib = $False;
    $Raw_Entry = "";
    $Use_Temporary_Flocator = $False;
    $Use_Libmap = $False;
    $Vmw = 0;
    $IsPlugin = 0;
    $NoRegenLibs = 0;
    $Mw_Libs = "$ENV{MWHOME}/lib/extra_libs/$ENV{MWOS}";
    $Nodef_Libs = $False;
    $Local_Mapfile = "";
    $Export_Mapfile = "";
    $Nt_Def_File = "";
    $Map_Support_Type = "";
    $ExportMapGenerated = $False;
    $OutDir = "";
    $Use_Priority = $True; 

    %SymbolsExportedFromDll = ();
    %A_Files = ();
    %SO_Files = ();
    %SO_Depends = ();
    %System_Dirs = ();
    %SO_Needed = ();
    %Nodef_Lib = ();
    %Def_Lib = ();

    @SO_Sorted = ();
    @Lib_Path = ();
    %Unknown_Libs = ();
    %Common_Syms = ();

    # Wait until $True etc are initialized
    $self->process_link_analyzer_flags(@_);
    
    $Check_Priority = ($Platform =~ /aix/ and $Building_So);

    if ( not $Platform ) { 
	la_error("\$MWOS is not set");
    }
    $Platform_Funcs = new Platform_funcs();
    $Pragma = new pragma();
    &::print_a_Perlocator_message();

    $SIG{INT}  = \&cleanup;
    $SIG{HUP}  = \&cleanup;
    $SIG{PIPE} = \&cleanup;
    $SIG{TERM} = \&cleanup;

    return $self;
}

# Always cleanup at exit.
END {    cleanup(); }

sub analyze
{
    my ($self, 
	$command_line,
	$link_params,
	$mkmapflags) = @_;

    @LA_Args = (@$command_line,
		@$link_params,
		@$mkmapflags,
		);

    return main($command_line,
		$link_params,
		$mkmapflags,
		);
}


sub raw_produce_lib
{
    my ($self) = shift;
    if ( ! $Building_So ) { return $False; }

    my $infile = $Target;
    my $pwd = `pwd`;
    chomp($pwd);
    if ( $Target !~ /^\// ) { $infile = "${pwd}/${Target}"; }
    my $outfile = $infile;
    if ( $Implib ) { $outfile = $Implib;  }
    my @cmd = ("-i$infile", "-o$outfile");
    if ( $Partial_Trace ) { push (@cmd, "-noquiet"); }
    if ( $ExportMapGenerated ) { 
        push(@cmd, "-create_from_def"); 
        # Push all exported symbols
        foreach( keys %SymbolsExportedFromDll ) {
            push(@cmd, "-exp_sym", $_);
        }
        # Push all unresolved symbols
        foreach( $link_parser->get_undefs_in_module() ) {
            push(@cmd, "-unres_sym", $_);
        }
        # We need to tell mwmklib about priority on AIX
        if( $Check_Priority ) {
            push(@cmd, "-priority", $Priority);
        }
    }
    if ( $Local_Mapfile ) { push (@cmd, "-local_map", $Local_Mapfile);}
    # Call mwmklib
    &mwmklib::main(\@cmd);

    return $True;
}

sub sym_defined
{
    my ($self,$entry) = @_;
    return $linker_analyzer::link_parser->defined_in_module($entry);
}

sub sym_common
{
    my ($self,$entry) = @_;
    return $Common_Syms{$entry};
}

sub get_so_lib_names
{
    my ($self) = shift;
    my @xxx = $linker_analyzer::link_parser->get_mainwin_sos();
    return \@xxx;
}

sub get_raw_entry
{
    my ($self) = shift;
    return $Raw_Entry;
}

sub uncalc_safe_name
{
    my ($safe_name) = @_;
    my $temp = $safe_name;
    # while there is an _ in $temp, interperet the next two characters as
    # a hex number, and replace the triplet with the character whos ordinal is
    # that hex number.
    my $so_far = "";
    while ($temp =~ /_(..)/) {
	my $c = chr( hex($1) );
	$so_far = $so_far . $` . $c;
	$temp = $';
    }
    return ($so_far . $temp);
}

} # end of package linker_analyzer;

