do "$ENV{'MWHOME'}/lib/common-utilities.pl";
do "$ENV{'MWHOME'}/XDE/lib/errors.pl";

package mwmklib;  


###############################
######## sub routines #########
###############################


#
# Description:
# redirect the nm output to a file
#
# Parameters:
#   [in] input file full path name (.so) 
#   [out] out file, full path name
#   [in] lib type
#
# Side effects:
#    created the output file
#
# Return:
#
sub process_file
{
    my($infile, $outfile, $lib_type) = @_;
    my($dev, $ino, $mode, $nlink, $uid, $gid, $rdev, $size, $atime, $mtime, $ctime, $blksize, $blocks) = lstat $infile;
    my $t = time;

    my $cmd = "sum $infile | ";
    open ( SUM , $cmd) || mwmklib_error("command failed: $cmd");
    my ($line, @words);
    $line = <SUM>;
    chomp ($line);
    @words = split(/\s+/,$line);

    my $tmpfile = "/tmp/${$}_$t.mwmklib";

    open(OUT,  "> $outfile") || mwmklib_error("can't write to $outfile");
    print OUT "This file was automatically generated by mwmklib at ", scalar localtime, ".\n";
    print OUT "Header Version: $MWMKLIB_Header_Version\n";
    print OUT "Source: $infile\n";
    print OUT "Type: ", ($lib_type eq $Archive_t ? "archive" : "shared library"), "\n";
    print OUT "Time stamp of original lib was: ", scalar localtime($mtime), 
    "  [", $mtime, "]\n";
    print OUT "Sum of original lib was: [", $words[0], "," ,$words[1], "]\n";
    print OUT "Flags: nm $NM_Flags\n";
    print OUT "CreatedFromDefFile: $CreateFromDefFile\n";
    
    if ( $lib_type eq  $Archive_t ) {
	process_a_file($infile, $tmpfile);
	print OUT "Priority: n/a\n";
	print OUT "Mainwin lib: n/a";
	if ( $Is_Stripped_Lib ) { print OUT "  (stripped)" }
	print OUT "\n";
	close OUT;
	`cat $tmpfile >> $outfile`;
	unlink $tmpfile;
    }
    else {
	process_so_file($infile, $tmpfile);
	print OUT "Priority: ", ($Check_Priority ? $Max_Priority : "n/a"), "\n";
	print OUT "Mainwin lib: ", ($Is_Mainwin_Lib ? "yes" : "no" );
	if ( $Is_Stripped_Lib ) { print OUT "  (stripped)" }
	print OUT "\n";
	close OUT;
	`cat $tmpfile >> $outfile`;
	unlink $tmpfile;
    }
}

sub mwmklib_msg
{
    &::print_msg("mwmklib", $_[0]);
}


sub mwmklib_error
{
    if ( $No_Errors ) {
	exit 1;
    }
    else {
	&::print_err("mwmklib", $_[0]);
    }
}

sub mwmklib_warning
{
    &::print_warn("mwmklib", $_[0]);
}


#
# Description:
# sets platform specific flags 
#
# Parameters:
#   none
#
# Side effects:
#   creates: 
#  $NM_Params_Flags  = nm flags
#  $NM_Params_Flags_A = nm flags for archives
#  $NM_Params_Order = permutation for canonical order
#
# Return:
#   void 
#   
#
sub init
{
#   nm_params{<platform>_<element-type>[_<module-type>_<action>]}
    my(%nm_params);
 

    if ( not $Platform ) { 
	mwmklib_error("\$MWOS is not set");
    }

    my($default_nm_regexp_D) = "^[TDBA]";
    my($default_nm_regexp_U) = "U";
    my($default_nm_regexp_C) = "^\$";
    my(@default_nm_order) = qw(2 3);
    my($default_nm_regexp_O) = ".*\\[(.*)\\]";
    my($default_nm_flags) = " -A -g -p ";

    # flags to invoke nm
    $nm_params{"sunos5_flags"} = $default_nm_flags; 
    $nm_params{"aix4_flags"} = " -A -p -P -g -C "; 
    $nm_params{"linux_flags"} =  " -A -P -g ";
    $nm_params{"irix6_flags"} = $default_nm_flags;
    $nm_params{"osf1v4_flags"} = " -A -B -g -mangled_name_only ";
    $nm_params{"ux10_flags"} = $default_nm_flags;
    $nm_params{"ux11_flags"} = $default_nm_flags;
    $NM_Params_Flags = $nm_params{"${Platform}_flags"};


    # flags to invoke nm for archives
    $nm_params{"sunos5_flags_A"} = $default_nm_flags; 
    $nm_params{"aix4_flags_A"} = " -A -p -P -g -C "; 
    $nm_params{"linux_flags_A"} =  " -A -P -g ";
    $nm_params{"irix6_flags_A"} = " -r $default_nm_flags";
    $nm_params{"osf1v4_flags_A"} = " -A -B -g -mangled_name_only ";
    $nm_params{"ux10_flags_A"} = $default_nm_flags;
    $nm_params{"ux11_flags_A"} = $default_nm_flags;
    $NM_Params_Flags_A = $nm_params{"${Platform}_flags_A"};

    # permutation to order the nm output line in canonical form

    my(@default_nm_order) = qw(2 3 0);
    my(@ord) = qw(1 2 0);
    my(@ord2) = qw(2 1 0);
    my(@ord3) = qw(2 3 0);
    my(@ord4) = qw(2 1 0);
    $nm_params{"sunos5_order"} = \@default_nm_order;
    $nm_params{"aix4_order"} =  \@ord2;
    $nm_params{"linux_order"} =  \@ord4;
    $nm_params{"irix6_order"} = \@ord;
    $nm_params{"osf1v4_order"} = \@ord3;
    $nm_params{"ux10_order"} = \@ord;
    $nm_params{"ux11_order"} = \@ord;
    *NM_Params_Order = $nm_params{"${Platform}_order"};

    # regexp to extract the current .o file name from archive 
    $nm_params{"sunos5_regexp_A_O"} = $default_nm_regexp_O; 
    $nm_params{"aix4_regexp_A_O"} = $default_nm_regexp_O;
    $nm_params{"linux_regexp_A_O"} =  $default_nm_regexp_O;
    $nm_params{"irix6_regexp_A_O"} = $default_nm_regexp_O;
    $nm_params{"osf1v4_regexp_A_O"} = $default_nm_regexp_O;
    $nm_params{"ux10_regexp_A_O"} = $default_nm_regexp_O;
    $nm_params{"ux11_regexp_A_O"} = $default_nm_regexp_O;
    $NM_Params_Regexp_A_O = $nm_params{"${Platform}_regexp_A_O"};

    # regexp for finding a defined symbol
    $nm_params{"sunos5_regexp_D"} = $default_nm_regexp_D;
    $nm_params{"aix4_regexp_D"} =  $default_nm_regexp_D;
    $nm_params{"linux_regexp_D"} =  "^[DTWVBCR]";
    $nm_params{"irix6_regexp_D"} = "^[TDBAGSRCE]";
    $nm_params{"osf1v4_regexp_D"} = "^[TDGCQBRSAEHJK]";
    $nm_params{"ux10_regexp_D"} = "^[TDABC]";
    $nm_params{"ux11_regexp_D"} = "^[TDABRS]";
    $NM_Params_Regexp_D = $nm_params{"${Platform}_regexp_D"};

    # regexp for finding an unresolved symbol
    $nm_params{"sunos5_regexp_U"} = $default_nm_regexp_U;
    $nm_params{"aix4_regexp_U"} = $default_nm_regexp_U;
    $nm_params{"linux_regexp_U"} =  "^[U]";
    $nm_params{"irix6_regexp_U"} = "^[UV]";
    $nm_params{"osf1v4_regexp_U"} = "^[UV]";
    $nm_params{"ux10_regexp_U"} = $default_nm_regexp_U;
    $nm_params{"ux11_regexp_U"} = $default_nm_regexp_U;
    $NM_Params_Regexp_U = $nm_params{"${Platform}_regexp_U"};

    # regexp for finding an unresolved symbol
    $nm_params{"sunos5_regexp_C"} = $default_nm_regexp_C;
    $nm_params{"aix4_regexp_C"} = $default_nm_regexp_C;
    $nm_params{"linux_regexp_C"} =  $default_nm_regexp_C;
    $nm_params{"irix6_regexp_C"} = $default_nm_regexp_C;
    $nm_params{"osf1v4_regexp_C"} = $default_nm_regexp_C;
    $nm_params{"ux10_regexp_C"} = $default_nm_regexp_C;
    $nm_params{"ux11_regexp_C"} = "^C";
    $NM_Params_Regexp_C = $nm_params{"${Platform}_regexp_C"};
}


#
# Description:
#   if output file is not preceded with a path, then current dir is considered .
#   output file is attached an extension of .lib.
#
# Parameters:
#   [in/out] @args: full path name of input file,
#           name or full path name of output file 
#   [out] lib type
#
# Side effects:
#  $NM_Flags    
# Return:
#   in_file, out_file, return True if all ok, False otherwise
#
sub process_command_line
{
    my($args, $lib_type, $infile, $outfile) =  @_;
    my $extension = $MWMKLIB_Extension;
    my $token;

    $$lib_type = $Sharedlib_t;
    my $pwd = `pwd`;
    chomp($pwd);

    if ( $#$args == -1 ) {
	usage();
	exit 1;
    }


    while ( $token = shift @$args ) {
	$_ = $token;
 	if    ( /^-noquiet/ )    { $Partial_Trace = $True; }
	elsif    ( /^-noerrors/)    { $No_Errors = $True; }
	elsif ( /^-i(.*)/ )      { 
	    if ( $1 eq "" ) { 
		$token = shift @$args;
		$token =~ /^\s*(.*)/;
		$$infile = $1; 
	    }
	    else {
		$$infile = $1; 
	    }
	}
	elsif ( /^-o(.*)/ ) {
	    if ( $1 eq "" ) { 
		$token = shift @$args;
		$token =~ /^\s*(.*)/;
		$$outfile = $1; 
	    }
	    else {
		$$outfile = $1; 
	    }
	}
	elsif ( /^-local_map/ ) {
		$Local_Mapfile = shift @$args;
	}
	elsif ( /^-create_from_def/ ) {
		$CreateFromDefFile = $True;
	}
	elsif ( /-exp_sym/ ) {
		$symbol_tbl{shift @$args} = "D";
	}
	elsif ( /-unres_sym/ ) {
		$symbol_tbl{shift @$args} = "U";
	}
	elsif ( /-priority/ ) {
		$Max_Priority = shift @$args;
	}
	else               { usage(); exit 1; }
    }

    if ( not -e $$infile ) {
	mwmklib_error("$$infile not found");
	return $False;
    }

    # set the lib type
    if ( $$infile =~ /\.a$/ ) {
	$$lib_type = $Archive_t;
    }

    # set the extension, remove lib from start
    my $path, $base, $ext;
    split_path($$outfile, \$path, \$base, \$ext);
    if ( $base =~ /^lib/ && lc($ext) ne "lib" ) { $base = $'; }
    if ( lc($extension) eq "lib" && lc($ext) eq "lib") { $extension = $ext };
    if ( $path eq "" ) { $path = "."; }
    $$outfile = "$path/$base.$extension";

    # convert outfile path to absolute path
    if ( $$infile !~ /^\// ) { $$infile = "$pwd/$$infile"; }

    if ( $$infile =~ /.*\.a/ ) {
	$NM_Flags = $NM_Params_Flags_A;
    }
    else {
	$NM_Flags = $NM_Params_Flags;
    }

    $Check_Priority = ( $ENV{MWOS} =~ /aix/ and $$lib_type eq $Sharedlib_t );
    if ( ! $Check_Priority ) { $Max_Priority = "n/a"; }
}


sub usage
{
	print STDERR "\nmwmklib -i<lib full path name> -o<outfile> [-local_map <LOCALS mapfile>] [-noerrors] [-noquiet]\n\n";
	print STDERR "outfile gets a \".$MWMKLIB_Extension\" extension, and the leading \"lib\" is stripped off  .\n\n";
}
###############################
########    main      #########
###############################

sub main {
    my ($args) = shift;
    my($lib_type, $infile, $outfile);

    # set some globals
    $True = 1;
    $False = 0;
    $MWMKLIB_Extension = "lib";
    $Archive_t = "a";
    $Sharedlib_t = $ENV{DLL_SUFFIX};
    $MWMKLIB_Header_Version = 2.0;
    $LA_Dir = "$ENV{'MWHOME'}/lib/la/";
    $Platform = $ENV{MWOS};
    $Full_Trace = (lc($ENV{MWTOOL_TRACE}) eq "full");
    $Partial_Trace = (defined $ENV{MWTOOL_TRACE} 
		      && ($ENV{MWTOOL_TRACE} ne "FALSE"));
    $Max_Priority = -2001;
    $First_SO_Encountered  = $True;
    $N_2_Exp_31 = 2147483648;
    $Is_Mainwin_Lib = $False;
    $Is_Stripped_Lib = $False;
    $No_Errors = $False;
    $Local_Mapfile = "";
    $CreateFromDefFile = $False;
    %symbol_tbl = ();

    do "$LA_Dir/${Platform}/module.pl";
    do "$LA_Dir/utils.pl";
    $Platform_Funcs = new Platform_funcs();
  
    init();
    process_command_line($args, \$lib_type, \$infile, \$outfile);

    if ( $Partial_Trace) {
	print STDERR  "mwmklib [$infile => $outfile]\n"; 
    }
    process_file($infile, $outfile, $lib_type);
    return 1;
    
}

sub process_local_mapfile
{
    my ($mapfile, $hidden_map) = @_;
    my $line;
    my @cols;

    if ( ! open(MAPFILE, "< $mapfile") ) {
	la_error("can't open local mapfile:  $mapfile");
	return $False;
    }
       
    while ( defined MAPFILE and $line = <MAPFILE> ) {
    	chomp($line);
	@cols = split/\s+/, $line;
	$$hidden_map{$cols[0]} = "";
    }
    close(MAPFILE);
}



sub process_so_file
{
    my($infile, $outfile) = @_;
    my($line, @ordered_words);
    my($cmd) = "nm $NM_Params_Flags $infile | ";
    my(@words, $ty, $info);
    my $el = $True;
    my $priority;
    my %hidden_syms;
    process_local_mapfile($Local_Mapfile, \%hidden_syms) if $Local_Mapfile;

    if( ! $CreateFromDefFile ) {

    if ( $Full_Trace ) {
	print STDERR "Executing command: \"$cmd\"\n";
    }

    open( NM,  "$cmd" ) || mwmklib_error("command failed: $cmd");
    while ( $line = <NM> ) {
	chomp($line);
	# get rid of  unimportant lines
	next if ( $el && ($line =~ m/:\s*$/o  or $line =~ m/^\s*$/o) );
#	next if $line =~ m/ [a-z] /o;
	$el = $False;

	@words = split (/\s+/, $line);
	if ( $Platform eq "osf1v4" ) {
	    $Platform_Funcs->fix_nm_columns(\@words);
	}

	# reorder to canonical order
	@ordered_words = @words[@NM_Params_Order];

	$ty = substr($ordered_words[0], 0, 1);
	$info = $ordered_words[1];
	if ( $Platform eq "ux11" && $info =~ m/:/o ) { 
	    $info = $'; 
	    # if it's empty go on.
	    next if $info =~ /^\s*$/;
	}
	if ( $Platform eq "linux" && $info =~ /@@/ ) {  $info = $`; }
	next if $Platform eq "aix4" &&  substr($info, 0, 1) eq ".";

	# is a defined symbol
	if ($ty =~ m/$NM_Params_Regexp_D/o ) {	
	    if ( substr($info, 0, 5) eq "Mw___" &&
		$info =~ m/^Mw___(.*)_DependancyNode$/o ) {
		$Is_Mainwin_Lib = $True;
	    }
	    # In ux11 the an undefined symbol may also apear as "T" so
	    # if it's already undefined don't define it.
	    next if $symbol_tbl{$info};
	    
	    if ( ! defined $hidden_syms{$info}  ) {
		    $symbol_tbl{$info} = "D";
	    }

	    if ( $Check_Priority and 
		 substr($info, 0,  12) eq "__priority0x" and
		 $info =~ m/^__priority0x(.*)/o ) {
	
		$priority = (hex $1) - $N_2_Exp_31;
		if ( $First_SO_Encountered ) {
		    $First_SO_Encountered = $False;
		    $Max_Priority = $priority;
		}
		elsif ( $priority > $Max_Priority ) {
		    $Max_Priority = $priority;
		}
	    }
	}
	# is an unresolved symbol
	elsif ($ty =~ m/$NM_Params_Regexp_U/o ) {
	    $symbol_tbl{$info} = "U";
	# is an unresolved symbol
	} elsif ($ty =~ m/$NM_Params_Regexp_C/o ) {
	    $symbol_tbl{$info} = "C";
	}
    }
    close NM;
    if ( $? ) { mwmklib_error("command failed: $cmd"); }
    }  # end of if ! createdfromdeffile
    open( OUTPUT, "> $outfile" );
    foreach $sym (keys %symbol_tbl) {
	print OUTPUT "$symbol_tbl{$sym} $sym\n";
    }
    if ( ! %symbol_tbl ) {
	$Is_Stripped_Lib = $True;
    }
    close OUTPUT;
}


sub process_a_file
{
    my($infile, $outfile) = @_;
    my $ty, $line, $info, $prev_o_file = "";
    my $el = $True;
    my(@words, @ordered_words);
    my $o_file = "";
    my($cmd) = "nm $NM_Params_Flags_A $infile | ";
    my %o_syms;
    my $o_filename_set = 0;
    $Is_Stripped_Lib = $False;

    if ( $Full_Trace ) {
	print STDERR "Executing command: \"$cmd\"\n";
    }

    my %so_members = ();
    if( $ENV{MWOS} eq 'aix4' ) {
	open( SIZE, "size -X32 -f $infile |") or mwmklib_error("can't run size for $infile");
	while(<SIZE>) {
	    if( /\(\.loader\)/ ) {
		if( /\[(.*?)\]/ ) {
		    $so_members{$1} = 1;
		}
	    }
	}
	close(SIZE);
    }
    
    open( NM,  "$cmd") || mwmklib_error("command failed: $cmd");
    open( OUTPUT, ">> $outfile" );

    while ( defined NM and $line = <NM> ) {
	chomp($line);

	# get rid of  unimportant lines
	next if ( $line =~ m/^\s*$/o );
#	next if $line =~ m/ [a-z] /o;
	if ($line =~ m/:\s*$/o) {
	    $o_file = $1 if ($line =~ m/[^\[]*\[(.*)\]:$/);
	    next;
	}

	@words = split (/\s+/, $line);
	if ( $Platform eq "osf1v4" ) {
	    $Platform_Funcs->fix_nm_columns(\@words);
	}
	@ordered_words = @words[@NM_Params_Order];

	$o_file = $1 if ($ordered_words[2] =~ m/\[(.*)\]/o);
	if ($Platform eq "irix6" && ($ordered_words[1] =~ /(.*):(.*)/)) {
	    $o_file = $1;
	    $ordered_words[1] = $2;
	}
	$Is_Stripped_Lib = $False;
		
	if ( $prev_o_file ne $o_file ) {
	    foreach $sym (keys %o_syms) {
		print OUTPUT "$o_syms{$sym} $sym\n";
	    }
	    %o_syms = ();
	    if( $so_members{$o_file} ) {
		print OUTPUT "S $o_file\n";
	    }
	    else {
		print OUTPUT "H $o_file\n";
	    }
	    $prev_o_file = $o_file;
	}

	$ty = substr($ordered_words[0], 0, 1);
	$info = $ordered_words[1];
	if ( $Platform eq "ux11" && $info =~ m/:/ ) { 
	    $info = $'; 
	    # if it's empty continue
	    next if $info =~ /^\s*$/;
	}
	if ($ty =~ m/$NM_Params_Regexp_D/o ) {
	    next if $Platform eq "aix4" &&  substr($info, 0, 1) eq ".";
	    $o_syms{$info} = "D"; 

	}
	elsif ($ty =~ m/$NM_Params_Regexp_U/o ) {
	    if ( $Platform eq "aix4" && substr($info, 0, 1) eq "." ) {
                $info = substr($info, 1);
	    }
	   $o_syms{$info} = "U"; 
	}
	elsif ($ty =~ m/$NM_Params_Regexp_C/o ) {
	    if ( $Platform eq "aix4" && substr($info, 0, 1) eq "." ) {
                $info = substr($info, 1);
	    }
	   $o_syms{$info} = "C"; 
	}
    
    }

    close NM;
    if ( $? ) { mwmklib_error("command failed: $cmd"); }
    foreach $sym (keys %o_syms) {
	print OUTPUT "$o_syms{$sym} $sym\n";
    }

    close OUTPUT;
}   

#package main;
#print "main: @ARGV\n";
#mwmklib::main(\@ARGV);
