do "$ENV{'MWHOME'}/lib/common-utilities.pl";
do "$ENV{'MWHOME'}/lib/la/mwla.pl";
do "$ENV{'MWHOME'}/lib/la/dip.pl";
do "$ENV{'MWHOME'}/XDE/lib/errors.pl";

package mwdip;
## Finds the module name, the obj target dir, whether an EXE is linked,
## whether a C++ application is linked.
## The last two parameters are important for AIX C linkage, since in this
## case if a symbol is not exported it is unaccessible, and we need to
## automatically export the DIP entry functions.
sub initiate_dip_params
{
    local (*list) = shift;

    # Give default values for configurable behaviour.
    $LA_Flags::Do_Not_Remove = $ENV{MWLA_NO_REMOVE};
    $LA_Flags::Do_Not_Reorder = $ENV{MWLA_NO_REORDER};
    $LA_Flags::Compliant = $ENV{MWLA_COMPLIANT};

    $Full_Trace = (lc($ENV{MWTOOL_TRACE}) eq "full");
    $Partial_Trace = (defined $ENV{MWTOOL_TRACE} 
		      && ($ENV{MWTOOL_TRACE} ne "FALSE"));
    $objdir = ".";
    $ismain = 0;
    $no_init_C = 0;
    $iscxx = 0;
    $genonly = 0;
    $noentry = 0; # this switch is ignored in case we are building a .so 
    $funcname = "DllMain"; # used for locating DllMain entry point of a dll
    $raw_funcname = $default_raw_funcname = "_pRawDllMain";
    $la_enabled = 1;
    $isplugin =0;
    $local_mapfile = "";
    $export_mapfile = "";
    $nt_def_file = "";
    $map_support_type = "";
    $raw_entry = ""; # use only for comparing the RawDllMain found by la to the one the user requested - in order to display a warning message in case of mismatch.
    $insert_link_path = 0;
    $define_raw_ptr = 0;
    $raw_func = "";
    $la_succ = 0;
    $use_libmap = 0; # if true, instructs la to map libnames through libmap
    $vmw = 0;
    $implib = ""; # the name of the .lib name to be created
    $noregenlib = 0;

    #global var to keep track of current position in the list of args
    $i =0;
WHILE_DIP_PARAM: for($_ = $list[$i]; $i <= $#list; $_ = $list[++$i]){
        $_ = $list[$i];
        SWITCH: {
		# la_minimal is translated to mwla-reorder-libs=0 for 
		# backwards compatibility
	    if(/^-la_minimal$/)     { $LA_Flags::Do_Not_Remove = 1; last SWITCH; }
	    if(/^-use_libmap$/)     { $use_libmap = 1; last SWITCH; }
	    if(/^-implib:/)         { $implib = $_; last SWITCH; }
            if(/^-vmw$/)            { $vmw = 1; }
            if(/^-outdir$/)         { $objdir = $list[++$i]; last SWITCH; }
            if(/^-ismain$/)         { $ismain = 1; last SWITCH;}
	    if(/^-no-init$/)        { $no_init_C = 1; last SWITCH; }
            if(/^-c\+\+$/)          { $iscxx = 1; last SWITCH; }
            if(/^-genonly$/)        { $genonly = 1; last SWITCH; }
            if(/^-noquiet$/)        { $Partial_Trace = 1; last SWITCH; }
            if(/^-v$/)        	    { $Partial_Trace = 1; last SWITCH; }
            if(/^-noentry$/)        { $noentry = 1;last SWITCH; }
            if(/^-plugin$/)         { $isplugin = 1; last SWITCH; }
            if(/^-funcname$/)       { $funcname = $list[++$i]; last SWITCH; }
	    if(/^-raw_funcname$/)   { $raw_funcname = $list[++$i]; last SWITCH; }
            if(/^-raw_entry$/)      { $raw_entry = $list[++$i]; last SWITCH; }
            if(/^-cflags$/)         { extract_cflags(*list); last SWITCH; }
	    if(/^-mkmapflags$/)     { extract_mkmapflags(*list); last SWITCH; }
            if(/^-pushlibpath$/)    { $insert_link_path = 1; last SWITCH; }
            if(/^-nolibpath$/)      { $insert_link_path = 0; last SWITCH; }
	    if(/^-local_map/)       { $local_mapfile = $list[++$i]; last SWITCH; }
	    if(/^-export_map/)      { $export_mapfile = $list[++$i]; last SWITCH; }
	    if(/^-nt_def_file/)     { $nt_def_file = $list[++$i]; last SWITCH; }
	    if(/^-mwla-remove-libs=(.*)/) {
		$LA_Flags::Do_Not_Remove = ! $1;
		last SWITCH;
	    }
	    if(/^-mwla-reorder-libs=(.*)/) {
		$LA_Flags::Do_Not_Reorder = ! $1;
		last SWITCH;
	    }
	    if (/^-compliant/)      { $LA_Flags::Compliant = 1; last SWITCH; }
	    if(/^-map_support:(.*)/){ $map_support_type = $1;
                                      if( ! ($map_support_type =~ m/(strong|weak|cplusplus)/) ) 
                                       {
                                          print "Bad parameter for -map_suport flag: $map_suport_type\n";
                                       }
                                      else
                                       {
                                          $map_support_type = $1;
                                       }
                                      last SWITCH;
                                    } 
	    if(/^-noregenlib/)      { $noregenlib = 1; }
	    if(/^-enddip$/)         { last WHILE_DIP_PARAM;}
        }
    }
    $i++;
    $insert_link_path = 1; # enforce pushing lib path
    if ($vmw) { $use_libmap = 1; }   
    convert_obsolete_libnames(*list);
    extract_object_files(*list);
}

sub extract_object_files {
    local(*list) = shift;
    my $seen_object_file =0;
    my $handled_minus_o = 0;
    my @list_tail = ();
    for($_ = $list[$i]; $i <= $#list; $_ = $list[++$i]) {
	if (/^-defaultlib|^-nodefaultlib/ ) {
	    push @list_tail, $_;
	    next;
	}
	if (/^-o/ && ! $handled_minus_o )  { 
	    $handled_minus_o = 1;
	    push @list, "-o";
	    undef $list[$i]; 
	    my $str = $';
	    if ( $str =~ /^\s*$/ ) {
		$str = $list[++$i];
		push @list, $str;
		undef $list[$i];
	    }
	    else { push @list, $' }

	    # Get the name without the last slash
	    $str =~ /.*\/(.*)$/;
	    if ($1 ne "") {
		$str = $1;
	    }
	    # Calculate the module name. If the output file is lib*.s[ol] then 
	    # remove the lib, .s[ol]
	    if ($str !~ /^lib(.*)\.s[ol]$/) {
		$module_name = $str;
	    } else {
		$module_name = $1;
	    }
	    next;
	}
	
	if (/^-l/)  { $seen_object_file = 1 }
	if(/\.o$/ ){
		push(@object_files,$_);
		$seen_object_file = 1;
	}
	elsif($seen_object_file) { push(@after_object_params, $_) }
	elsif(/^-L/)             { push(@after_object_params, $_) }
        else                     { push(@before_object_params, $_) }
    }
    push @after_object_params, @list_tail;
}

sub convert_obsolete_libnames {
    local(*list) = shift;
    my %libs;
    $libs{"-lmw32"} = ["-lmsvcrt", "-lgdiuser32", "-lkernel32"];
    $libs{"-lntrtl"} = $libs{"-lcoolmisc"} = [];
    
    my @new_list;
    foreach $i (@list) {
	if (defined ($libs{$i})) {
	    push (@new_list, @{$libs{$i}});
	    &::print_warn("mwdip", "use of $i is anachronistic!");
	}
	else {
	    push (@new_list, $i);
	}
    }
    @list = @new_list;
}

sub extract_mkmapflags {
    local(*list) = shift;
    
    for($_ = $list[$i]; $i <= $#list; $_ = $list[++$i]){
	undef $list[$i];
        last if(/^-endmkmapflags$/);
        push(@mkmapflags,$_) unless /^-mkmapflags$/o;
    }
}

sub extract_cflags {
    local(*list) = shift;

    for($_ = $list[$i]; $i <= $#list; $_ = $list[++$i]){
	undef $list[$i];
        last if(/^-endcflags$/);
        push(@cflags,$_) unless /^-cflags$/o;
    }

    if ($::MWOS eq "osf1v4"){
	push(@cflags, "-taso -xtaso_short -DDEC_TASO_SHORT");
    }
}


sub generate_dip_src
{
    my @link_line = @_;
	my ($dir);
	local (@initial_lib_path);
	foreach $dir (@::ld_lib_path)
		{ push (@initial_lib_path, "-L$dir") if ($dir) }
	my @params = ("-suffix" ,$::lib_suffix, 
		      "-modulename", $module_name, 
		      "-outdir", $objdir);
    push(@params, "-ismain") if $ismain;
    push(@params, @link_line);
	$dip_obj = new dip();

	if ( !  $la_enabled ) {
	    my @empty_lib_list = ();
	    $dip_obj->gen(\@empty_lib_list, 
			  \@params,
			  $define_raw_ptr,
			  $raw_func);
	}
	else {
	    $dip_obj->gen($la_obj->get_so_lib_names(), 
			  \@params, 
			  $define_raw_ptr,
			  $raw_func);
	}
}

## Compiles the generated dip file. Assumes tha "-o" works for all C compilers.
sub compile_dip_src
{
    my $safe_name = &::calc_safe_name($module_name);
    my $defineRaw = "";
    $defineRaw = "-D$default_raw_funcname=$raw_funcname" if($raw_funcname ne $default_raw_funcname);

    &ExecuteCommand(@cflags, $defineRaw, "-c",
		    "$objdir/${module_name}_init.c",
		    "-o", "$objdir/${module_name}_init.o");	

	# no need to compile init.C in programs.
	if ($ismain or $no_init_C) {	return; }

	my @cmd = (@cflags, "-c", "-DDLL_NAME=${module_name}",
			   "-DSAFE_DLL_NAME=${safe_name}", "$::MWHOME/init.C");
	if ($::MWOS eq "aix4"){
		push(@cmd ,"-qunique");
		$high_priority = "-qpriority=-214748300";
		$low_priority = "-qpriority=214748300";
	}

    if( not defined $MWCC ) { $MWCC = $ENV{MWCC}; }
    if( not defined $MWCCPP ) { $MWCCPP = $ENV{MWCCPP}; }

	#always use C++ compiler    
	$cmd[0] =~ s/^mwcc$/mwc++/;
	$cmd[0] =~ s/^::$MWCC$/::$MWCCPP/;
    push(@cmd ,"-DMAIN") if ($ismain);
	push(@cmd ,"-DPLUGIN") if ($isplugin);
	ExecuteCommand(@cmd,$high_priority, $defineRaw,
		       "-o","$objdir/${module_name}_init2.o");
	ExecuteCommand(@cmd,$low_priority,"-DDEFINE_SYMBOLS", $defineRaw ,
		       "-o","$objdir/${module_name}_init1.o");
}

## Links the library. Adds the compiled dip file to the link arguments.
## Removes all the dip arguments from ARGV.
sub link_binary
{
    my  (@obj_params);
    my $line = shift;
    my @cmd = split(' ', $line);


    my $Where2Splice = 2;
    while (($cmd[$Where2Splice] !~ /\.o$/) &&
	   ($cmd[$Where2Splice] !~ /lib.*\.a/) &&
	   ($cmd[$Where2Splice] !~ /^-l.*/)) {
	++$Where2Splice;
    }

    splice(@cmd, $Where2Splice, 0, "$objdir/${module_name}_init.o", @link_params);
    if ( !$ismain && !$no_init_C ) {
	splice(@cmd, $Where2Splice, 0, "$objdir/${module_name}_init1.o");
	push @cmd, "$objdir/${module_name}_init2.o";
    }
    
    ExecuteCommand (@cmd);
  
    if ( $la_succ && ! $ismain &&  $la_enabled  ){
		local $^W = 0; # Turn off warning for this part (redefining methods)
	    $la_obj->raw_produce_lib();
    }
 
    
}

sub add_ld_link_path
{
    my $dir;
    foreach $dir (@::ld_link_path)
    { 
	push (@link_params, "-L$dir");
    }
}

sub link_analyze
{
    add_ld_link_path();

    my @before_object_params_tmp =  @before_object_params;
    my @object_files_tmp = @object_files; 
    my @link_params_tmp = @link_params;
    my @after_object_params_tmp = @after_object_params;

    my @internal_flags;
    push @internal_flags, "-ismain" if ($ismain);
    push (@internal_flags, "-noquiet") if $Partial_Trace;
    push (@internal_flags, "-use_libmap") if $use_libmap;
    push (@internal_flags, "-vmw") if $vmw;
    push (@internal_flags, "-local_map", $local_mapfile) if $local_mapfile;
    push (@internal_flags, "-export_map", $export_mapfile) if $export_mapfile;
    push (@internal_flags, "-nt_def_file", $nt_def_file) if $nt_def_file;
    push (@internal_flags, "-outdir", $objdir) if $objdir;
    push (@internal_flags, "-noregenlib") if $noregenlib;
    push (@internal_flags, "-plugin_dll") if $isplugin;
    push (@internal_flags, "-map_support:".$map_support_type) if( $map_support_type ne "");

    if ( not $ismain ) {
        push (@internal_flags, $implib) if $implib;
        # add an unresolved symbol in the kernel, to make sure that it is linked in.
        push (@internal_flags, "-unresolve", "MwKernel32Init");
    }

    $la_obj = new linker_analyzer(@internal_flags);
    my $ret;
    $ret =  $la_obj->analyze(\@list,
			     \@link_params, # -L flags
			     \@mkmapflags,
			     );
    if ( not $ret ) {
	@before_object_params =  @before_object_params_tmp;
	@object_files = @object_files_tmp; 
	@link_params = @link_params_tmp;
	@after_object_params = @after_object_params_tmp;
    }
    else {
	my @tmp_list;

	if ( ! $ismain ) { 
	    my $la_noentry = $la_obj->sym_defined($funcname) ? 0 : 1;
	    if (! $vmw && !$la_noentry &&  $noentry  ) {
		&::print_warn("mwdip", "-noentry flag inconsistency.I'm using entry point: $funcname");
	    }
	    if ( $la_noentry && ! $noentry ) {
		&::print_warn("mwdip", "DLL entry point: $funcname is not defined");
	    }
	    $noentry = $la_noentry;
	    if (  $la_obj->sym_common($raw_funcname) || 
		! $la_obj->sym_defined($raw_funcname) ){ 
		$define_raw_ptr = 1;
		my $la_raw_entry = $la_obj->get_raw_entry();
		
		if ( ( $la_raw_entry ne "" ) &&
		    ( $la_obj->sym_defined($la_raw_entry) ) ){
		    $raw_func = $la_raw_entry;
		}
	    }

	    # warn the user if there is a mismatch between -raw_entry flag and the entry found by la. In any case the la entry is taken.
	    if ($define_raw_ptr && 
		$raw_func ne "" &&
		$raw_entry ne "" && 
		$raw_entry ne $raw_func ) {
		&::print_warn("mwdip", "Not using -raw_entry=$raw_entry flag, but using:-raw_entry=$raw_func. In order to use this entry define in you code _pRawDllMain = $raw_entry");
		}
	    }
  
	while ( $token = shift @list ) {
	    next if $token eq "-noentry"; 
	    if ( $token eq "-raw_entry" ) { shift @list; next; }
	    if ( $token eq "-enddip") {
		if ( $noentry )   { push @tmp_list, "-noentry"; }
		push @tmp_list, "-enddip";
		last;
	    }
	    push @tmp_list, $token;
	}
	
	@list = (@tmp_list, 
		 @before_object_params,
		 @object_files,
		 @link_params,
		 @after_object_params,
		 @mislibs);
    }
    return $ret;
}

sub ExecuteCommand {
    if ($Partial_Trace) {
      main::ShowAndExecuteCommand(@_);
    } else {
      main::SilentExecuteCommand (@_);
    }
}


sub TestPlatform {
  local $MWROOT = $ENV{'MWROOT'};
  local $MWOS = $ENV{'MWOS'};
  local $MWCCPP = $ENV{'MWCCPP'};
  local $FNAMEC = "/tmp/test_os$$.C";
  local $FNAMEO = "/tmp/test_os$$.o";
  
  if ((!$MWROOT) || $ENV{'MWNO_OSCHECK'}) { return 1; }

  open(TMPC,">${FNAMEC}");
  print TMPC "#include \"ver/mwcheckos.h\"\n";
  print TMPC "void f() {}\n";
  close TMPC;

  ExecuteCommand("${MWCCPP} -c -I${MWROOT}/build/userx/public/mainwin/inc -D${MWOS} -o ${FNAMEO} ${FNAMEC}");
  unlink($FNAMEC);
  unlink($FNAMEO);
}

sub main {
    local (*list) = shift;
    initiate_dip_params (*list);
    TestPlatform();

    remove_undefs_in_array(\@list);
    # Get rid of all the dip args
    my @dipparams = ();
    if( grep(/^-enddip$/, @list) ) {
	while( $list[0] ne "-enddip") {
	    my $flag = shift @list;
	    push(@dipparams, $flag) unless $flag eq '-noentry';
	    # -noentry is removed but returned later if needed
	}
	push(@dipparams, shift @list); # -enddip
    }

    my $link_line = link_analyze();
    $la_succ = ! ! $link_line; # convert to boolean
    
    # if noentry is set add it as last element (before -enddip).
    splice(@dipparams,  $#dipparams, 0, '-noentry') if $noentry;

    if ( ! generate_dip_src( split(' ', $link_line), @dipparams ) ) {
	exit 1;
    }
    return if($genonly); 
    compile_dip_src ();
    link_binary ($link_line);
    exit 0;
}

# return number of undefs removed.
sub remove_undefs_in_array {
    my $arr = shift;
    # initial number of elements
    my $elems = scalar @$arr;
    
    for(my $i=0; $i < @$arr; ++$i) {
        if ( not defined $$arr[$i] ) {
            # remove element
            splice(@$arr, $i, 1); 
            # current element gone, we need to access $$arr[$i] 
            # (for this $i) again on the next iteration.
            --$i; 
        }
    }
    return $elems - scalar @$arr;
}

package main;
mwdip::main(*ARGV);
1;
