# This is a -*- perl -*- script

#########################################################################
#                                                                       #
#  This program is the CONFIDENTIAL and PROPRIETARY property            #
#  of Mainsoft(R) Corporation. Any unauthorized use, reproduction       #
#  or transfer of this computer program is strictly prohibited.         #
#                                                                       #
#  Copyright (c) 1991-1998 Mainsoft Corporation.                        #
#  This is an unpublished work, and is subject to limited distribution  #
#  and restricted disclosure only. ALL RIGHTS RESERVED.                 #
#                                                                       #
#########################################################################

BEGIN { 	
    push( @INC, "$ENV{MWHOME}/lib/" );
}

require "common-utilities.pl";
do "PrintEx.pl";
use PPService;
use atlpp;



# -- file types --
@src_c_files   = ( '*.c' );
@src_cpp_files = ( '*.[Cc][Pp][Pp]', '*.[Cc][Xx][Xx]', '*.[Ii][Nn][Ll]', '*.C' );
@src_hpp_files = ( '*.[Hh]', '*.[Hh][Hh]', '*.[Rr][Hh]', '*.[Hh][Pp][Pp]', '*.[Hh][Xx][Xx]', '*.[Hh][Pp][Jj]', '*.[Tt][Ll][Hh]' );
@src_idl_files = ( '*.[Ii][Dd][Ll]', '*.[Oo][Dd][Ll]' );
@src_mc_files  = ( '*.[Mm][Cc]' );
@src_res_files = ( '*.[Dd][Ll][Gg]', '*.[Rr][Cc]', '*.[Rr][Cc]2', @src_mc_files );
@src_rsb_files = ( '*.[Ii][Cc][Oo]', '*.[Bb][Mm][Pp]' );  # binary resources
@src_mak_files = ( '*.[Dd][Ee][Ff]', '*.[Mm][Aa][Kk]' );
@src_rsd_files = ( @src_res_files, @src_hpp_files );  # 'res' directory - specific 
@src_all_files = ( @src_cpp_files, @src_hpp_files, @src_idl_files, @src_res_files, @src_c_files );

$pps;
$orig_dir = "ORIG";
$bSpecificFiles = 0;
$bIfdefChanges = 1;

# -- flags --
$bChangeCppCmt;
$bKeepOrig;
$bGenLink;
$bNoLowercase;

$bMwxdeMode = 0;
$bEnableTracing = defined( $ENV{ MWXDE_TRACE } ) ? 1 : 0;


# Signal handlers
$SIG{ INT } = \&IntHandler;



main();


sub main {
    ProcessArgs();
    CheckPermissions();

    $pps = new PPService( $bMwxdeMode, $bEnableTracing );

    SaveOrig() if $bKeepOrig;
    if( @src_all_files || @src_mak_files ) {
 	$pps->PrintTitle( scalar( @src_all_files + @src_mak_files ), "Preparing line terminators for UNIX" );
	Execute( 
	    "tounix.pl",
	    "ToUnix",
	    "Processing file %s", 
	    < @src_all_files @src_mak_files @src_mc_files >
	);
    }

    if( @src_all_files ) {
	$pps->PrintTitle( scalar( @src_all_files ), "Preparing #include directives for UNIX" );
	Execute( 
	    "changeinclude.pl",
	    "ChangeInclude",
	    "Processing file %s",
	    $bNoLowercase ? "-nolowercase" : undef,
	    < @src_all_files >
	);
    }

    if( @src_all_files && $bAtl ) {
	my $atlpp = new atlpp( \$pps, "$ENV{MWHOME}/lib/atldtp.dat", $bIfdefChanges );
	$atlpp->Run( "@src_all_files" );
    }
    
    if( $bGenLink && @src_cpp_files ) {
 	$pps->PrintTitle( scalar( @src_cpp_files ), "Creating soft links for cpp/cxx files to C files" );
	CreateSoftLink( "cxx", "C" );
	CreateSoftLink( "cpp", "C" );
    }
    
    if( @src_cpp_files ) {
 	$pps->PrintTitle( scalar( @src_cpp_files ), "Preparing MFC message map for MainWin" );
	Execute( 
	    "changemessagemap.pl", 
	    "ChangeMessageMap",
	    "Processing file %s",
	    < @src_cpp_files >
	);
    }
    
    if( @src_cpp_files || @src_hpp_files ) {
 	$pps->PrintTitle( 
	     scalar( @src_cpp_files + @src_hpp_files ), 
	     "Preparing nonstandard C++ constructors for UNIX" 
	);
	Execute( 
	    "fixc++.pl",
	    "FixNonStdCtors",
	    "Processing file %s",
	    < @src_cpp_files @src_hpp_files >
	);
    }

#    if( @src_cpp_files || @src_c_files || @src_hpp_files ) {
# 	$pps->PrintTitle( 
#	     scalar( @src_cpp_files + @src_c_files + @src_hpp_files ), 
#	     "Preparing except and finally blocks for UNIX" 
#	);
#	Execute( 
#	    "except.pl",
#	    "FixExcept",
#	    "Processing file %s",
#	    < @src_cpp_files @src_hpp_files @src_c_files >
#	);
#    }

    if( @src_idl_files ) {
 	$pps->PrintTitle( scalar( @src_idl_files ), "Processing MIDL files" );
	ProcessMidlFiles();
    }

    FnamesToLowercase( < @src_all_files @src_rsb_files > ) unless $bNoLowercase;

    ProcessResDirectory();
    
    PrintEx( "\n\n-- PREPROCESSING COMPLETED --\n", $WM_BOLD ) unless $bMwxdeMode;

    WarnAboutBackslashes();

    # Force a call to the PPService destructor to properly terminate the connection with MWXDE.
    undef $pps;

    print "\n\n" unless $bMwxdeMode;
}


sub CheckPermissions {
    Abort( "Current directory does not appear to be writable" ) unless( -w "." );
}


sub ProcessArgs {
    local( @files, $opt );
    $bChangeCppCmt = 0;
    $bKeepOrig = 0;
    $bGenLink = 0;
    $bNoLowercase = 0;
    
    while( $opt = shift (@ARGV) ) {
	if( $opt eq "-nolowercase" || $opt eq "-n" ) {
	    $bNoLowercase = 1;
	} elsif( $opt eq "-atl" || $opt eq "-a" ) {
	    $bAtl = 1;
	} elsif( $opt eq "-noifdef" || $opt eq "-i" ) {
	    $bIfdefChanges = 0;
	} elsif( $opt eq "-genlink" || $opt eq "-l" ) {
	    $bGenLink = 1;
	} elsif($opt eq "-changecppcmt" || $opt eq "-c" ) {
	    $bChangeCppCmt = 1;
	} elsif($opt eq "-keeporig" || $opt eq "-k" ) {
	    $bKeepOrig = 1;
	    if( $opt = shift( @ARGV ) ) {
		if( $opt !~ /^-/ ) {
		    $orig_dir = $opt;
		}
		else {
		    unshift( @ARGV, $opt );
		}
	    }
	} elsif( $opt eq "-help" || $opt eq "-h" ) {
	    if( shift( @ARGV ) eq "ptf" ) {
		DisplayUsage( 0 );
	    }
	    else {
		DisplayUsage();
	    }
	} elsif( $opt !~ /^-/ ) {
	    push( @files, $opt );
	} elsif( $opt eq "-mwxde" ) {
	    $bMwxdeMode = 1;
	} elsif( $opt eq "-t" ) {
	    $bEnableTracing = 1;
	} else {
	    print STDERR "\a\n\nInvalid option:";
	    PrintEx( "'$opt'\n\n", $WM_BOLD, $OS_STDERR );
	    DisplayUsage();
	}
    }
    if( !@files ) {
	@files = glob( join(" ",@src_all_files)." ".join(" ",@src_mak_files) );
    }
    else {
	$bSpecificFiles = 1;
    }
    while ($ff = shift (@files))
	{ $files {$ff} = 1; }
    FilterList( *src_c_files, *files );
    FilterList( *src_cpp_files, *files );
    FilterList( *src_hpp_files, *files );
    FilterList( *src_idl_files, *files );
    FilterList( *src_mc_files, *files );
    FilterList( *src_res_files, *files );
    FilterList( *src_mak_files, *files );
    FilterList( *src_all_files, *files );
}

sub FilterList {
    local( *old_list, *FilterList ) = @_;
    my @list = glob(join(" ",@old_list));
    my @list2;
    my %list2;
    while ($f = shift (@list)) {
	if ($FilterList{$f}) {
	    if (!defined ($list2 {$f}))
		{ push (@list2, $f); $list2 {$f} = 1; }
	}
    }
    @old_list = @list2;
}

sub ProcessResDirectory {
    my $file;
    my $orig_dir = "../$orig_dir/res";
    my @rsd_files;

    # user-specified files must be processed explicitly in each directory
    if( -d "res" && !$bSpecificFiles ) {
        chdir "res";

        @rsd_files = < @src_rsd_files >;
	system( "chmod -x @src_rsb_files 2> /dev/null" );

	if( @rsd_files ) {
	    $pps->PrintTitle( @rsd_files * ( 2 + $bKeepOrig ), "Processing resource directory" );
	    if( $bKeepOrig ) {
		if( ! -d "$orig_dir" ) {
		    system( "mkdir $orig_dir" ) && Abort( "Could not create directory $orig_dir" );
		}
		foreach $file( @rsd_files )  {
		    $pps->PrintItem( "Saving file $file in directory $orig_dir" );
		    system( "cp $file $orig_dir 2> /dev/null" ) && Abort( "Unable to copy $file to $orig_dir" );
		}
	    }
	    Execute( 
		    "tounix.pl",
		    "ToUnix",
		    "Preparing line terminators for UNIX in file %s",
		    @rsd_files
	    ); 
	    Execute( 
		    "changeinclude.pl", 
		    "ChangeInclude",
		    "Preparing #include directives for UNIX in file %s",
		    $bNoLowercase ? "-nolowercase" : undef,
		    @rsd_files
	    );
	}
	FnamesToLowercase( @rsd_files, < @src_rsb_files > ) unless $bNoLowercase;
	chdir "..";
    }
}


sub ProcessMidlFiles {
    my $file;

    foreach $file( @src_idl_files ) {
	$pps->PrintItem( "Updating include directives in file $file" );
	open( FROM_FILE, "<$file" );
	open( TO_FILE, ">$file.out" );
	while( <FROM_FILE> ) {
	    if( /^\s*import.*\s*\(*\s*\"\s*.+\s*\"\s*\)*\s*\;/ ) {
		$_ = "\L$_" unless $bNoLowercase;
	    }
	    print TO_FILE $_;
	}
	close( FROM_FILE );
	close( TO_FILE );

	if( ! -z "$file.out" ) {
	    if( 0 == system( "diff $file $file.out > /dev/null" ) ) {
		unlink( "$file.out" ) || Abort( "Couldn't delete file $file.out" );
	    }
	    else {
		rename( "$file.out", "$file" );
	    }
	}
	else {
            $pps->PrintError( "Conversion failed: couldn't create file of size " . ( -s $file ) . ".\nPlease check your disk space.\n" );
            unlink( "$file.out" ) || Abort( "Couldn't delete file $file.out" );
        }
    }
}


sub CreateSoftLink {
    my( $from_suffix, $to_suffix ) = @_;
    my( $from, $to );

    foreach $from( <*.$from_suffix> ) {
	$to = $from;
	$to =~ s/(.*)\.$from_suffix/$1.$to_suffix/;
	$pps->PrintItem( "Generating soft link: $to -> $from" );
	system( "ln -s $from $to" ) unless ( -f $to );
    }
}


sub SaveOrig {
    my $file;
    my @mak_files = < @src_mak_files *makefile >;

    if( !scalar( @src_all_files + @mak_files ) ) {
	return;
    }

    if( ! -d $orig_dir ) {
	system( "mkdir $orig_dir" ) && Abort( "Could not create directory ./$orig_dir/" );
    }
    $pps->PrintTitle( scalar( @src_all_files + @mak_files ), "Saving original files in directory $orig_dir" );
    foreach $file( < @src_all_files @mak_files > )  {
	TRACE( "LOCAL OP: Saving file $file" );
	$pps->PrintItem( "Saving file $file" );
	system( "cp $file $orig_dir 2> /dev/null" );
    }
}


sub FnamesToLowercase {
    my( $lcfile, $file );

    if( @_ ) {
	$pps->PrintTitle( scalar( @_ ), "Fixing filenames case" );
	foreach $file( @_ ) {
	    if( ! -d $file ) {
		$lcfile = $file;
		$lcfile =~ y/[A-Z]/[a-z]/;
                if( $lcfile eq $file )
                {
                    next ;
                }
		$pps->PrintItem( "Renaming $file to $lcfile" );
		rename( "$file", "$lcfile" ) || Abort( "Can't rename $file to $lcfile" );
	    }
	}
    }
}


sub ChangeCppComments {
    local( $file );

    foreach $file( < *.c *.h *.cpp *.hpp *.d *.dlg *.rc > ) {
	if( ( -f $file ) && ( ! -l $file ) ) {
	    system ("${MWLIB}-${MWCONFIG_NAME}/Slash $file");
	}
    }
}


sub WarnAboutBackslashes {
    my @files = < @src_c_files @src_cpp_files @src_hpp_files >;
    @files = `grep -l '\\\\\\\\' @files 2> /dev/null` if( @files );
    if( @files ) {
	print STDERR "\n\n\n";
	PrintEx( "\aWARNING:\n", $WM_BOLD, $OS_STDERR );
	print STDERR "The backslash (\\) characters can be filename delimiters\n";
	print STDERR "for Windows NT. They should be replaced by the macro\n";
	print STDERR "FILENAME_SEPARATOR, defined in the 'mainwin.h' header file.\n";
	print STDERR "The above characters were found in the following files:\n";
        foreach $file( @files ) {
	    print STDERR "   $file";
	}
    }
}


# -----------------------------------------------------------------
# ------------------------- UNDOCUMENTED --------------------------
# Passing 'ptf' after -help makes mwprepro print its help message
# in plain text format.
# -----------------------------------------------------------------

sub DisplayUsage {
    my( $wm_ext ) = @_ ? ( $_[0] ? 1 : 0 ) : 1;

    PrintEx( "\nSynopsis\n\n", ( $WM_BOLD | $WM_UNDERLINE ) * $wm_ext );
    print "   mwprepro [-changecppcmt] [-nolowercase] [-atl] { [-keeporig] [";
     PrintEx( "dir", $WM_UNDERLINE * $wm_ext );
     print "] } [-genlink] [-nocond] [-help]  [";
     PrintEx( "file", $WM_UNDERLINE * $wm_ext );
     print "...]\n\n\n";

    PrintEx( "Options and operands\n\n", ( $WM_BOLD | $WM_UNDERLINE ) * $wm_ext );
    print "   -"; 
     PrintEx( "n", $WM_BOLD * $wm_ext ); 
     print "olowercase    Do not change filenames and include directives to lowercase.\n";
   print "   -"; 
     PrintEx( "a", $WM_BOLD * $wm_ext ); 
     print "tl            Perform ATL-specific preprocessing.\n";
    print "   -"; 
     PrintEx( "k", $WM_BOLD * $wm_ext ); 
     print "eeporig       Make a copy of the original source files in subdirectory ";
     PrintEx( "dir", $WM_UNDERLINE * $wm_ext );
     print "\n                   before preprocessing.\n";
    print "   -gen"; 
     PrintEx( "l", $WM_BOLD * $wm_ext ); 
     print "ink        Generate a C symbolic link file to the cpp/cxx file.\n";
    print "   -"; 
     PrintEx( "c", $WM_BOLD * $wm_ext ); 
     print "hangecppcmt   Replace single-line comments with ANSI C comments.\n";
    print "   -no"; 
     PrintEx( "i", $WM_BOLD * $wm_ext ); 
     print "fdef        Apply changes made by mwprepro unconditionally (normally such\n";
     print "                   changes would be #ifdef'ed). Note: currently this option can\n";
     print "                   only be used for ATL preprocessing (";
     PrintEx( "-atl", $WM_BOLD * $wm_ext );
     print ").\n";
    print "   -"; 
     PrintEx( "h", $WM_BOLD * $wm_ext ); 
     print "elp           Display this message.\n\n";
    print "    ";
     PrintEx( "file", $WM_UNDERLINE * $wm_ext );
     print "           Specifies source file(s) to be preprocessed. If omitted,\n";
     print "                   all relevant files in current directory will be preprocessed.\n\n";
    print "    ";
     PrintEx( "dir", $WM_UNDERLINE * $wm_ext );
     print "            Specifies the directory to which to copy the original files.\n";
     print "                   This operand can be used only with the '-keeporig' option.\n\n\n"; 

    PrintEx( "Description\n\n", ( $WM_BOLD | $WM_UNDERLINE ) * $wm_ext );
    print "   mwprepro processes source text ";
    PrintEx( "file", $WM_UNDERLINE * $wm_ext );
     print "(s) to make them compatible with\n";
    print "   the UNIX programming environment. Your source code will still compile\n";
    print "   correctly in Windows NT. \n\n";
    exit(0);
}


sub Abort {
    $pps->PrintError( shift ) if( @_ );
    exit( -1 );
}


sub TRACE {
    print STDERR "\e[1mTRACE[ prepro.pl ]:\e[0m $_[0]\r\n" if( $bEnableTracing );
}


# Executes a function from a given perl script file.
# AUTOLOAD gets confused when using same module more than once
sub Execute {
    my $ScriptName = shift;
    my $MainProc = shift;
    my $message = shift;
    my $MessageFmt;
    my @operands;
    my @flags;

    # Separate flags from operands
    foreach( @_ ) {
	if( /^-/ ) {
	    push( @flags, $_ );
	}
	else {
	    push( @operands, $_ );
	}
    }

    foreach $op( @operands ) {
	next if !defined( $op );
	$MessageFmt = sprintf( "$message", $op );
	$pps->PrintItem( $MessageFmt );
	require "$MWLIB/$ScriptName";
	TRACE( "Calling MainProc = $MainProc( @flags, $op ) at $ScriptName" );
	&$MainProc( @flags, $op );
    }
}


sub IntHandler {
    print "\e[M\e[D\n<CTRL-C> signaled. Exiting.\n\n" unless $bMwxdeMode;
    exit;
}
